<?php
/*****************************************************************
* UCStats - A stats generation program for Half-Life games.
* Copyright (c) 2003 Mike Cao <mike@mikecao.com>
*****************************************************************/

/*
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/*****************************************************************
* Document class
*****************************************************************/

Class Document {

	var $document = array();
	var $template = array();
	var $data = array();
	
	// Constructor
	function Document() {
		$this->document = array();
		$this->data = array();
	}
	
	// Creates the document from the set template
	function createDocument() {
		if (sizeof($this->template)) {
			$this->parseTemplate($this->template);
			return 1;
		}
		else {
			$this->document[] .= L_ERR_TEMPLATE.'<br>';
			return 0;
		}
	}
	
	// Parses a template
	function parseTemplate($template) {
		$parse_state = true;
		$parse_key = "";
		if (sizeof($template)) {
			// Go through template line by line
			for ($i = 0; $i < sizeof($template); $i++) {
				// Conditional statement found
				if ($parse_state && preg_match("/<!--BEGIN EXISTS data=\"(.*)\"-->/",$template[$i],$args)) {
					list(,$key) = $args;
					
					// Clear tag
					$template[$i] = '';

					// Check if data exists or has records in document
					if (is_null($this->getDocumentData($key)) || !sizeof($this->getDocumentData($key))) {
						$parse_state = false;
						$parse_key = "<!--END EXISTS data=\"$key\"-->";
					}
				}
				// Loop found
				if ($parse_state && preg_match("/<!--BEGIN LOOP data=\"(.*)\"-->/",$template[$i],$args)) {
					list(,$key) = $args;
					
					// Clear tag
					$template[$i] = '';

					// Start one line after
					$loop_start = $i+1;
					
					// Get specified data set
					$dataset = $this->getDocumentData($key);
					
					// Loop start
					$j = 0;
					
					// Data exists in document
					if (!is_null($dataset) && sizeof($dataset)) {
						// Parse individual records in data set
						foreach ($dataset as $datakey => $dataval) {
							foreach ($this->data[$key][$datakey] as $k => $v) {
								$this->data[$k] = $v;
							}
							// Parse records until end found
							while (!preg_match("/<!--END LOOP-->/",$template[$i])) {
								$this->document[] = $this->parseCode($template[$i]);
								if ($i++ > sizeof($template)) break;
							}

							// Restart loop
							if ($j++ < sizeof($dataset)-1) $i = $loop_start;
						}
					}
					// Turn off parsing
					else {
						$parse_state = false;
						$parse_key = "<!--END LOOP-->";
					}
				}
				// If allowed to parse
				if ($parse_state) {
					if (!preg_match("/<!--END/",$template[$i])) {
						$this->document[] = $this->parseCode($template[$i]);
					}
				}
				// Look for key end statement
				else {
					if (preg_match("/$parse_key/",$template[$i])) {
						$parse_state = true;
					}
				}
			}
		}
	}
	
	// Sets the document template
	function setTemplate($file) {
		if (file_exists($file)) {
			$this->template = file($file);
			return 1;
		}
		else {
			$this->document[] .= get_token_string(L_ERR_FILE_NOT_FOUND, array($file));
			return 0;
		}
	}
	
	// Adds to the document from a file
	function addFile($file) {
		if (file_exists($file)) {
			$fp = fopen($file, "r");
			$this->document[] = $this->parseCode(fread($fp, filesize($file)));
			fclose($fp);
			return 1;
		}
		$this->document[] .= get_token_string(L_ERR_FILE_NOT_FOUND, array($file));
		return 0;
	}
	
	// Adds to the document from a string
	function addString($str) {
		$this->document[] = $this->parseCode($str);
	}
	
	// Prints this document to the browser
	function publishToBrowser() {
		echo implode("",$this->document);
	}
	
	// Writes this document to a file
	function publishToFile($file) {
		$fp = fopen($file, "w+");
		flock($fp,2);
		fwrite($fp, implode("",$this->document));
		fclose($fp);
		chmod($file,0660);
	}
	
	// Returns document as a string
	function publishToString() {
		return implode("",$this->document);
	}
	
	// Clears the document
	function clearDocument() {
		$this->document = array();
	}
	
	// Clears the document template
	function clearTemplate() {
		$this->template = array();
	}
	
	// Sets data for document
	function setDocumentData($key,$data) {
		$this->data[$key] = $data;
	}
	
	// Gets data from document
	function getDocumentData($key) {
		if (isset($this->data[$key])) {
			return $this->data[$key];
		}
	}
	
	// Destroys document data
	function unsetDocumentData($key) {
		unset($this->data[$key]);
	}
	
	// Parse PHP code from string
	function parseCode($str) {
		preg_match_all("/{(.*?)}/",$str,$args);
		foreach ($args[1] as $code) {
			// Document data
			if (strlen($code) && $code{0}=='$') {
				$str = str_replace('{'."$code".'}',$this->getDocumentData(substr($code,1)),$str);
			}
			// Functions and global variables
			else if (substr($code,0,5) == 'exec:') {
				$code = substr($code,5);
				if (strlen($code) && $code{0} == '$') eval("global $code;");
				if (strlen($code) && $code{0} != '$') {
					if (!defined($code) && !preg_match("/^(.*)\((.*)\)$/",$code)) $code = "null";
				}
				eval("\$replace = $code;");
				$str = str_replace('{'."exec:$code".'}',$this->parseCode($replace),$str);
			}
			else if (substr($code,0,8) == 'include:') {
				$code = substr($code,8);
				eval("\$file = $code;");
				if (file_exists($file)) {
					$this->parseTemplate(file($file));
				}
				$str = str_replace('{'."include:$code".'}','',$str);
			}
		}
		return $str;
	}
}
?>