<?php
/*****************************************************************
* UCStats - A stats generation program for Half-Life games.
* Copyright (c) 2003 Mike Cao <mike@mikecao.com>
*****************************************************************/

/*
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, 
Boston, MA 02111-1307, USA.
*/

/*****************************************************************
* Parser class
*****************************************************************/

Class Parser {

	var $parser_config = array();
	var $parser_steamid = array();

	// Query object
	var $query;
	
	// Constructor
	function Parser(&$query) {
		$this->query = &$query;
	}
	
	// Sets parser config
	function setConfig($key, $value) {
		$this->parser_config[$key] = $value;
	}
	
	// Gets parser config
	function getConfig($key) {
		return $this->parser_config[$key];
	}
	
	// Gets the date the parser was last run
	function getLastDate() {
		$result = db_get_key('PARSER','LASTDATE');
		if (!is_null($result)) {
			$row = mysql_fetch_row($result);
			return $row[0];
		}
		else {
			return -1;
		}
	}
	
	// Parser
	function parse($str) {
		global $GAME;
		$args = array();
		$datetime = substr($str,0,23);
		$log = substr($str,25);
		
		// Set the log time
		if (preg_match("/^L ([0-9]{2})\/([0-9]{2})\/([0-9]{4}) - "
		."([0-9]{2}):([0-9]{2}):([0-9]{2})$/",$datetime,$args)) {
			list(,
				$month,
				$day,
				$year,
				$hour,
				$minute,
				$second
			) = $args;

			if (sizeof($args)) {
				$this->query->setTime(date("U",mktime($hour,$minute,$second,$month,$day,$year)));
			}
		}
		else {
			return 0;
		}

		// Say
		if ($this->getConfig('lf_enabled') && strpos($log,'" say "') !== false) {
			if (preg_match("/^\"(.*)<([0-9]+)><(.*)><(.*)>\" say \"(.*)\"$/",$log,$args)) {
				list(,
					$name,,
					$id,
					$team,
					$say
				) = $args;

				if ($id == '0') $id = 'BOT';	
				if ($id == 'BOT' && CFG_BOT_STATS == false) return;

				if (sizeof($args) && strlen($name) && $say{0} == '@') {
					$code = substr($say,0,strpos($say,' '));
					if ($code == '@resetme') {
						if (CFG_DEBUG||CFG_DEBUG_LOG) log_parser("RESET: \"$name\" ($id)");
						$this->query->queryPlayerReset($name,$id);
					}
					else if ($code == '@password') {
						if (CFG_DEBUG||CFG_DEBUG_LOG) log_parser("PASSWORD: \"$name\" ($id)");
						$str = explode(' ',$say);
						if (isset($str[1]) && strlen($str[1])) {
							$this->query->queryPlayerPassword($name,$id,$str[1]);
						}
					}
				}
			}
			return;
		}
		
		// Player
		if ($this->getConfig('ps_enabled')) {

			// Player kill
			if (strpos($log,'" killed "') !== false) {
				if (preg_match("/^\"(.*)<([0-9]+)><(.*)><(.*)>\" killed "
				."\"(.*)<([0-9]+)><(.*)><(.*)>\" with \"(.*)\""
				."$/",$log,$args)) {
					list(,
						$name1,,
						$id1,
						$team1,
						$name2,,
						$id2,
						$team2,
						$weapon
					) = $args;

					if ($id1 == '0') $id1 = 'BOT';
					if ($id2 == '0') $id2 = 'BOT';
					if (($id1 == 'BOT' || $id2 == 'BOT') && CFG_BOT_STATS == false) return;
					
					if (sizeof($args) && strlen($name1) && strlen($name2) && is_valid_weapon($weapon)) {
						if (CFG_DEBUG||CFG_DEBUG_LOG) log_parser("KILL: \"$name1\" ($id1) killed \"$name2\" ($id2) with $weapon");
						$this->query->queryPlayerKill($name1,$id1,$team1,$name2,$id2,$team2,$weapon);
					}
				}
				return;
			}

			// Player entered the game
			if (strpos($log,'" entered the game') !== false) {
				if (preg_match("/^\"(.*)<([0-9]+)><(.*)><>\" entered the game$/",$log,$args)) {
					list(,
						$name,,
						$id,
					) = $args;

					if ($id == '0') $id = 'BOT';	
					if ($id == 'BOT' && CFG_BOT_STATS == false) return;
		
					if (sizeof($args) && strlen($name)) {
						if (CFG_DEBUG||CFG_DEBUG_LOG) log_parser("ENTERED GAME: \"$name\" ($id)");
						$this->query->queryPlayerEntered($name,$id);
					}
				}
				return;
			}
			
			// Player connected
			if (strpos($log,'" connected') !== false) {
				if (preg_match("/^\"(.*)<([0-9]+)><(.*)><(.*)>\" connected, address "
				."\"([\d]+\.[\d]+\.[\d]+\.[\d]+):([0-9]+)\"$/",$log,$args)) {
					list(,
						$name,,
						$id,,
						$ip,
						$port
					) = $args;
	
					if ($id == '0') $id = 'BOT';	
					if ($id == 'BOT' && CFG_BOT_STATS == false) return;
		
					if (sizeof($args) && strlen($name)) {
						if ($id == 'STEAM_ID_PENDING') {
							$this->parser_steamid[md5($name)] = $ip;
						}
						else {
							if (CFG_DEBUG||CFG_DEBUG_LOG) log_parser("CONNECTED: \"$name\" ($id)");
							$this->query->queryPlayerConnect($name,$id,$ip);
						}
					}
				}
				return;
			}
			
			// Steam ID validation
			if (strpos($log,'" STEAM USERID validated') !== false) {
				if (preg_match("/^\"(.*)<([0-9]+)><(.*)><(.*)>\" STEAM USERID validated$/",$log,$args)) {
					list(,
						$name,,
						$id,
						$team
					) = $args;
				
					if ($id == '0') $id = 'BOT';	
					if ($id == 'BOT' && CFG_BOT_STATS == false) return;
					
					if (sizeof($args) && strlen($name)) {
						if (CFG_DEBUG||CFG_DEBUG_LOG) log_parser("VALIDATED: \"$name\" ($id)");
						$this->query->queryPlayerConnect($name,$id,$this->parser_steamid[md5($name)]);
					}
				}
				return;
			}
			
			// Player disconnected
			if (strpos($log,'" disconnected') !== false) {
				if (preg_match("/^\"(.*)<([0-9]+)><(.*)><(.*)>\" disconnected$/",$log,$args)) {
					list(,
						$name,,
						$id,
						$team
					) = $args;
				
					if ($id == '0') $id = 'BOT';	
					if ($id == 'BOT' && CFG_BOT_STATS == false) return;
					
					if (sizeof($args) && strlen($name)) {
						if (CFG_DEBUG||CFG_DEBUG_LOG) log_parser("DISCONNECTED: \"$name\" ($id)");
						$this->query->queryPlayerDisconnect($name,$id);
					}
				}
				return;
			}
			
			// Player suicide
			if (strpos($log,'" committed suicide with "') !== false) {
				if (preg_match("/^\"(.*)<([0-9]+)><(.*)><(.*)>\" committed suicide with \"(.*)\"$/",$log,$args)) {
					list(,
						$name,,
						$id,
						$team,
						$suicide
					) = $args;
					
					if (sizeof($args) && strlen($name)) {
						if ($GAME == 'dod') {
							if ($suicide == 'grenade2' || $suicide == 'mills_bomb') {
								$suicide == 'grenade';
							}
						}
						if (CFG_DEBUG||CFG_DEBUG_LOG) log_parser("SUICIDE: \"$name\" ($id)");
						$this->query->queryPlayerAction($name,$id,$team,$suicide.'_suicide');
					}
				}
				return;
			}
			
			// Name change
			if (strpos($log,'" changed name to "') !== false) {
				if (preg_match("/^\"(.*)<([0-9]+)><(.*)><(.*)>\" changed name to \"(.*)\"$/",$log,$args)) {
					list(,
						$name,,
						$id,
						$team1,
						$newname
					) = $args;

					if ($id == '0') $id = 'BOT';	
					if ($id == 'BOT' && CFG_BOT_STATS == false) return;
		
					if (sizeof($args) && strlen($name) && strlen($newname)) {
						if (CFG_DEBUG||CFG_DEBUG_LOG) log_parser("NAME CHANGE: \"$name\" ($id) changed name to \"$newname\"");
						$this->query->queryPlayerEntered($newname,$id);
						$this->query->queryPlayerDisconnect($name,$id);
					}
				}
				return;
			}
		} // End player
		
		// Triggers
		if (strpos($log,'triggered') !== false) {

			// Fix for dod
			if ($GAME == 'dod') {
				$log = str_replace('" triggered a "','" triggered "',$log);
			}

			// Player triggers
			if ($this->getConfig('ps_enabled') && strpos($log,'" triggered "') !== false) {
				// Player actions
				if (preg_match("/^\"(.*)<([0-9]+)><(.*)><(.*)>\" triggered \"(([\w\s]+))\"/",$log,$args)) {
					list(,
						$name,,
						$id,
						$team,
						$action
					) = $args;
					
					if ($id == '0') $id = 'BOT';
					if ($id == 'BOT' && CFG_BOT_STATS == false) return;
					
					if (sizeof($args) && strlen($name) && is_valid_action(strtolower($action))) {
						if (CFG_DEBUG||CFG_DEBUG_LOG) log_parser("TRIGGER: \"$name\" ($id) triggered \"$action\"");
						$this->query->queryPlayerAction($name,$id,$team,strtolower($action));
						return; // should only return if found
					}
				}

				// Weapon stats
				if (preg_match("/^\"(.*)<([0-9]+)><(.*)><(.*)>\" triggered \"Weapon_Stats\" "
					."\((.*) \"([0-9]+),([0-9]+),([0-9]+),([0-9]+),([0-9]+),([0-9]+),([0-9]+)\"\)$/",$log,$args)) {
					list(,
						$name,,
						$id,
						$team,
						$weapon,
						$kills,
						$deaths,
						$headshots,
						$teamkills,
						$shots,
						$hits,
						$damage
					) = $args;
					
					// Fix
					$weapon = str_replace('weapon_','',$weapon);
					if ($weapon == 'hegrenade') $weapon = 'grenade';
				
					if ($id == '0') $id = 'BOT';	
					if ($id == 'BOT' && CFG_BOT_STATS == false) return;
					
					if (sizeof($args) && strlen($name) && is_valid_weapon($weapon)) {
						if (CFG_DEBUG||CFG_DEBUG_LOG) log_parser(
							"WEAPON STATS: \"$name\" ($id) $weapon,$kills,$deaths,$headshots,$teamkills,$shots,$hits,$damage"
						);
						$this->query->queryWeaponStats($name,$id,
							array(
								'weapon_name'=>(string)$weapon,
								'weapon_kills'=>(int)$kills,
								'weapon_deaths'=>(int)$deaths,
								'weapon_headshots'=>(int)$headshots,
								'weapon_teamkills'=>(int)$teamkills,
								'weapon_shots'=>(int)$shots,
								'weapon_hits'=>(int)$hits,
								'weapon_damage'=>(int)$damage
							)
						);
					}
					return;
				}

				// Weapon hits
				if (preg_match("/^\"(.*)<([0-9]+)><(.*)><(.*)>\" triggered \"Weapon_Hits\" "
					."\((.*) \"([0-9]+),([0-9]+),([0-9]+),([0-9]+),([0-9]+),([0-9]+),([0-9]+),([0-9]+)\"\)$/",$log,$args)) {
					list(,
						$name,,
						$id,
						$team,
						$weapon,
						$generic,
						$head,
						$chest,
						$stomach,
						$leftarm,
						$rightarm,
						$leftleg,
						$rightleg
					) = $args;
	
					// Fix
					$weapon = str_replace('weapon_','',$weapon);
					if ($weapon == 'hegrenade') $weapon = 'grenade';
				
					if ($id == '0') $id = 'BOT';	
					if ($id == 'BOT' && CFG_BOT_STATS == false) return;
					
					if (sizeof($args) && strlen($name) && is_valid_weapon($weapon)) {
						if (CFG_DEBUG||CFG_DEBUG_LOG) log_parser(
							"WEAPON HITS: \"$name\" ($id) $weapon,$generic,$head,$chest,$stomach,$leftarm,$rightarm,$leftleg,$rightleg"
						);
						$this->query->queryWeaponHits($name,$id,
							array(
								'weapon_name'=>(string)$weapon,
								'hits_generic'=>(int)$generic,
								'hits_head'=>(int)$head,
								'hits_chest'=>(int)$chest,
								'hits_stomach'=>(int)$stomach,
								'hits_leftarm'=>(int)$leftarm,
								'hits_rightarm'=>(int)$rightarm,
								'hits_leftleg'=>(int)$leftleg,
								'hits_rightleg'=>(int)$rightleg
							)
						);
					}
					return;
				}

				// Player stats
				if ($this->getConfig('ps_enabled') && preg_match("/^\"(.*)<([0-9]+)><(.*)><(.*)>\" triggered \"Player_Stats\" "
					."\((.*) \"([0-9]+),([0-9]+),([0-9]+),([0-9]+),([0-9]+),([0-9]+)\"\)$/",$log,$args)) {
					list(,
						$name,,
						$id,
						$team,
						$map,
						$kills,
						$deaths,
						$killstreak,
						$rounds_survived,
						$rounds_played,
						$playing_time
					) = $args;
				
					if ($id == '0') $id = 'BOT';	
					if ($id == 'BOT' && CFG_BOT_STATS == false) return;
					
					if (sizeof($args) && strlen($name)) {
						$this->query->queryPlayerStats($name,$id,
							array(
								'map'=>(string)$map,
								'kills'=>(int)$kills,
								'deaths'=>(int)$deaths,
								'killstreak'=>(int)$killstreak,
								'rounds_survived'=>(int)$rounds_survived,
								'rounds_played'=>(int)$rounds_played,
								'playing_time'=>(int)$playing_time,
							)
						);
					}
					return;
				}
			} // End player triggers
			
			// Server triggers
			if ($this->getConfig('ss_enabled') && strpos($log,'Server triggered "') !== false) {

				// Map stats
				if (preg_match("/^Server triggered \"Map_Stats\" "
					."\((.*) \"([0-9]+),([0-9]+),([0-9]+),([0-9]+),([0-9]+),([0-9]+)\"\)$/",$log,$args)) {
					list(,
						$map,
						$kills,
						$actions,
						$score1,
						$score2,
						$time,
						$connects
					) = $args;
	
					if (sizeof($args)) {
						$this->query->queryMapStats(
							$this->query->getOwner(),
							array(
								'map_name'=>(string)$map,
								'map_kills'=>(int)$kills,
								'map_actions'=>(int)$actions,
								'map_wins_team1'=>(int)$score1,
								'map_wins_team2'=>(int)$score2,
								'map_time'=>(int)$time,
								'map_connects'=>(int)$connects
							)
						);
					}
					return;
				}

				// Server stats
				if (preg_match("/^Server triggered \"Server_Stats\" "
					."\(\"(.*),([0-9]+),([0-9]+)\"\)$/",$log,$args)) {
					list(,
						$map,
						$slots,
						$connects
					) = $args;
					
					if (sizeof($args)) {
						$this->query->queryServerStats(
							$this->query->getOwner(),
							(int)((date('w',$this->query->getTime())+1).$hour.(floor($minute/5)*5)),
							array(
								'server_map'=>(string)$map,
								'server_slots'=>(int)$slots,
								'server_connects'=>(int)$connects
							)
						);
					}
					return;
				}
			} // End server triggers
			
			// Game specific triggers
			if ($GAME == 'cstrike') {
				// Team score
				if (preg_match("/^Team \"(.*)\" triggered \"(.*)\" \((.*) \"([0-9]+)\"\) \((.*) \"([0-9]+)\"\)$/",$log,$args)) {
					list(,
						$team,
						$action,
						$team1,
						$score1,
						$team2,
						$score2
					) = $args;
					
					if (sizeof($args) && is_valid_team_action(strtolower($action))) {
						if (CFG_DEBUG||CFG_DEBUG_LOG) log_parser("TRIGGER: \"$team\" triggered \"$action\"");
						$this->query->queryGameState($this->query->getOwner(),'TEAMSCORERATIO',$score2/(($score1==0)?1:$score1));
					}
					return;
				}

				// World triggers
				if (preg_match("/^World triggered \"(.*)\"$/",$log,$args)) {
					list(,$action) = $args;
					if (sizeof($args) && is_valid_world_action(strtolower($action))) {
						// TODO
					}
					return;
				}
			} // End game triggers
			
			return;
		} // End triggers

		// Number of players
		if (strpos($log,'Server has "') !== false) {
			if (preg_match("/^Server has \"([0-9]+)\" players$/",$log,$args)) {
				list(,
					$num,
				) = $args;
	
				if (sizeof($args)) {
					if (CFG_DEBUG||CFG_DEBUG_LOG) log_parser("NUM PLAYERS: $num");
					$this->query->queryGameState($this->query->getOwner(),'NUMPLAYERS',$num);
				}
			}
			return;
		}

		// Map loading
		if (strpos($log,'Loading map "') !== false) {
			if (preg_match("/^Loading map \"(.*)\"$/",$log,$args)) {
				list(,
					$map
				) = $args;
				
				if (sizeof($args)) {
					if (CFG_DEBUG||CFG_DEBUG_LOG) log_parser("MAP LOADING: $map.");
					$this->query->queryGameReset($this->query->getOwner());
				}
			}
			return;
		}

		// Log started
		if (strpos($log,'Log file started (file "') !== false) {
			if (preg_match("/^Log file started \(file \"(.*)\"\) \(game \"(.*)\"\) \(version \"(.*)\"\)$/",$log,$args)) {
				list(,
					$logfile,
					$game,
					$version
				) = $args;
				
				if (sizeof($args)) {
					if (CFG_DEBUG||CFG_DEBUG_LOG) log_parser("LOG FILE STARTED");

					// End of month check
					$thisdate = ($this->getConfig('use_log_date'))?$this->query->getTime():time();
					$lastdate = $this->getLastDate();
					
					if ($thisdate > $lastdate) {
						if ($lastdate > 0 && (date('n',$thisdate) > date('n',$lastdate) || date('Y',$thisdate) > date('Y',$lastdate))) {				
							// Hand out awards
							if (CFG_AWARDS) {
								if (CFG_DEBUG||CFG_DEBUG_LOG) log_parser("AWARDS: ".date('F',$lastdate)." ".date('Y',$lastdate));
								$this->query->queryAwards(date('n',$lastdate),date('Y',$lastdate));
							}
							// Reset database
							if (CFG_DB_RESET) {
								if (CFG_DEBUG||CFG_DEBUG_LOG) log_parser("DATABASE RESET: ".date('F',$lastdate)." ".date('Y',$lastdate));
								$this->query->queryDBReset();
							}
						}
					}
					
					// Set new last date
					if ($this->getConfig('use_log_date') || $thisdate > $lastdate) {
						$this->query->queryGameState('PARSER','LASTDATE',$thisdate);
					}
				}
			}
			return;
		}

		// Log ended
		if (preg_match("/Log file closed/",$log)) {
			if (CFG_DEBUG||CFG_DEBUG_LOG) log_parser("LOG FILE CLOSED");
			$this->query->queryCleanup();
			return;
		}		
	} // End parse method
}
?>
