<?php
/*****************************************************************
* UCStats - A stats generation program for Half-Life games.
* Copyright (c) 2003 Mike Cao <mike@mikecao.com>
*****************************************************************/

/*
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, 
MA 02111-1307, USA.
*/

/*****************************************************************
* Query class
*****************************************************************/

Class Query {

	var $query_num;
	var $query_changes;
	var $query_access;
	var $query_cache;
	var $query_owner;
	
	// Constructor
	function Query() {
		$this->resetCache();
		$this->query_num = 0;
		$this->query_changes = 0;
		$this->query_access = 0;
	}
	
	// Player stats
	function queryPlayer($name,$id) {
		global $GAME;
		$uid = get_unique_id($name,$id);

		// Get cached data
		$cache = $this->getCache('player',$uid);

		// Not in cache, get from database
		if (is_null($cache)) {
			$result = db_get_player($uid);
			$this->query_num++;
			
			// Not in DB, insert and cache
			if (is_null($result)) {
				$this->query_changes += db_new_player($GAME,$uid,$name,$id,get_clan_name($name),CFG_SCORE_START,$this->getTime());
				$this->query_num++;
				
				// New player record
				$data = array(
					'score'=>CFG_SCORE_START,
					'kills'=>0,
					'deaths'=>0,
					'online'=>0,
					$this->getTime(),
					'0.0.0.0'
				);
				$this->doCache('player',$uid,$data);
				
				// New history record
				$this->queryPlayerScore($name,$id,CFG_SCORE_START,"start\"$name\"$id");
				$this->doCache('score',$uid,array('score_history'=>(CFG_SCORE_START."\"start\"$name\"$id")));
			}
			else {
				$data = mysql_fetch_array($result,MYSQL_ASSOC);
				$this->doCache('player',$uid,$data);
			}
			return $data;
		}
		// Is cached
		else {
			$this->query_access++;
			return $cache;
		}
	}
	
	// Player kills
	function queryPlayerKill($name1,$id1,$team1,$name2,$id2,$team2,$weapon) {

		$uid1 = get_unique_id($name1,$id1);
		$uid2 = get_unique_id($name2,$id2);
	
		$stats1 = $this->queryPlayer($name1,$id1);
		$stats2 = $this->queryPlayer($name2,$id2);

		$stats1['kills'] += 1;
		$stats2['deaths'] += 1;

		if ($team1 != $team2) {
			if ($stats1['score'] > 0) $ratio = $stats2['score']/$stats1['score'];
			else $ratio = 1;
			
			// Weapon mod
			$ratio = ($ratio+get_weapon_mod($weapon))/2;
			
			// Score change
			$change = (int)($ratio * CFG_SCORE_BASE);

			// Difficulty bonus
			if (CFG_DIFFICULTY) {
				$change += $change*($this->queryDifficulty($team1));
			}
			
			$stats1['score'] += ($change > CFG_SCORE_MAX) ? CFG_SCORE_MAX : $change;
			$stats2['score'] -= ($change > CFG_SCORE_MIN) ? CFG_SCORE_MIN : $change;
			
			if ($stats1['score'] < 1) $stats1['score'] = 1;
			if ($stats2['score'] < 1) $stats2['score'] = 1;
			
			// Update score history
			$this->queryPlayerScore($name1,$id1,$stats1['score'],"kill\"$name2\"$id2\"$weapon");
			$this->queryPlayerScore($name2,$id2,$stats2['score'],"death\"$name1\"$id1\"$weapon");
		}
				
		// Team kill
		else {
			$stats1['score'] -= CFG_SCORE_BASE;
			$this->query_changes += db_update_player($uid1,array('score'=>$stats1['score']));
			$this->query_num++;
			
			// Update score history
			$this->queryPlayerScore($name1,$id1,$stats1['score'],"tk\"$name2\"$id2\"$weapon");
		}
		
		$this->query_changes += db_update_player($uid1,
									array(
										'kills'=>$stats1['kills'],
										'score'=>$stats1['score']
									)
								);
		$this->query_changes += db_update_player($uid2,
									array(
										'deaths'=>$stats2['deaths'],
										'score'=>$stats2['score']
									)
								);
		$this->query_num += 2;

		$this->doCache('player',$uid1,$stats1);
		$this->doCache('player',$uid2,$stats2);

		// Get cached data
		$cache = $this->getCache('kills',$uid1.'_'.$uid2);
		
		// Not found, try reverse lookup
		if (is_null($cache)) $this->getCache('kills',$uid2.'_'.$uid1);

		// Not in cache, get from database
		if (is_null($cache)) {
			$result = db_get_kills($uid1,$uid2);
			$this->query_num++;

			// Not in DB, insert and cache
			if (is_null($result)) {
				$this->query_changes += db_new_kill($uid1,$uid2);
				$this->query_num++;
				$this->doCache('kills',$uid1.'_'.$uid2,array('UID1'=>$uid1,'UID2'=>$uid2,'kills1'=>1,'kills2'=>0));
			}
			else {
				$data = mysql_fetch_array($result,MYSQL_ASSOC);

				// Check for inverse
				if ($data['UID1']==$uid1) {
					$data['kills1'] += 1;
					$this->query_changes += db_update_kill($uid1,$uid2,$data['kills1'],$data['kills2']);
					$this->doCache('kills',$uid1.'_'.$uid2,$data);
				}
				else if ($data['UID1']==$uid2) {
					$data['kills2'] += 1;
					$this->query_changes += db_update_kill($uid2,$uid1,$data['kills1'],$data['kills2']);
					$this->doCache('kills',$uid2.'_'.$uid1,$data);
				}
				$this->query_num++;
			}
		}
		// Is cached
		else {
			$this->query_access++;

			// Check for inverse
			if ($cache['UID1']==$uid1) {
				$cache['kills1'] += 1;
				$this->query_changes += db_update_kill($uid1,$uid2,$cache['kills1'],$cache['kills2']);
				$this->doCache('kills',$uid1.'_'.$uid2,$cache);
			}
			else if ($cache['UID1']==$uid2) {
				$cache['kills2'] += 1;
				$this->query_changes += db_update_kill($uid1,$uid2,$cache['kills1'],$cache['kills2']);
				$this->doCache('kills',$uid2.'_'.$uid1,$cache);
			}
			$this->query_num++;
		}

		// Update timestamp
		$this->queryTimestamp();
	}
	
	// Player actions
	function queryPlayerAction($name,$id,$team,$action) {
		// Don't count this if not enough players in the server
		if ($this->queryNumPlayers() >= CFG_PLAYERS_MIN) {

			$uid = get_unique_id($name,$id);
			$action_num = 0;
			
			// Get cached data
			$cache = $this->getCache('actions',$uid.$action);
	
			// Not in cache, get values from database
			if (is_null($cache)) {
				$result = db_get_action($uid,$action);
				$this->query_num++;
				
				// Not in DB, insert and cache
				if (is_null($result)) {
					$this->query_changes += db_new_action($uid,$action,1);
					$this->query_num++;
					$this->doCache('actions',$uid.'_'.$action,array('action_name'=>$action,'action_num'=>1));
					$action_num = 1;
				}
				else {
					$data = mysql_fetch_array($result,MYSQL_ASSOC);
					$data['action_num']++;
					$this->query_changes += db_update_action($uid,$data['action_name'],$data['action_num']);
					$this->query_num++;
					$this->doCache('actions',$uid.'_'.$action,$data);
					$action_num = $data['action_num'];
				}
			}
			// Is cached
			else {
				$this->query_access++;
	
				$cache['action_num']++;
				$this->query_changes += db_update_action($uid,$cache['action_name'],$cache['action_num']);
				$this->query_num++;
				$this->doCache('actions',$uid.'_'.$action,$cache);
				$action_num = $cache['action_num'];
			}
	
			// Get player info
			$stats = $this->queryPlayer($name,$id);
			
			// Apply point modifiers
			$mod = get_action_mod($action);
			
			if ($mod != 0) {
				// Score change
				$change = (int)($mod * CFG_SCORE_BASE);
			
				// Difficulty
				if (CFG_DIFFICULTY) {
					$change += $change*($this->queryDifficulty($team));
				}

				if ($change > CFG_SCORE_MAX) {
					$change = CFG_SCORE_MAX;
				}
				else if ($change < (-1*CFG_SCORE_MIN)) {
					$change = (-1*CFG_SCORE_MIN);
				}
	
				$stats['score'] += $change;
				if ($stats['score'] < 1) $stats['score'] = 1;
				
				// Update player score
				$this->query_changes += db_update_player($uid,array('score'=>$stats['score']));
				$this->query_num++;
				$this->doCache('player',$uid,$stats);
				
				// Update score history
				$this->queryPlayerScore($name,$id,$stats['score'],"action\"$action\"$action_num");
			}
			
			// Update timestamp
			$this->queryTimestamp();
		}
	}
	
	// Player score history
	function queryPlayerScore($name,$id,$score,$reason) {

		$uid = get_unique_id($name,$id);

		// Get cached data
		$cache = $this->getCache('score',$uid);
		
		// Not in cache, get from database
		if (is_null($cache)) {
			$result = db_get_score($uid);
			$this->query_num++;
			
			if (is_null($result)) {
				$this->query_changes += db_new_score($uid,"$score\"$reason");
				$this->query_num++;
				$this->doCache('score',$uid,array('uid'=>$uid,'score_history'=>"$score\"$reason"));
			}
			else {
				$row = mysql_fetch_array($result,MYSQL_ASSOC);
				$history = $row['score_history']."\n$score\"$reason";

				$scores = explode("\n",$history);
				if (sizeof($scores) > CFG_SCORES_MAX) {
					$history = "";
					$scores = array_slice($scores,sizeof($scores)-CFG_SCORES_MAX);
					for ($i = 0; $i < sizeof($scores); $i++) {
						$history .= $scores[$i];
						if ($i < sizeof($scores)-1) $history .= "\n";
					}
				}
				$this->query_changes += db_update_score($uid,$history);
				$this->query_num++;
				$this->doCache('score',$uid,array('score_history'=>$history));
			}
		}
		// Is cached
		else {
			$this->query_access++;
		
			$history = $cache['score_history']."\n$score\"$reason";
		
			$scores = explode("\n",$history);
			if (sizeof($scores) > 100) {
				$history = "";
				for ($i = (sizeof($scores)-100); $i < sizeof($scores); $i++) {
					$history .= $scores[$i];
					if ($i < sizeof($scores)-1) $history .= "\n";
				}
			}
			$this->query_changes += db_update_score($uid,$history);
			$this->query_num++;
			$this->doCache('score',$uid,array('score_history'=>$history));
		}
	}
	
	// Player connect
	function queryPlayerConnect($name,$id,$ip) {
		$stats = $this->queryPlayer($name,$id);
		$stats['ip'] = $ip;
		$stats['lastonline'] = $this->getTime();

		$uid = get_unique_id($name,$id);
		$this->query_changes += db_update_player($uid,array('ip'=>$stats['ip'],'lastonline'=>$stats['lastonline']));
		$this->query_num++;
		$this->doCache('player',$uid,$stats);
		
		// Update timestamp
		$this->queryTimestamp();
	}
	
	// Player disconnect
	function queryPlayerDisconnect($name,$id) {
		$uid = get_unique_id($name,$id);
		$stats = $this->queryPlayer($name,$id);

		if (isset($stats['lastonline']) && $stats['lastonline'] > 0) {
			$stats['online'] += $this->getTime() - $stats['lastonline'];
			$this->query_changes += db_update_player($uid,array('online'=>$stats['online']));
			$this->query_num++;
		}
		$stats['lastonline'] = $this->getTime();
		$this->query_changes += db_update_player($uid,array('lastonline'=>$stats['lastonline']));
		$this->query_num++;

		$this->doCache('player',$uid,$stats);
		
		// Update timestamp
		$this->queryTimestamp();
	}
	
	// Player entered the game
	function queryPlayerEntered($name,$id) {
		$stats = $this->queryPlayer($name,$id);
		$stats['lastonline'] = $this->getTime();

		$uid = get_unique_id($name,$id);
		
		// Update aliases (ID only mode)
		if (CFG_STATS_MODE == 2) {			
			$result = db_get_alias($uid,$name);
			if (!is_null($result)) {
				$data = mysql_fetch_array($result,MYSQL_ASSOC);
				$this->query_changes += db_update_alias($uid,$name,$data['alias_num']+1,$this->getTime());
			}
			else {
				$this->query_changes += db_new_alias($uid,$name,$this->getTime());
			}
			$this->query_num++;
		}
		
		$this->query_changes += db_update_player($uid,array('name'=>$name,'lastonline'=>$stats['lastonline']));
		$this->query_num++;
		$this->doCache('player',$uid,$stats);
		
		// Update timestamp
		$this->queryTimestamp();
	}
	
	// Player stats
	function queryPlayerStats($name,$id,$stats) {
		$uid = get_unique_id($name,$id);
		$map = $stats['map'];
		$result = db_get_player_stats($uid,$map);
		$this->query_num++;

		if (is_null($result)) {
			$this->query_changes += db_new_player_stats($uid,$stats);
			$this->query_num++;
		}
		else {
			$data = mysql_fetch_array($result,MYSQL_ASSOC);
			foreach ($data as $key => $value) {
				if ($key != 'map' && $key != 'killstreak') {
					$stats[$key] += (int)$value;
				}
			}
			if ($stats['killstreak'] <= $data['killstreak']) {
				$stats['killstreak'] = (int)$data['killstreak'];
			}

			unset($stats['UID']);
			unset($stats['map']);

			$this->query_changes += db_update_player_stats($uid,$map,$stats);
			$this->query_num++;
		}

		// Update timestamp
		$this->queryTimestamp();
	}
	
	// Player stats reset
	function queryPlayerReset($name,$id) {
		$uid = get_unique_id($name,$id);

		// Reset player stats
		$this->query_changes += db_delete_player($uid);
		$this->query_num++;

		// Remove player from cache
		$this->deleteCache($uid);
		
		// Create player account
		$this->queryPlayerEntered($name,$id);
		
		// Update timestamp
		$this->queryTimestamp();
	}
	
	// Player password
	function queryPlayerPassword($name,$id,$pw) {
		$uid = get_unique_id($name,$id);
		// TODO
		
		// Update timestamp
		$this->queryTimestamp();
	}
	
	// Weapon stats
	function queryWeaponStats($name,$id,$stats) {
		$uid = get_unique_id($name,$id);
		$result = db_get_weapon($uid,$stats['weapon_name']);
		$this->query_num++;

		if (is_null($result)) {
			$this->query_changes += db_new_weapon($uid,$stats);
			$this->query_num++;
		}
		else {
			$data = mysql_fetch_array($result,MYSQL_ASSOC);
			foreach ($data as $key => $value) {
				if ($key != 'weapon_name') {
					$stats[$key] += $value;
				}
			}
			$this->query_changes += db_update_weapon($uid,$stats);
			$this->query_num++;
		}

		// Update timestamp
		$this->queryTimestamp();
	}
	
	// Weapon hits
	function queryWeaponHits($name,$id,$stats) {
		$uid = get_unique_id($name,$id);
		$result = db_get_hits($uid,$stats['weapon_name']);
		$this->query_num++;

		if (is_null($result)) {
			$this->query_changes += db_new_hits($uid,$stats);
			$this->query_num++;
		}
		else {
			$data = mysql_fetch_array($result,MYSQL_ASSOC);
			foreach ($data as $key => $value) {
				if ($key != 'weapon_name') {
					$stats[$key] += $value;
				}
			}
			$this->query_changes += db_update_hits($uid,$stats);
			$this->query_num++;
		}

		// Update timestamp
		$this->queryTimestamp();
	}
	
	// Map info
	function queryMapStats($address,$stats) {
		$sid = db_get_server_id($address);

		if (!is_null($sid)) {
			$result = db_get_map($sid,$stats['map_name']);
			$this->query_num++;
			
			if (is_null($result)) {
				$this->query_changes += db_new_map($sid,$stats);
				$this->query_num++;
			}
			else {
				$data = mysql_fetch_array($result,MYSQL_ASSOC);
				foreach ($data as $key => $value) {
					if ($key != 'map_name') {
						$stats[$key] += $value;
					}
				}
				$this->query_changes += db_update_map($sid,$stats);
				$this->query_num++;
			}
			// Update timestamp
			$this->queryTimestamp();
		}
	}
	
	// Server stats
	function queryServerStats($address,$date,$stats) {
		$sid = db_get_server_id($address);

		if (!is_null($sid)) {
			$this->query_changes += db_update_server_stats($sid,$date,$stats);
			$this->query_num++;
	
			// Update timestamp
			$this->queryTimestamp();
		}
	}
	
	// Updates timestamp
	function queryTimestamp() {
		$this->query_changes += db_update_key('STATS','TIMESTAMP',time());
		$this->query_num++;
	}
	
	// Give out awards
	function queryAwards($month,$year) {
		global $GAME;
		global $GAME_weapons;
		global $GAME_misc;

		$this->query_changes += db_reset_awards($GAME,$month,$year);
		$this->query_num++;
		
		// Weapon awards
		foreach ($GAME_weapons as $key => $value) {
			$this->query_changes += db_award_cat($GAME,$month,$year,$key);
			$this->query_num++;
		}
		
		// Misc awards
		foreach ($GAME_misc as $key => $value) {
			$this->query_changes += db_award_cat($GAME,$month,$year,$key);
			$this->query_num++;
		}
		
		$this->query_changes += db_award_topten($GAME,$month,$year);
		$this->query_num++;
		
		// Update timestamp
		$this->queryTimestamp();
	}
	
	// Get number of players
	function queryNumPlayers() {
		$result = db_get_key($this->query_owner,'NUMPLAYERS');
		if (!is_null($result)) {
			$data = mysql_fetch_array($result,MYSQL_ASSOC);
			$num = $data['key_value'];
			return $num;
		}
		else {
			return 0;
		}
	}
	
	// Get difficulty
	function queryDifficulty($team) {
		global $GAME_teams;
		$result = db_get_key($this->query_owner,'TEAMSCORERATIO');
		if (!is_null($result)) {
			$data = mysql_fetch_array($result,MYSQL_ASSOC);
			$ratio = $data['key_value'];
			if ($ratio <= 0) $ratio = 1;
		}
		else {
			$ratio = 1;
		}
		
		// If on the other team, flip the ratio
		if ($team == $GAME_teams['team1']['id']) {
			$ratio = 1/$ratio;
		}

		if ($ratio > CFG_DIFFICULTY_MAX) {
			$ratio = CFG_DIFFICULTY_MAX;
		}
		
		return $ratio;
	}
	
	// Cleanup procedures
	function queryCleanup() {
		// Cleanup database
		if (CFG_DB_CLEAN) {
			global $GAME;
			$changes = db_clean($GAME);
			$this->query_changes += $changes;
			$this->query_num++;
			
			if (CFG_USAGE_LOG) log_usage("Database cleanup: $changes records purged.");
		}
		
		// Cleanup web cache
		if (CFG_CACHE) {
			$changes = task_dir_cleanup(WEB_PATH.'/cache/',time());
			if (CFG_USAGE_LOG) log_usage("Web cache cleanup: $changes files removed.");
		}

		// Reset query cache
		$this->resetCache();
		
		// Log query usage
		$this->logQueryUsage();
		
		// Update timestamp
		$this->queryTimestamp();
	}
	
	// Resets the database
	function queryDBReset() {
		global $GAME;
		$this->query_changes += db_reset($GAME);
		$this->query_num++;
	}
	
	// Updates the game state
	function queryGameState($owner,$key,$value) {
		$result = db_get_key($owner,$key);
		if (is_null($result)) {
			$this->query_changes += db_new_key($owner,$key,$value);
			$this->query_num++;
		}
		else {
			$this->query_changes += db_update_key($owner,$key,$value);
			$this->query_num++;
		}
	}
	
	// Resets the game state
	function queryGameReset($owner) {
		$this->query_changes += db_reset_keys($owner);
		$this->query_num++;
	}
	
	// Sets the log time
	function setTime($time) {
		$this->query_time = $time;
	}
	
	// Gets the log time
	function getTime() {
		return $this->query_time;
	}
	
	// Set the owner of the current query process
	function setOwner($owner) {
		$this->query_owner = $owner;
	}
	
	// Gets the owner
	function getOwner() {
		return $this->query_owner;
	}
	
	// Returns data from cache
	function getCache($cat,$key) {
		if (array_key_exists($key,$this->query_cache[$cat])) {
			return $this->query_cache[$cat][$key];
		}
		else {
			return null;
		}
	}
	
	// Caches data
	function doCache($cat,$key,$data) {
		$this->query_cache[$cat][$key] = $data;
	}
	
	// Delete cached data
	function deleteCache($uid) {
		foreach ($this->query_cache as $cat => $data) {
			if ($cat != 'kills') {
				foreach($data as $key => $value) {
					if (strpos($key,$uid) !== false) {
						unset($this->query_cache[$cat][$key]);
					}
				}
			}
		}
	}

	// Resets the cache
	function resetCache() {
		$this->query_cache = array(
			'player'=>array(),
			'kills'=>array(),
			'actions'=>array(),
			'score'=>array()
		);
	}
	
	// Logs query usage
	function logQueryUsage() {
		if (CFG_USAGE_LOG) {
			log_usage('Query usage: total '.$this->query_num.', changes '.$this->query_changes.', cached '.$this->query_access);
		}
		$this->query_changes = 0;
		$this->query_num = 0;
		$this->query_access = 0;
		$this->query_time = 0;
	}
}
?>
