<?php
/*****************************************************************
* UCStats - A stats generation program for Half-Life games.
* Copyright (c) 2003 Mike Cao <mike@mikecao.com>
*****************************************************************/

/*
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, 
MA 02111-1307, USA.
*/

/*****************************************************************
* Socket class
*****************************************************************/

Class Socket {

	var $server_ip;
	var $server_port;
	var $server_password;
	var $socket;
	var $connected;
	var $challenge;

	// Constructor
	function Socket() {
		$this->connected = false;
		$this->challenge = 0;
	}
	
	// Open socket
	function connect($server_ip,$server_port,$server_password) {
		$this->server_ip = $server_ip;
		$this->server_port = $server_port;
		$this->server_password = $server_password;
	
		$fp = fsockopen('udp://'.$this->server_ip,$this->server_port, $errno, $errstr,5);
		if ($fp) {
			$this->connected = true;
		}
		else {
			$this->connected = false;
			return false;
		}

		$this->socket = $fp;
		return true;
	}

	// Close socket
	function disconnect() {
		if ($this->isConnected()) {
			fclose($this->socket);
		}
		$this->connected = false;
	}
	
	// Send command through open socket
	function sendCommand($command) {
		if (!$this->isConnected()) {
			return $this->connected;
		}
		
		// Get challenge number
		if ($this->challenge == 0) {
			$challenge = "\xff\xff\xff\xffchallenge rcon\n";
			$result = $this->getResult($challenge);
		
			// No connection
			if (trim($result)=='') {
				$this->connected = false;
				return false;
			}
			$result = explode(' ',$result);
			$this->challenge = trim($result[2]);
		}
		
		// Build command
		$command = "\xff\xff\xff\xffrcon $this->challenge \"$this->server_password\" $command\n";
		
		// Return command result
		return trim($this->getResult($command));
	}

	// Get result of a command
	function getResult($command) {
		if (!$this->isConnected()) {
			return $this->connected;
		}

		// Send command
		if (strlen($command)) {
			fputs($this->socket,$command,strlen($command));
		}
		
		// Get result
		$result = fread($this->socket,1);
		$status = socket_get_status($this->socket);
		$result .= fread($this->socket,$status['unread_bytes']);
		
		// If there is another package waiting
		if(substr($result,0,4) == "\xfe\xff\xff\xff") {
			$result2 = fread($this->socket, 1);
			$status = socket_get_status($this->socket);
			$result2 .= fread($this->socket,$status['unread_bytes']);
			
			if(strlen($result) > strlen($result2)) {
				$result = substr($result,14).substr($result2,9);
			}
			else {
				$result = substr($result2,14).substr($result,9);
			}
		}
		else {
			$result = substr($result, 5);
		}
		
		// Return result
		return $result;
	}
	
	// Get connection status
	function isConnected() {
		return $this->connected;
	}
}
?>