<?php
/*****************************************************************
* UCStats - A stats generation program for Half-Life games.
* Copyright (c) 2003 Mike Cao <mike@mikecao.com>
*****************************************************************/

/*
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/*****************************************************************
* Database functions
*****************************************************************/

// Select query handler
function db_select($query,$func) {
	if (CFG_DEBUG||CFG_DEBUG_LOG) {
		log_db("DB: $query ($func)");
	}

	$result = mysql_query($query)
		or db_error("ERROR: Select query failed. ($func)",$query);

	if (mysql_num_rows($result)) {
		return $result;
	}
	return null;
}

// Insert query handler
function db_insert($query,$func) {
	if (CFG_DEBUG||CFG_DEBUG_LOG) {
		log_db("DB: $query ($func)");
	}

	$result = mysql_query($query)
		or db_error("ERROR: Insert query failed. ($func)",$query);

	return mysql_affected_rows();
}

// Update query handler
function db_update($query,$func) {
	if (CFG_DEBUG||CFG_DEBUG_LOG) {
		log_db("DB: $query ($func)");
	}

	$result = mysql_query($query)
		or db_error("ERROR: Update query failed. ($func)",$query);

	return mysql_affected_rows();
}

// Delete query handler
function db_delete($query,$func) {
	if (CFG_DEBUG||CFG_DEBUG_LOG) {
		log_db("DB: $query ($func)");
	}

	// If there is no where clause we have to count the records in the table
	// because mysql doesn't return the correct number of rows affected.
	$count = 0;
	if (!preg_match('/where /i',$query)) {
		$result = db_select("
			SELECT count(*)
			  FROM ".substr($query,strrpos($query," ")),
			'db_delete:count'
		);
		$row = mysql_fetch_row($result);
		$count = $row[0];
	}
	
	$result = mysql_query($query)
		or db_error("ERROR: Delete query failed. ($func)",$query);
	
	if ($count > 0) {
		return $count;
	}
	else {
		return mysql_affected_rows();
	}
}

// Database error
function db_error($error,$query) {
	$mysql_error = mysql_errno().": ". mysql_error();
	if (CFG_ERROR_LOG) {
		log_error($error.' ['.$mysql_error.'] QUERY: '.$query);
	}
	die("$error\n<br>QUERY: $query\n<br>$mysql_error");
}


/*****************************************************************
* Stats queries
*****************************************************************/

// Get game state key
function db_get_key($owner,$key) {
	return db_select("
		SELECT key_value
		  FROM ucstats_keys
		 WHERE key_owner='".addslashes($owner)."'
		   AND key_name='".addslashes($key)."'",
		'db_get_key'
	);
}

// Insert new game state key
function db_new_key($owner,$key,$value) {
	return db_insert("
		INSERT INTO ucstats_keys (
			key_owner,
			key_name,
			key_value
		)
		VALUES (
			'".addslashes($owner)."',
			'".addslashes($key)."',
			'".addslashes($value)."'
		)",
		'db_new_key'
	);
}

// Update game state key
function db_update_key($owner,$key,$value) {
	return db_update("
		UPDATE ucstats_keys
		   SET key_value='".addslashes($value)."'
		 WHERE key_owner='".addslashes($owner)."'
		   AND key_name='".addslashes($key)."'",
		'db_update_key'
	);
}

// Resets a game state
function db_reset_keys($owner) {
	return db_delete("
		DELETE
		  FROM ucstats_keys
		 WHERE key_owner='".addslashes($owner)."'",
		'db_reset_keys'
	);
}

// Insert new player
function db_new_player($game,$uid,$name,$id,$clan,$score,$lastonline) {
	return db_insert("
		INSERT INTO ucstats_players (
			GAME,
			UID,
			name,
			id,
			clan,
			score,
			kills,
			deaths,
			online,
			lastonline
		)
		VALUES (
			'".addslashes($game)."',
			'".addslashes($uid)."',
			'".addslashes($name)."',
			'".addslashes($id)."',
			'".addslashes($clan)."',
			$score,
			0,
			0,
			0,
			$lastonline
		)",
		'db_new_player'
	);
}

// Get player stats
function db_get_player($uid) {
	return db_select("
		SELECT score,
			   kills,
			   deaths,
			   online,
			   lastonline,
			   ip
		  FROM ucstats_players
		 WHERE UID='$uid'",
		'db_get_player'
	);
}

// Update player stats
function db_update_player($uid,$stats) {
	return db_update("
		UPDATE ucstats_players
		   SET ".get_set_string($stats)."
		 WHERE UID='$uid'",
		'db_update_player'
	);
}

// Gets player rank
function db_get_player_rank($score) {
	$result = db_select("
		SELECT count(*) as rank
		  FROM ucstats_players
		 WHERE score >= $score",
		'db_web_player'
	);
	if (!is_null($result)) {
		$row = mysql_fetch_array($result,MYSQL_ASSOC);
		return $row['rank'];
	}
}

// Get player kill
function db_get_kills($uid1,$uid2) {
	$result = db_select("
		SELECT UID1,
			   UID2,
			   kills1,
			   kills2
		  FROM ucstats_kills
		 WHERE UID1='$uid1'
		   AND UID2='$uid2'",
		'db_get_kills:1'
	);
	
	if (is_null($result)) {
		return db_select("
			SELECT UID1,
				   UID2,
				   kills1,
				   kills2
			  FROM ucstats_kills
			 WHERE UID1='$uid2'
			   AND UID2='$uid1'",
			'db_get_kills:2'
		);
	}
	else {
		return $result;
	}
}

// Insert new player kill
function db_new_kill($uid1,$uid2) {
	return db_insert("
		INSERT INTO ucstats_kills (
			UID1,
			UID2,
			kills1,
			kills2
		)
		VALUES (
			'$uid1',
			'$uid2',
			1,
			0
		)",
		'db_new_kill'
	);
}

// Update player kill
function db_update_kill($uid1,$uid2,$kills1,$kills2) {
	return db_update("
		UPDATE ucstats_kills
		   SET kills1=$kills1,
		   	   kills2=$kills2
		 WHERE UID1='$uid1'
		   AND UID2='$uid2'",
		'db_update_kill'
	);
}

// Get player actions
function db_get_action($uid,$action) {
	return db_select("
		SELECT action_name,
			   action_num
		  FROM ucstats_actions
		 WHERE UID='$uid'
		   AND action_name='$action'",
		'db_get_actions'
	);
}

// Insert new player action
function db_new_action($uid,$action,$num) {
	return db_insert("
		INSERT
		  INTO ucstats_actions
		VALUES ('$uid','$action',$num)",
		'db_new_action'
	);
}

// Update player action
function db_update_action($uid,$action,$num) {
	return db_update("
		UPDATE ucstats_actions
		   SET action_num=$num
		 WHERE UID='$uid'
		   AND action_name='$action'",
		'db_update_action'
	);
}

// Insert new player alias
function db_new_alias($uid,$name,$date) {
	return db_insert("
		INSERT
		  INTO ucstats_alias
		VALUES ('$uid','".addslashes($name)."',1,$date)",
		'db_new_alias'
	);
}

// Update player alias
function db_update_alias($uid,$name,$num,$date) {
	return db_update("
		UPDATE ucstats_alias
		   SET alias_num=$num,alias_date=$date
		 WHERE UID='$uid'
		   AND alias_name='".addslashes($name)."'",
		'db_update_alias'
	);
}

// Get player alias by name
function db_get_alias($uid,$name) {
	return db_select("
		SELECT alias_name,
			   alias_num
		  FROM ucstats_alias
		 WHERE UID='$uid'
		   AND alias_name='".addslashes($name)."'",
		'db_get_alias'
	);		
}

// Insert new player stats
function db_new_player_stats($uid,$stats) {
	return db_insert("
		INSERT INTO ucstats_playerstats (
			UID,
			map,
			kills,
			deaths,
			killstreak,
			rounds_survived,
			rounds_played,
			playing_time
		)
		VALUES (
			'$uid',
			'".$stats['map']."',
			".$stats['kills'].",
			".$stats['deaths'].",
			".$stats['killstreak'].",
			".$stats['rounds_survived'].",
			".$stats['rounds_played'].",
			".$stats['playing_time']."
		)",
		'db_new_player_stats'
	);
}

// Get player stats
function db_get_player_stats($uid,$map) {
	return db_select("
		SELECT *
		  FROM ucstats_playerstats
		 WHERE UID='".addslashes($uid)."'
		   AND map='".addslashes($map)."'",
		'db_get_player_stats'
	);
}

// Update player stats
function db_update_player_stats($uid,$map,$stats) {
	return db_update("
		UPDATE ucstats_playerstats
		   SET ".get_set_string($stats)."
		 WHERE UID='".addslashes($uid)."'
		   AND map='".addslashes($map)."'",
		'db_update_player_stats'
	);
}

// Get weapon stats
function db_get_weapon($uid,$weapon) {
	return db_select("
		SELECT weapon_kills,
			   weapon_deaths,
			   weapon_headshots,
			   weapon_teamkills,
			   weapon_shots,
			   weapon_hits,
			   weapon_damage
		  FROM ucstats_weapons
		 WHERE UID='$uid'
		   AND weapon_name='$weapon'",
		'db_get_weapon'
	);
}

// Insert new weapon stats
function db_new_weapon($uid,$stats) {
	return db_insert("
		INSERT INTO ucstats_weapons (
			UID,
			weapon_name,
			weapon_kills,
			weapon_deaths,
			weapon_headshots,
			weapon_teamkills,
			weapon_shots,
			weapon_hits,
			weapon_damage
		)
		VALUES (
			'$uid',
			'".$stats['weapon_name']."',
			".$stats['weapon_kills'].",
			".$stats['weapon_deaths'].",
			".$stats['weapon_headshots'].",
			".$stats['weapon_teamkills'].",
			".$stats['weapon_shots'].",
			".$stats['weapon_hits'].",
			".$stats['weapon_damage']."
		)",
		'db_new_weapon'
	);
}

// Update weapon stats
function db_update_weapon($uid,$stats) {
	return db_update("
		UPDATE ucstats_weapons
		   SET ".get_set_string($stats)."
		 WHERE UID='$uid'
		   AND weapon_name='".$stats['weapon_name']."'",
		'db_update_weapon'
	);
}

// Get weapon hits
function db_get_hits($uid,$weapon) {
	return db_select("
		SELECT hits_generic,
			   hits_head,
			   hits_chest,
			   hits_stomach,
			   hits_leftarm,
			   hits_rightarm,
			   hits_leftleg,
			   hits_rightleg
		  FROM ucstats_hits
		 WHERE UID='$uid'
		   AND weapon_name='$weapon'",
		'db_get_hits'
	);
}

// Insert new weapon hits
function db_new_hits($uid,$stats) {
	$stats['UID'] = $uid;
	return db_insert("
		INSERT INTO ucstats_hits (
			UID,
			weapon_name,
			hits_generic,
			hits_head,
			hits_chest,
			hits_stomach,
			hits_leftarm,
			hits_rightarm,
			hits_leftleg,
			hits_rightleg
		)
		VALUES (
			'$uid',
			'".$stats['weapon_name']."',
			".$stats['hits_generic'].",
			".$stats['hits_head'].",
			".$stats['hits_chest'].",
			".$stats['hits_stomach'].",
			".$stats['hits_leftarm'].",
			".$stats['hits_rightarm'].",
			".$stats['hits_leftleg'].",
			".$stats['hits_rightleg']."
		)",
		'db_new_hits'
	);
}

// Update weapon hits
function db_update_hits($uid,$stats) {
	return db_update("
		UPDATE ucstats_hits
		   SET ".get_set_string($stats)."
		 WHERE UID='$uid'
		   AND weapon_name='".$stats['weapon_name']."'",
		'db_update_hits'
	);
}

// Get score history
function db_get_score($uid) {
	return db_select("
		SELECT score_history
		  FROM ucstats_score
		 WHERE UID='$uid'",
		'db_get_score'
	);
}

// Insert new score history
function db_new_score($uid,$history) {
	return db_insert("
		INSERT INTO ucstats_score
			(UID,score_history)
		VALUES
			('$uid','".addslashes($history)."')",
		'db_new_score'
	);
}

// Update score history
function db_update_score($uid,$history) {
	return db_update("
		UPDATE ucstats_score
		   SET score_history='".addslashes($history)."'
		 WHERE UID='$uid'",
		'db_update_score'
	);
}

// Get map stats
function db_get_map($sid,$map) {
	return db_select("
		SELECT map_kills,
			   map_actions,
			   map_wins_team1,
			   map_wins_team2,
			   map_time,
			   map_connects
		  FROM ucstats_maps
		 WHERE SID='$sid'
		   AND map_name='$map'",
		'db_get_map'
	);
}

// Insert new map
function db_new_map($sid,$stats) {
	return db_insert("
		INSERT INTO ucstats_maps (
			SID,
			map_name,
			map_kills,
			map_actions,
			map_wins_team1,
			map_wins_team2,
			map_time,
			map_connects
		)
		VALUES (
			'$sid',
			'".$stats['map_name']."',
			".$stats['map_kills'].",
			".$stats['map_actions'].",
			".$stats['map_wins_team1'].",
			".$stats['map_wins_team2'].",
			".$stats['map_time'].",
			".$stats['map_connects']."
		)",
		'db_new_map'
	);
}

// Update map stats
function db_update_map($sid,$stats) {
	return db_update("
		UPDATE ucstats_maps
		   SET ".get_set_string($stats)."
		 WHERE SID='$sid'
		   AND map_name='".$stats['map_name']."'",
		'db_update_map'
	);
}

// Update server stats by address
function db_update_server_stats($sid,$date,$stats) {
	return db_update("
		UPDATE ucstats_serverstats
		   SET ".get_set_string($stats)."
		 WHERE SID='$sid'
		   AND server_date='$date'",
		'db_update_server_stats'
	);
}

// Gets category rankings
function db_get_rank($game,$key,$limit) {
	// Weapon
	if (is_valid_weapon($key)) {
		return db_select("
			SELECT ucstats_players.UID,
				   name,
				   id,
				   clan,
				   weapon_kills as $key
			  FROM ucstats_players,
			  	   ucstats_weapons
			 WHERE weapon_name='$key'
			   AND weapon_kills > 0
			   AND ucstats_weapons.UID=ucstats_players.UID
			   AND ucstats_players.GAME='".addslashes($game)."'
		     ORDER
			    BY weapon_kills
			  DESC
			 LIMIT $limit",
			'db_get_rank:'.$key
		);
	}
	// Action
	else if (is_valid_action($key)) {
		return db_select("
			SELECT ucstats_players.UID,
				   name,
				   id,
				   clan,
				   action_num as $key
			  FROM ucstats_players,
			  	   ucstats_actions
			 WHERE action_name='$key'
			   AND action_num > 0
			   AND ucstats_actions.UID=ucstats_players.UID
			   AND ucstats_players.GAME='".addslashes($game)."'
			 ORDER
			    BY $key
			  DESC
			 LIMIT $limit",
			'db_get_rank:'.$key
		);
	}
	// Player
	else if ($key == 'kills' || $key == 'deaths' || $key == 'online') {
		return db_select("
			SELECT UID,
				   name,
				   id,
				   clan,
				   $key
			  FROM ucstats_players
			 WHERE $key > 0
			   AND GAME='".addslashes($game)."'
			 ORDER
			    BY $key
			  DESC
			 LIMIT $limit",
			'db_get_rank:'.$key
		);
	}
	// Other
	else if ($key == 'points') {
		return db_select("
			SELECT ucstats_players.UID,
				   name,
				   id,
				   clan,
				   ((score-".CFG_SCORE_START.")/(kills+deaths+SUM(action_num))) AS $key
			  FROM ucstats_players,
			  	   ucstats_actions
			 WHERE ucstats_players.UID=ucstats_actions.UID
			   AND kills > ".CFG_KILLS_MIN."
			   AND ucstats_players.GAME='".addslashes($game)."'
			 GROUP
			    BY ucstats_players.UID
			 ORDER
			    BY $key
			  DESC
			 LIMIT $limit",
			'db_get_rank:'.$key
		);
	}
	else if ($key == 'best_kpd') {
		return db_select("
			SELECT UID,
				   name,
				   id,
				   clan,
				   (kills/deaths) AS $key
			  FROM ucstats_players
			 WHERE kills > ".CFG_KILLS_MIN."
			   AND deaths > 0
			   AND GAME='".addslashes($game)."'
			 ORDER
			    BY $key
			  DESC
			 LIMIT $limit",
			'db_get_rank:'.$key
		);
	}
	else if ($key == 'worst_kpd') {
		return db_select("
			SELECT UID,
				   name,
				   id,
				   clan,
				   (kills/deaths) AS $key
			  FROM ucstats_players
			 WHERE kills > ".CFG_KILLS_MIN."
			   AND deaths > 0
			   AND GAME='".addslashes($game)."'
			 ORDER
			    BY $key
			   ASC
			 LIMIT $limit",
			'db_get_rank:'.$key
		);
	}
	else if ($key == 'best_kpm') {
		return db_select("
			SELECT UID,
				   name,
				   id,
				   clan,
				   ((kills/online)*60) AS $key
			  FROM ucstats_players
			 WHERE kills > ".CFG_KILLS_MIN."
			   AND online > 60
			   AND GAME='".addslashes($game)."'
			 ORDER
			    BY $key
			  DESC
			 LIMIT $limit",
			'db_get_rank:'.$key
		);
	}
	else if ($key == 'worst_kpm') {
		return db_select("
			SELECT UID,
				   name,
				   id,
				   clan,
				   ((kills/online)*60) AS $key
			  FROM ucstats_players
			 WHERE kills > ".CFG_KILLS_MIN."
			   AND online > 60
			 ORDER
			    BY $key
			   ASC
			 LIMIT $limit",
			'db_get_rank:'.$key
		);
	}
	else if ($key == 'best_shot') {
		return db_select("
			SELECT ucstats_players.UID,
				   name,
				   id,
				   clan,
				   (SUM(weapon_hits)/SUM(weapon_shots)) AS $key
			  FROM ucstats_players,
			  	   ucstats_weapons
			 WHERE kills > ".CFG_KILLS_MIN."
			   AND ucstats_players.UID=ucstats_weapons.UID
			   AND ucstats_players.GAME='".addslashes($game)."'
			 GROUP
			    BY UID
			 ORDER
			    BY $key
			  DESC
			 LIMIT $limit",
			'db_get_rank:'.$key
		);
	}
	else if ($key == 'worst_shot') {
		return db_select("
			SELECT ucstats_players.UID,
				   name,
				   id,
				   clan,
				   (SUM(weapon_hits)/SUM(weapon_shots)) AS $key
			  FROM ucstats_players,
			  	   ucstats_weapons
			 WHERE kills > ".CFG_KILLS_MIN."
			   AND ucstats_players.UID=ucstats_weapons.UID
			   AND ucstats_players.GAME='".addslashes($game)."'
			 GROUP
			    BY UID
			 ORDER
			    BY $key
			   ASC
			 LIMIT $limit",
			'db_get_rank:'.$key
		);
	}
	else if ($key == 'best_harm') {
		return db_select("
			SELECT ucstats_players.UID,
				   name,
				   id,
				   clan,
				   (SUM(weapon_damage)/SUM(weapon_kills)) AS $key
			  FROM ucstats_players,
			  	   ucstats_weapons
			 WHERE kills > ".CFG_KILLS_MIN."
			   AND weapon_kills > 0
			   AND ucstats_players.UID=ucstats_weapons.UID
			   AND ucstats_players.GAME='".addslashes($game)."'
			 GROUP
			    BY ucstats_weapons.UID
			 ORDER
			    BY $key
			  DESC
			 LIMIT $limit",
			'db_get_rank:'.$key
		);
	}
	else if ($key == 'worst_harm') {
		return db_select("
			SELECT ucstats_players.UID,
				   name,
				   id,
				   clan,
				   (SUM(weapon_damage)/SUM(weapon_kills)) AS $key
			  FROM ucstats_players,
			  	   ucstats_weapons
			 WHERE kills > ".CFG_KILLS_MIN."
			   AND weapon_kills > 0
			   AND ucstats_players.UID=ucstats_weapons.UID
			   AND ucstats_players.GAME='".addslashes($game)."'
			 GROUP
			    BY ucstats_weapons.UID
		     ORDER
			    BY $key
			   ASC
			 LIMIT $limit",
			'db_get_rank:'.$key
		);
	}
	else if ($key == 'survivor') {
		return db_select("
			SELECT ucstats_players.UID,
				   name,
				   id,
				   clan,
				   (SUM(ucstats_playerstats.rounds_survived)/SUM(ucstats_playerstats.rounds_played)) AS $key
			  FROM ucstats_players,
			  	   ucstats_playerstats
			 WHERE ucstats_players.kills > ".CFG_KILLS_MIN."
			   AND ucstats_playerstats.rounds_played > 0
			   AND ucstats_players.UID=ucstats_playerstats.UID
			   AND ucstats_players.GAME='".addslashes($game)."'
			 GROUP
			    BY ucstats_players.UID
			 ORDER
			    BY $key
			  DESC
			 LIMIT $limit",
			'db_get_rank:'.$key
		);
	}
	else if ($key == 'killstreak') {
		return db_select("
			SELECT ucstats_players.UID,
				   name,
				   id,
				   clan,
				   MAX(ucstats_playerstats.killstreak) AS $key
			  FROM ucstats_players,
			  	   ucstats_playerstats
			 WHERE ucstats_players.kills > ".CFG_KILLS_MIN."
			   AND ucstats_players.UID=ucstats_playerstats.UID
			   AND ucstats_players.GAME='".addslashes($game)."'
			 GROUP
			    BY ucstats_players.UID
			 ORDER
			    BY $key
			  DESC
			 LIMIT $limit",
			'db_get_rank:'.$key
		);
	}
}

// Give out awards for categories
function db_award_cat($game,$month,$year,$award) {
	$result = db_get_rank($game,$award,1);
	if (!is_null($result)) {
		$data = mysql_fetch_array($result);

		// Hack
		if (strlen(strstr($award,'best_')) || strlen(strstr($award,'worst_')) || $award == 'survivor') {
			$data[$award] *= 1000;
		}
		
		return db_insert("
			INSERT INTO ucstats_awards (
				GAME,
				UID,
				name,
				id,
				clan,
				award_name,
				award_month,
				award_year,
				award_num
			)
			VALUES (
				'".addslashes($game)."',
				'".addslashes($data['UID'])."',
				'".addslashes($data['name'])."',
				'".addslashes($data['id'])."',
				'".addslashes($data['clan'])."',
				'$award',
				$month,
				$year,
				".$data[$award]."
			)",
			'db_award_cat:insert:'.$award
		);
	}
}

// Top 10 player awards
function db_award_topten($game,$month,$year) {
	$result = db_select("
		SELECT UID,
			   name,
			   id,
			   clan,
			   score
		  FROM ucstats_players
		 WHERE score > 0
		   AND GAME = '".addslashes($game)."'
		 ORDER
		    BY score
		  DESC
		 LIMIT 10",
		'db_award_topten:select'
	);

	if (!is_null($result)) {
		$changes = 0;
		$rank = 1;
		while ($data = mysql_fetch_array($result)) {
			$changes += db_insert("
				INSERT INTO ucstats_awards (
					GAME,
					UID,
					name,
					id,
					clan,
					award_name,
					award_month,
					award_year,
					award_num
				)
				VALUES (
					'".addslashes($game)."',
					'".addslashes($data['UID'])."',
					'".addslashes($data['name'])."',
					'".addslashes($data['id'])."',
					'".addslashes($data['clan'])."',
					'$rank',
					$month,
					$year,
					".$data['score']."
				)",
				'db_award_topten:insert'
			);
			$rank++;
    	}
		return $changes;
	}
}

function db_get_log($owner) {
	return db_select("
		SELECT *
		  FROM ucstats_logs
		 WHERE log_owner='".addslashes($owner)."'
		 ORDER
		    BY log_num
		  ASC",
		'db_get_log'
	);
}

function db_new_log($owner,$value) {
	return db_insert("
		INSERT INTO ucstats_logs (
			log_owner,
			log_value
		)
		VALUES (
			'".addslashes($owner)."',
			'".addslashes($value)."'
		)",
		'db_new_log'
	);
}

function db_delete_log($owner,$num) {
	return db_delete("
		DELETE
		  FROM ucstats_logs
		 WHERE log_owner='".addslashes($owner)."'
		 AND log_num <= $num",
		'db_delete_log'
	);
}

/*****************************************************************
* Web queries
*****************************************************************/

// Gets all players
function db_web_players($game,$sort,$dir,$query,$type) {
	if (!is_null($query) && !is_null($type)) {
		$where = "WHERE GAME = '".addslashes($game)."'";
		if ($type == 'name') {
			$where .= "AND name like '%".addslashes($query)."%'";
		}
		else if ($type == 'ID') {
			$where .= "AND id='".addslashes($query)."'";
		}
		else if ($type == 'clan') {
			$where .= "AND name like '".addslashes($query)."%' OR name like '%".addslashes($query)."'";
		}
	}
	return db_select("
		SELECT UID,
			   name,
			   id,
			   score,
			   kills,
			   deaths,
			   (kills/deaths) AS kpd,
			   ((kills/online)*60) AS kpm,
			   online
		  FROM ucstats_players
		$where
		 ORDER
		    BY $sort $dir",
		'db_web_players'
	);
}

// Gets all clans
function db_web_clans($game,$sort,$dir) {
	return db_select("
		SELECT
	  DISTINCT clan,
	  		   AVG(score) as score,
			   COUNT(clan) as members,
	  		   SUM(kills) as kills,
			   SUM(deaths) as deaths,
	  		   (SUM(kills)/SUM(deaths)) as kpd,
	  		   (SUM(kills)/(SUM(online)/60)) as kpm,
			   SUM(online) as online
		  FROM ucstats_players
		 WHERE clan != ''
		   AND ucstats_players.GAME='".addslashes($game)."'
		 GROUP
		    BY clan
	    HAVING members > 1
		 ORDER
		    BY $sort $dir",
		'db_web_clans'
	);
}

// Gets player info
function db_web_player($uid) {
	if (strlen($uid)) {
		return db_select("
			SELECT name,
				   id,
				   score,
				   kills,
				   deaths,
				   (kills/deaths) AS kpd,
				   ((kills/online)*60) AS kpm,
				   online,
				   lastonline
			  FROM ucstats_players
			 WHERE UID='$uid'",
			'db_web_player'
		);
	}
}

// Gets player weapons info
function db_web_player_weapons($uid,$sort,$dir) {
	$result = db_select("
		SELECT SUM(weapon_kills) as sum_weapon_kills,
			   SUM(weapon_deaths) as sum_weapon_deaths
		  FROM ucstats_weapons
		 WHERE UID='$uid'",
		'db_web_player_weapons:1'
	);

	$row = mysql_fetch_array($result);
	$kills = (int)$row['sum_weapon_kills'];
	$deaths = (int)$row['sum_weapon_deaths'];

	return db_select("
		SELECT weapon_name,
			   weapon_kills,
			   (weapon_kills/".(($kills>0)?$kills:1).") as weapon_kills_percent,
			   weapon_deaths,
			   (weapon_deaths/".(($deaths>0)?$deaths:1).") as weapon_deaths_percent,
			   weapon_shots,
			   weapon_hits,
			   (weapon_hits/weapon_shots) as weapon_accuracy,
			   weapon_damage,
			   (weapon_kills/weapon_deaths) as weapon_kpd,
			   weapon_headshots,
			   (weapon_headshots/$kills) as weapon_headshot_percent
		  FROM ucstats_weapons
		 WHERE UID='$uid'
		 GROUP
		    BY weapon_name
		 ORDER
		    BY $sort $dir",
		'db_web_player_weapons:2'
	);
}

// Gets player kill history
function db_web_player_kills($uid) {
	$result1 = db_select("
		SELECT ucstats_players.UID,
			   name,
			   kills1 as kills,
			   kills2 as deaths
		  FROM ucstats_players,
		  	   ucstats_kills
		 WHERE ucstats_players.UID=ucstats_kills.UID2
		   AND UID1='$uid'
		 ORDER
		    BY kills1
		  DESC",
		'db_web_player_kills:1'
	);

	$result2 = db_select("
		SELECT ucstats_players.UID,
			   name,
			   kills2 as kills,
			   kills1 as deaths
		  FROM ucstats_players,
			   ucstats_kills
		 WHERE ucstats_players.UID=ucstats_kills.UID1
		   AND UID2='$uid'
		 ORDER
		    BY kills1
		  DESC",
		'db_web_player_kills:2'
	);
	return array('result1'=>$result1,'result2'=>$result2);
}

// Get player aliases
function db_web_player_alias($uid,$sort,$dir) {
	return db_select("
		SELECT *
		  FROM ucstats_alias
		 WHERE UID='$uid'
		 ORDER
		    BY $sort $dir",
		 'db_web_player_alias'
	);
}

// Get player scores
function db_web_player_scores($uid) {
	return db_select("
		SELECT score_history
		  FROM ucstats_score
		 WHERE UID='$uid'",
		'db_web_player_scores'
	);
}

// Get player actions
function db_web_player_actions($uid) {
	return db_select("
		SELECT *
		  FROM ucstats_actions
		 WHERE UID='$uid'
		 ORDER
		    BY action_num
		  DESC",
		'db_web_player_actions'
	);
}

// Get player actions
function db_web_player_stats($uid,$sort,$dir) {
	return db_select("
		SELECT map,
			   kills,
			   deaths,
			   (kills/deaths) as kpd,
			   (kills/playing_time*60) as kpm,
			   killstreak,
			   rounds_played,
			   rounds_survived,
			   (rounds_survived/rounds_played) as rounds_percent,
			   playing_time
		  FROM ucstats_playerstats
		 WHERE UID='$uid'
		 ORDER
		    BY $sort $dir",
		'db_web_player_stats'
	);
}

// Get player hits
function db_web_player_hits($uid) {
	return db_select("
		SELECT weapon_name,
			   (hits_generic+hits_head+hits_chest+hits_stomach
			   +hits_leftarm+hits_rightarm+hits_leftleg+hits_rightleg) as hits_sum
		  FROM ucstats_hits
		 WHERE UID='$uid'
		   AND (hits_generic+hits_head+hits_chest+hits_stomach
		   	   +hits_leftarm+hits_rightarm+hits_leftleg+hits_rightleg) > 0
		 ORDER
		    BY hits_sum
		  DESC",
		'db_web_player_hits'
	);
}

// Get player weapon hits
function db_web_player_weapon_hits($uid,$weapon) {
	if (strlen($weapon)) {
		return db_select("
			SELECT *
			  FROM ucstats_hits
			 WHERE UID='$uid'
			   AND weapon_name='$weapon'",
			'db_web_player_hits:'.$weapon
		);	
	}
	else {
		return db_select("
			SELECT SUM(hits_generic) as hits_generic,
			       SUM(hits_head) as hits_head,
				   SUM(hits_chest) as hits_chest,
				   SUM(hits_stomach) as hits_stomach,
			       SUM(hits_leftarm) as hits_leftarm,
				   SUM(hits_rightarm) as hits_rightarm,
				   SUM(hits_leftleg) as hits_leftleg,
				   SUM(hits_rightleg) as hits_rightleg
			  FROM ucstats_hits
			 WHERE UID='$uid'",
			'db_web_player_hits'
		);	
	}
}

// Get awards for a player
function db_web_player_awards($game,$uid) {
	return db_select("
		SELECT *
		  FROM ucstats_awards
		 WHERE GAME='".addslashes($game)."'
		   AND UID='".addslashes($uid)."'
		 ORDER
		    BY award_year DESC,
			   award_month DESC",
		'db_web_awards_player'
	);
}

// Get list of servers
function db_web_servers($game) {
	$where = '';
	if (!is_null($game)) {
		$where = "WHERE GAME='".addslashes($game)."'";
	}
	return db_select("
		SELECT *
		  FROM ucstats_servers
		$where
		 ORDER
		    BY server_num
		   ASC",
		'db_web_servers'
	);
}

// Get server statistics
function db_web_server_stats($game,$sid) {
	$and = '';
	if (strlen($sid) > 0 && $sid != 'all') {
		$and = "AND ucstats_servers.SID='".addslashes($sid)."'";
	}
	return db_select("
		SELECT server_date,
		       SUM(server_slots) as server_slots,
		       SUM(server_connects) as server_connects
		  FROM ucstats_servers, ucstats_serverstats
		 WHERE ucstats_servers.GAME='".addslashes($game)."' $and
		   AND ucstats_servers.SID=ucstats_serverstats.SID
		 GROUP
		    BY server_date",
		'db_web_server_stats'
	);
}

// Get maps statistics
function db_web_map_stats($game,$sid,$sort,$dir) {
	$result = db_select("
		SELECT SUM(map_time) as map_time
		  FROM ucstats_servers,ucstats_maps
		 WHERE ucstats_servers.GAME = '".addslashes($game)."'
                   AND ucstats_servers.SID = ucstats_maps.SID",
		'db_web_map_stats:1'
	);
	$row = mysql_fetch_array($result,MYSQL_ASSOC);
	$total = (int)$row['map_time'];
	
	$where = '';
	if ($sid > 0) {
		$where = "WHERE SID='".addslashes($sid)."'";
	}
	return db_select("
		SELECT map_name,
			   map_kills,
			   map_actions,
			   (map_wins_team1+map_wins_team2) as map_rounds,
			   map_wins_team1,
			   map_wins_team2,
			   map_connects,
			   (map_wins_team1/(map_wins_team1+map_wins_team2)) as map_percent_team1,
			   (map_wins_team2/(map_wins_team1+map_wins_team2)) as map_percent_team2,
			   map_time,
			   (map_time/$total) as map_percent
		  FROM ucstats_servers,ucstats_maps
		 WHERE ucstats_servers.GAME = '".addslashes($game)."'
		   AND ucstats_servers.SID = ucstats_maps.SID
		 GROUP
		    BY map_name
		 ORDER
		    BY $sort $dir",
		'db_web_map_stats'
	);
}

// Get weapon statistics
function db_web_weapon_stats($game,$sort,$dir) {
	$result = db_select("
		SELECT (SUM(weapon_kills)+SUM(weapon_deaths)) as sum_weapon_usage
		  FROM ucstats_players, ucstats_weapons
		 WHERE ucstats_players.GAME='".addslashes($game)."'
		   AND ucstats_players.UID=ucstats_weapons.UID",
		'db_web_weapon_stats:1'
	);
	$row = mysql_fetch_array($result);
	if ($row['sum_weapon_usage'] > 0) {
		return db_select("
			SELECT weapon_name,
				   SUM(weapon_kills) as sum_kills,
				   SUM(weapon_deaths) as sum_deaths,
				   ((SUM(weapon_kills)+SUM(weapon_deaths))/".$row['sum_weapon_usage'].") as sum_usage,
				   SUM(weapon_shots) as sum_shots,
				   SUM(weapon_hits) as sum_hits,
				   (SUM(weapon_hits)/SUM(weapon_shots)) as sum_accuracy,
				   SUM(weapon_damage) as sum_damage,
				   (SUM(weapon_damage)/SUM(weapon_kills)) as avg_damage,
				   SUM(weapon_headshots) as sum_headshots
                 	  FROM ucstats_players, ucstats_weapons
        	         WHERE ucstats_players.GAME='".addslashes($game)."'
	                   AND ucstats_players.UID=ucstats_weapons.UID
			 GROUP
			    BY weapon_name
			 ORDER
			    BY $sort $dir",
			'db_web_weapon_stats:2'
		);
	}
}

// Get live statistics
function db_web_live_stats($owner) {
	return db_select("
		SELECT *
		  FROM ucstats_keys
		 WHERE key_owner='$owner'",
		'db_web_live_stats'
	);
}

// Returns awards for a given date
function db_web_awards($game,$month,$year) {
	return db_select("
		SELECT *
		  FROM ucstats_awards
		 WHERE GAME='".addslashes($game)."'
		   AND award_month=$month
		   AND award_year=$year
		 ORDER
		    BY award_name",
		'db_web_awards'
	);
}

// Returns number of awards won for all players for a given date
function db_web_awards_num($game,$month,$year) {
	return db_select("
		SELECT UID,
			   name,
			   id,
			   count(*) as num
		  FROM ucstats_awards
		 WHERE GAME='".addslashes($game)."'
		   AND award_month=$month
		   AND award_year=$year
		 GROUP
		    BY name,id
		 ORDER
		    BY num
		  DESC",
		'db_web_awards_num'
	);
}

// Returns total number of awards won for all players
function db_web_awards_total($game) {
	return db_select("
		SELECT UID,
			   count(*) as award_count
		  FROM ucstats_awards
		 WHERE GAME='".addslashes($game)."'
		 GROUP
		    BY UID",
		'db_web_awards_total'
	);
}

// Returns all the years of awards
function db_web_awards_years($game) {
	return db_select("
		SELECT
	  DISTINCT award_year
		  FROM ucstats_awards
		 WHERE GAME='".addslashes($game)."'
		 ORDER
		    BY award_year
		  DESC",
		'db_web_awards_years'
	);
}

// Returns all the months of awards for a year
function db_web_awards_months($game,$year) {
	return db_select("
		SELECT
	  DISTINCT award_month,
	  		   award_year
		  FROM ucstats_awards
		 WHERE GAME='".addslashes($game)."'
		   AND award_year=$year
		 ORDER
		    BY award_month
		  DESC",
		'db_web_awards_months'
	);
}

// Checks if login is correct
function db_web_login($login,$password) {
	$result = db_select("
		SELECT count(*)
		  FROM ucstats_users
		 WHERE user_login='".addslashes($login)."'
		   AND user_password='".addslashes($password)."'",
		'db_web_login'
	);
	if (!is_null($result)) {
		$row = mysql_fetch_row($result);
		return $row[0];
	}
}


/*****************************************************************
* Admin queries
*****************************************************************/

// Gets configs
function db_get_config($owner,$class,$name,$type) {
	$where = '';
	if (strlen($owner) > 0) {
		$where .= "WHERE config_owner='".addslashes($owner)."'";
	}
	if (strlen($class) > 0) {
		$where .= " AND config_class='".addslashes($class)."'";
	}
	if (strlen($name) > 0) {
		$where .= " AND config_name='".addslashes($name)."'";
	}
	if (strlen($type) > 0) {
		$where .= " AND config_type = '$type'";
	}
	return db_select("
		SELECT *
		  FROM ucstats_configs
		$where
	     ORDER
		    BY config_owner",
		'db_get_config'
	);
}

// Updates configs
function db_update_config($owner,$class,$name,$type,$data) {
	if (strlen($owner) > 0) {
		$where = "WHERE config_owner='".addslashes($owner)."'";

		if (strlen($class) > 0) {
			$where .= " AND config_class='".addslashes($class)."'";
		}
		if (strlen($name) > 0) {
			$where .= " AND config_name='".addslashes($name)."'";
		}
		if (strlen($type) > 0) {
			$where .= " AND config_type = '$type'";
		}

		return db_update("
			UPDATE ucstats_configs
			   SET ".get_set_string($data)."
			$where",
			'db_update_config'
		);
	}
}

// Update users
function db_update_user($login,$data) {
	return db_update("
		UPDATE ucstats_users
		   SET ".get_set_string($data)."
		 WHERE user_login='".addslashes($login)."'",
		'db_update_user'
	);
}

// Gets number of records in a table
function db_get_count($table) {
	$result = db_select("
		SELECT count(*)
		  FROM $table",
		'db_get_count'
	);
	if (!is_null($result)) {
		$row = mysql_fetch_row($result);
		return $row[0];
	}
}

// Add a new server to the registry
function db_new_server($game,$sid,$num,$address,$name,$ps_enabled,$ss_enabled,$lf_enabled) {
	if (is_valid_address($address)) {
		$changes = 0;
		$changes += db_insert("
			INSERT INTO ucstats_servers (
				GAME,
				SID,
				server_num,
				server_address,
				server_name,
				ps_enabled,
				ss_enabled,
				lf_enabled
			)
			VALUES (
				'$game',
				'$sid',
				$num,
				'$address',
				'$name',
				'$ps_enabled',
				'$ss_enabled',
				'$lf_enabled'
			)",
			'db_new_server'
		);
		
		$sid = db_get_server_id($address);
		
		$changes += db_init_server_stats($sid);
		
		return $changes;
	}
}

// Removes a server from the registry
function db_remove_server($sid) {
	$changes = 0;
	$changes += db_delete("
		DELETE
		  FROM ucstats_servers
		 WHERE SID='$sid'",
		'db_remove_server:1'
	);
	$changes += db_delete("
		DELETE
		  FROM ucstats_serverstats
		 WHERE SID='$sid'",
		'db_remove_server:2'
	);
	$changes += db_delete("
		DELETE
		  FROM ucstats_maps
		 WHERE SID='$sid'",
		'db_remove_server:3'
	);
	
	// Re-index the servers
	$result = db_select("
		SELECT *
		  FROM ucstats_servers
		 ORDER
		    BY server_num
		   ASC",
		'db_remove_server:4'
	);
	if (!is_null($result)) {
		$i = 1;
		while($row = mysql_fetch_array($result)) {
			if ($row['server_num'] != $i) {
				$changes += db_update("
					UPDATE ucstats_servers
					   SET server_num=$i
					 WHERE server_num=".$row['server_num'],
					'db_remove_server:5'
				);
			}
			$i++;
		}
	}
}

// Gets the server id by address
function db_get_server_id($address) {
	$result = db_select("
		SELECT SID
		  FROM ucstats_servers
		 WHERE server_address='".addslashes($address)."'",
		'db_get_server'
	);
	if (!is_null($result)) {
		$row = mysql_fetch_row($result);
		return $row[0];
	}
	else {
		return null;
	}
}

// Gets the number of servers
function db_get_server_count() {
	$result = db_select("
		SELECT count(*)
		  FROM ucstats_servers",
		'db_get_server_count'
	);
	$row = mysql_fetch_row($result);
	return $row[0];
}


// Update server
function db_update_server($game,$sid,$address,$name,$ps_enabled,$ss_enabled,$lf_enabled) {
	return db_update("
		UPDATE ucstats_servers
		   SET GAME='$game',
		   	   server_address='$address',
		   	   server_name='$name',
			   ps_enabled='$ps_enabled',
			   ss_enabled='$ss_enabled',
			   lf_enabled='$lf_enabled'
		 WHERE SID='$sid'",
		'db_update_server'
	);
}

// Change a server position
function db_move_server($num,$action) {
	$changes = 0;
	if ($action == "move_up") {
		$switch = $num-1;
	}
	else if ($action == "move_down") {
		$switch = $num+1;
	}
	$temp = -1 * time();
	$changes += db_update("
		UPDATE ucstats_servers
		   SET server_num=$temp
		 WHERE server_num=$switch",
		 'db_move_server:servers:1'
	);
	$changes += db_update("
		UPDATE ucstats_servers
		   SET server_num=$switch
		 WHERE server_num=$num",
		 'db_move_server:servers:2'
	);
	$changes += db_update("
		UPDATE ucstats_servers
		   SET server_num=$num
		 WHERE server_num=$temp",
		 'db_move_server:servers:3'
	);
	return $changes;
}

// Initialize server stats
function db_init_server_stats($sid) {
	$changes = 0;
	for ($day = 1; $day < 8; $day++) {
		for ($hour = 0; $hour < 24; $hour++) {
			for ($min = 0; $min < 60; $min += 5) {
				$changes += db_insert("
					INSERT INTO ucstats_serverstats (
						SID,
						server_date,
						server_slots,
						server_connects
					)
					VALUES (
						'$sid',
						".(int)($day.(($hour < 10)?('0'.$hour):$hour).(($min < 10)?('0'.$min):$min)).",
						0,
						0
					)",
					'db_init_server_stats:insert'
				);
			}
		}
	}
	return $changes;
}

// Deletes all award records for a date
function db_reset_awards($game,$month,$year) {
	return db_delete("
		DELETE
		  FROM ucstats_awards
		 WHERE award_month=$month
		   AND award_year=$year
		   AND GAME='".addslashes($game)."'",
		'db_reset_awards'
	);
}

// Delete all inactive player records
function db_clean($game) {
	$changes = 0;
	$where = '';
	if (strlen($game)) {
		$where = "AND ucstats_players.GAME='".addslashes($game)."'";
	}
        $result = db_select("
                SELECT UID
                  FROM ucstats_players
		 WHERE online < 60
		    OR (kills = 0 AND deaths = 0)
		$where",
                'db_clean'
        );
        if (!is_null($result)) {
                while($row = mysql_fetch_array($result)) {
			$changes += db_delete_player($row['UID']);
                }
        }
	return $changes;
}

// Resets the database
function db_reset($game) {
	$where = '';
	if (strlen($game) > 0) {
		$where = "AND ucstats_players.GAME='".addslashes($game)."'";
	}
	
	// Tables to be reset
	$index = array(
		'ucstats_playerstats',
		'ucstats_weapons',
		'ucstats_hits',
		'ucstats_actions',
		'ucstats_score',
		'ucstats_alias'
	);
	
	$changes = 0;
	foreach($index as $value) {
		$changes += db_delete("
			DELETE 
			  FROM $value
			 USING ucstats_players, $value
			WHERE ucstats_players.UID=$value.UID $where",
			'db_reset:'.$value
		);
	}
	
	$changes += db_delete("
		DELETE 
		  FROM ucstats_kills
		 USING ucstats_players, ucstats_kills
		WHERE ucstats_players.UID=ucstats_kills.UID1 $where",
		'db_reset:kills1'
	);

	$changes += db_delete("
		DELETE 
		  FROM ucstats_kills
		 USING ucstats_players, ucstats_kills
		WHERE ucstats_players.UID=ucstats_kills.UID2 $where",
		'db_reset:kills2'
	);
	
	$changes += db_delete("
		DELETE 
		  FROM ucstats_players
		 WHERE GAME='$game'",
		'db_reset:players'
	);
	
	$changes += db_delete("
		DELETE 
		  FROM ucstats_maps
		 USING ucstats_servers, ucstats_maps
		 WHERE ucstats_servers.GAME='".addslashes($game)."'
		   AND ucstats_maps.SID=ucstats_servers.SID",
		   'db_reset:maps'
	);
	return $changes;
}

// Clears the database
function db_clear($game) {
	$where = '';
	if (strlen($game)) {
		$where = "AND ucstats_servers.GAME='".addslashes($game)."'";
	}
	$changes = 0;
	$changes += db_reset($game);
	$changes += db_delete("
		DELETE 
		  FROM ucstats_servers,
		  	   ucstats_serverstats
		 USING ucstats_servers,
			   ucstats_serverstats
		 WHERE ucstats_servers.SID=ucstats_serverstats.SID $where",
		 'db_clear:servers'
	);
	$where = '';
	if (strlen($game)) {
		$where = "WHERE GAME='".addslashes($game)."'";
	}
	$changes += db_delete("DELETE FROM ucstats_awards $where",'db_clear:awards');
	return $changes;
}

// Delete a player
function db_delete_player($uid) {
	$changes = 0;
	$changes += db_delete("DELETE FROM ucstats_players WHERE UID='".addslashes($uid)."'",'db_delete_player:players');
	$changes += db_delete("DELETE FROM ucstats_weapons WHERE UID='".addslashes($uid)."'",'db_delete_player:weapons');
	$changes += db_delete("DELETE FROM ucstats_hits WHERE UID='".addslashes($uid)."'",'db_delete_player:hits');
	$changes += db_delete("DELETE FROM ucstats_actions WHERE UID='".addslashes($uid)."'",'db_delete_player:actions');
	$changes += db_delete("DELETE FROM ucstats_alias WHERE UID='".addslashes($uid)."'",'db_delete_player:alias');
	$changes += db_delete("DELETE FROM ucstats_playerstats WHERE UID='".addslashes($uid)."'",'db_delete_player:playerstats');
	$changes += db_delete("DELETE FROM ucstats_score WHERE UID='".addslashes($uid)."'",'db_delete_player:score');
	$changes += db_delete("DELETE FROM ucstats_kills WHERE UID1='".addslashes($uid)."' AND kills2 > 0",'db_delete_player:players');
	$changes += db_delete("DELETE FROM ucstats_kills WHERE UID2='".addslashes($uid)."' AND kills1 > 0",'db_delete_player:players');
	return $changes;
}

// Find and delete a player
function db_find_delete_player($game,$id,$type) {
	$changes = 0;
	$where = '';
	if ($type == 'uid') {
		return db_delete_player($id);
	}
	else if ($type == 'name') {
		$where = "AND name='".addslashes($id)."'";
	}
	else if ($type == 'id') {
		$where = "AND id='".addslashes($id)."'";
	}
	$result = db_select("
		SELECT UID
		  FROM ucstats_players
		 WHERE GAME='".addslashes($game)."' $where",
		'db_delete_player'
	);
	if (!is_null($result)) {
		while ($row = mysql_fetch_array($result,MYSQL_ASSOC)) {
			$changes += db_delete_player($row['UID']);
		}
	}
	return $changes;
}
?>
