<?php
/*****************************************************************
* UCStats - A stats generation program for Half-Life games.
* Copyright (c) 2003 Mike Cao <mike@mikecao.com>
*****************************************************************/

/*
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/*****************************************************************
* Get Functions
*****************************************************************/

// Gets a unique id based on configs
function get_unique_id($name,$id) {
	global $GAME;
	// ID only
	if (CFG_STATS_MODE == 2) {
		return md5($GAME.$id.STATS_SEED);
	}
	// Name only
	else if (CFG_STATS_MODE == 1) {
		return md5($GAME.$name.STATS_SEED);
	}
	// Name and ID
	else {
		return md5($GAME.$name.$id.STATS_SEED);
	}
}

// Gets the name of a mod
function get_mod_name($mod) {
	global $GAME_mods;
	if (isset($GAME_mods[$mod])) {
		return $GAME_mods[$mod];
	}
	return $mod;
}

// Gets clan name from a given player name
function get_clan_name($name) {
	if (CFG_STATS_MODE == 2) return '';
	$args = array();
	if (preg_match("/^([^\w]{1,3}[\w\s]{1,10}[^\w]{1,3})/",$name,$args)) {}
	else if (preg_match("/([^\w]{1,3}[\w\s]{1,10}[^\w]{1,3})$/",$name,$args)) {}
	else if (preg_match("/^([\w]{1,10}[\s]{1,2}[^\w]{1,3})/",$name,$args)) {}
	else if (preg_match("/^([\w]{1,10}[~`!@#$%^&*\(\)_+-=\[\]\{\}\|\/\\\]{1,3})/",$name,$args)) {}
	if (sizeof($args)) {
		return trim($args[1]);
	}
	return '';
}

// Gets a string based on tokens
function get_token_string($str,$tokens) {
	if (sizeof($tokens)) {
		for ($i = 0; $i < sizeof($tokens); $i++) {
			$str = str_replace('{'.$i.'}',$tokens[$i],$str);
		}
	}
	return $str;
}

// Gets the set string for an update query
function get_set_string($args) {
	$set = '';
	if (sizeof($args)) {
		$i = 0;
		foreach ($args as $key => $value) {
			if (is_string($value) && strlen($value)) $set .= "$key='".addslashes($value)."'";
			else $set .= $key.'='.((is_null($value))?'NULL':$value);
			if ($i < sizeof($args)-1) { $set .= ','; $i++; }
		}
	}
	return $set;
}

// Get last update
function get_last_update() {
	$result = db_get_key('STATS','TIMESTAMP');
	if (!is_null($result)) {
		$row = mysql_fetch_row($result);
		if ($row) {
			return date('l m/d/Y - g:i a',$row[0]);
		}
	}
}

// Get time online
function get_time_online($time) {
	if($time > 3600) {
		if ($time % 3600) {
			return round($time/3600)." h ".(int)(($time % 3600)/60)." min";
		} else {
			return round($time/3600)." h";
		}
	} else {
		return round($time/60)." min";
	}
}

// Get time in microseconds
function get_time_micro() {
	list($usec, $sec) = explode(' ',microtime()); 
    return ((float)$usec + (float)$sec); 
}

// Get weapon points modifier
function get_weapon_mod($weapon) {
	global $GAME_weapons;
	if (isset($GAME_weapons[$weapon]['mod'])) {
		return $GAME_weapons[$weapon]['mod'];
	}
	else {
		return 0;
	}
}

// Get action points modifier
function get_action_mod($action) {
	global $GAME_actions;
	if (isset($GAME_actions[$action]['mod'])) {
		return $GAME_actions[$action]['mod'];
	}
	return 0;
}

// Gets weapon name
function get_weapon_name($weapon) {
	global $GAME_weapons;
	if (isset($GAME_weapons[$weapon]['name'])) {
		return $GAME_weapons[$weapon]['name'];
	}
	return $weapon;
}

// Get action name
function get_action_name($action) {
	global $GAME_actions;
	if (isset($GAME_actions[$action]['name'])) {
		return $GAME_actions[$action]['name'];
	}
	return $action;
}

// Gets misc category name
function get_misc_name($action) {
	global $GAME_misc;
	if (isset($GAME_misc[$action]['name'])) {
		return $GAME_misc[$action]['name'];
	}
	return $action;
}

// Get award points
function get_award_points($award) {
	if (is_numeric($award)) {
		if ($award == 1) {
			return 1000;
		}
		else if ($award == 2) {
			return 500;
		}
		else if ($award == 3) {
			return 200;
		}
		else {
			return 100;
		}
	}
	else if (is_valid_award($award)) {
		return 100;
	}
}

// Get proper number format for actions
function get_action_format($action,$num) {
	if ($action == 'online') {
		return get_time_online($num);
	}
	if ($action == 'survivor' || $action =='best_shot' || $action == 'worst_shot') {
		return number_format((int)($num*100),0,'',',').'%';
	}
	else if (!strlen(strstr($num,'.')) || $action == 'points') {
		return number_format((int)$num,0,'',',');
	}
	else {
		return number_format($num,2,'.','');
	}
}

// Gets score event description
function get_score_event($game,$event,$name,$id,$weapon) {
	global $GAME_actions;
	if ($event == 'kill') {
		return get_token_string(L_KILLED_PLAYER, array('<a href="'.PHP_SELF.'?game='.$game.'&q=player&uid='.get_unique_id($name,$id).'">'.htmlspecialchars($name).'</a>',$weapon));
	}
	else if ($event == 'death') {
		return get_token_string(L_DIED_FROM_PLAYER, array('<a href="'.PHP_SELF.'?game='.$game.'&q=player&uid='.get_unique_id($name,$id).'">'.htmlspecialchars($name).'</a>',$weapon));
	}
	else if ($event == 'action' && is_valid_action($name)) {
		return $GAME_actions[$name]['name']." ($id).";
	}
	else if ($event == 'tk') {
		return get_token_string(L_KILLED_TEAMMATE, array('<a href="'.PHP_SELF.'?game='.$game.'&q=player&uid='.get_unique_id($name,$id).'">'.htmlspecialchars($name).'</a>',$weapon));
	}
	else if ($event == 'start') {
		return L_STARTING_SCORE.'.';
	}
}

// Returns a multi-dimensional array sorted by key
function get_sorted_array($arr,$key) {
	$sorted = array();
	foreach ($arr as $row) {
		$sorted[] = $row[$key];
	}
	array_multisort($sorted,$arr);
	return array_reverse($arr);
}

// Gets award name
function get_award_name($award) {
	if (is_valid_weapon($award)) {
		return get_token_string(L_MOST_WEAPON_KILLS, array(get_weapon_name($award)));
	}
	else if (is_numeric($award)) {
		if ($award <= 10) {
			if ($award == 1) {
				return L_BEST_PLAYER;
			}
			else if ($award == 2) {
				return L_TOP_TEN_2ND;
			}
			else if ($award == 3) {
				return L_TOP_TEN_3RD;
			}
			else {
				return get_token_string(L_TOP_TEN_NTH, array($award));
			}
		}
	}
	else if (is_valid_award($award)) {
		return get_misc_name($award);
	}
}

// Get the last modified log file from a directory
function get_last_mod_file($src) {
	$lastmodtime = 0;
	$lastmodfile = '';

	$dir = opendir($src);
	while (($file = readdir($dir)) != false) {
		if ($file != '.' && $file != '..' && is_file($src.'/'.$file)) {
			if (filemtime($src.'/'.$file) > $lastmodtime) {
				$lastmodfile = $file;
				$lastmodtime = filemtime($src.'/'.$file);
			}
			else if (filemtime($src.'/'.$file) == $lastmodtime) {
				if (filesize($src.'/'.$file) <  filesize($src.'/'.$lastmodfile)) {
					$lastmodfile = $file;
					$lastmodtime = filemtime($src.'/'.$file);
				}
			}
		}
	}
	closedir($dir);
	
	return $lastmodfile;
}

// Creates a display for page links
function get_page_links($url,$rows,$max,$page) {
	$str = '';
	if ($rows) {
		// Calculate pages
		$pages = (int)($rows/$max);
		if($rows % $max) {
			$pages++;
		}
		
		$start = 1;
		$div = 10;
		$end = $div;
		if ($page > $div) {
			while (!($page >= $start && $page <= $end)) {
				$start += $div;
				$end += $div;
			}
		}
		if($end > $pages) {
			$end = $pages;
		}
		
		// Get page links and drop-down box if needed
		$str .= '<form name="pageform">';
		if ($start > $div) {
			$str .= L_PAGES.': '.get_hyperlink('&laquo;',$url.'&page='.($start-$div));
		} else {
			$str .= L_PAGES.': ';
		}
		for ($i = $start; $i <= $end; $i++) {
			if (!$i) {
				$str .= '&nbsp;';
			} else if ($i != $page && $page) {
				$str .= '&nbsp;'.get_hyperlink($i,$url.'&page='.$i);
			} else {
				$str .= '&nbsp;'.$i;
			}
		}
		if ($end < $pages) {
			$str .= '&nbsp;'.get_hyperlink('&raquo;',$url.'&page='.($start+$div));
		}
		if ($pages > $div) {
			$str .= '&nbsp;&nbsp;-&nbsp;&nbsp;'.L_PAGE_SET.':</b>&nbsp;&nbsp;
			<select name="pageselect" onchange="goUrl(\''.$url.'&page=\'+document.pageform.pageselect.options[document.pageform.pageselect.selectedIndex].value)">
			<option>select</option>';
			for ($i = 0; $i <= $pages; $i++) {
				if ($i && !($i % $div)) {
					$str .= '<option value="'.($i-$div+1).'">'.($i-$div+1).' - '.($i).'</option>'."\n";
					if (($pages-$i) && ($pages-$i) < $div) {
						$str .= '<option value="'.($i+1).'">'.($i+1).'+</option>'."\n";
					}
				}
			}
			$str .= '</select></form>';
		}
	}
	return $str;
}

// Creates an image url
function get_image($src,$name,$height,$width) {
	if (!$height && !$width && file_exists(WEB_PATH.'/'.$src)) {
		$size = getimagesize($src);
		$height = $size[1];
		$width = $size[0];
	}
	return "<img src=\"$src\" name=\"$name\" width=\"$width\" height=\"$height\" border=\"0\" alt=\"\">";
}

// Creates a hyperlink
function get_hyperlink($text,$href) {
	return "<a href=\"$href\">$text</a>";
}

// Gets sorting arrows
function get_sort_arrows(&$Doc,$keys,$sort,$dir,$script,$bookmark) {
	$arrows = array();
	foreach($keys as $key) {
		$arrows['sort_url_'.$key] = $script.'&sort='.$key.'&dir='.$dir.$bookmark;
		$arrows['sort_img_'.$key] = constant(strtoupper('IMG_SORT_'.$dir.'_'.(($sort==$key)?'on':'off')));
	}

	// Highlighted arrow
	$arrows['sort_url_'.$sort] = $script.'&sort='.$sort.'&dir='.(($dir=='desc')?'asc':'desc').$bookmark;

	foreach($arrows as $key => $value) {
		$Doc->setDocumentData($key,$value);
	}
}

// Gets the default sort direction
function get_sort_default($key,$sort,$dir,$script) {
	return get_hyperlink(get_image('images/icons/arrow_'.$dir.'_'.(($sort==$key)?'on':'off').'.gif','',0,0),
	$script.'&sort='.$key.'&dir='.$dir);
}

// Gets the set sort direction
function get_sort_set($sort,$dir,$script) {
	return get_hyperlink(get_image('images/icons/arrow_'.(($dir=='desc')?'desc':'asc').'_on.gif','',0,0),
	$script.'&sort='.$sort.'&dir='.(($dir=='desc')?'asc':'desc'));
}

// Gets the proper color for a number
function get_number_color($num) {
	if ($num >= 0) {
		return '<span class="positive">+'.$num.'</span>';
	}
	else {
		return '<span class="negative">'.$num.'</span>';
	}
}

// Gets color coded yes/no
function get_yesno_color($num) {
	if ($num) {
		return '<span class="positive">'.L_YES.'</span>';
	}
	else {
		return '<span class="negative">'.L_NO.'</span>';
	}
}

// Gets the proper color for a given team
function get_team_color($name,$team) {
	if ($team == 1) {
		return '<span class="team1">'.htmlspecialchars($name).'</span>';
	}
	else if ($team == 2) {
		return '<span class="team2">'.htmlspecialchars($name).'</span>';
	}
	else if ($team == 0) {
		return '<span class="webchat">'.htmlspecialchars($name).'</span>';
	}
	return $name;
}

// Gets the stats mode text
function get_stats_mode($mode) {
	if ($mode == 0) {
		return L_NAME_ID;
	}
	else if ($mode == 1) {
		return L_NAME_ONLY;
	}
	else {
		return L_ID_ONLY;
	}
	return $mode;
}

// Gets the color set by the template config
function get_config_color(&$img,$conf) {
	$colors = explode(',',constant($conf));
	return imagecolorallocate($img,$colors[0],$colors[1],$colors[2]);
}

// Gets a list of all the folders in a directory
function get_folders($path) {
	$dir = opendir($path);
	$folders = array();
	while(($file = readdir($dir)) != false) {
		if (is_dir($path.'/'.$file) && $file != "." && $file != "..") {
			array_push(
				$folders,
				array('name'=>$file)
			);
		}
	}
	closedir($dir);
	return $folders;
}

// Gets a list of all files in a directory
function get_files($path,$strip) {
	$dir = opendir($path);
	$files = array();
	while(($file = readdir($dir)) != false) {
		if (is_file($path.'/'.$file) && $file != "." && $file != "..") {
			array_push(
				$files,
				array('name'=>(($strip)?(substr($file,0,strpos($file,"."))):$file))
			);
		}
	}
	closedir($dir);
	return $files;
}

// Gets a process id number
function get_pid($file) {
	clearstatcache();
	if (file_exists($file)) {
		return (int)trim(implode('',file($file)));
	}
	else {
		return -1;
	}
}

// Gets an icon based on number of awards
function get_award_icon($num) {
	if ($num > 0) {
		$awards = array(20,10,5,1);
		$str = '';
		foreach($awards as $value) {
			if (floor($num/$value)) {
				for ($i = 0; $i < floor($num/$value); $i++) {
					$str .= constant('IMG_AWARDS_'.$value);
				}
				$num -= floor($num/$value)*$value;
			}
		}
		return $str;
	}
	else {
		return '';
	}
}
?>
