<?php
/*****************************************************************
* UCStats - A stats generation program for Half-Life games.
* Copyright (c) 2003 Mike Cao <mike@mikecao.com>
*****************************************************************/

/*
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/*****************************************************************
* Task functions
*****************************************************************/

// Copies a directory to another location
function task_dir_copy($from_path,$to_path) {
	$this_path = getcwd();
	if (!is_dir($to_path)) {
		mkdir($to_path, 0775);
	}
	if (is_dir($from_path)) {
		chdir($from_path);
		$handle=opendir('.');
		while (($file = readdir($handle))!==false) {
			if (($file != '.') && ($file != '..')) {
				if (is_dir($file)) {
					chdir($this_path);
					task_dir_copy($from_path.$file.'/', $to_path.$file.'/');
					chdir($this_path);
					chdir($from_path);
				}
				if (is_file($file)) {
					chdir($this_path);
					copy($from_path.$file, $to_path.$file);
					chdir($from_path);
				}
			}
		}
		closedir($handle); 
	}
	chdir($this_path);
} 


// Deletes all files in a directory older than a certain modtime
function task_dir_cleanup($src,$modtime) {
	$count = 0;
	$dir = opendir($src);
	if ($dir) {
		while(($file = readdir($dir)) != false) {
			if (is_file($src.'/'.$file) && $file != '.' && $file != '..') {
				if (filemtime($src.'/'.$file) < $modtime) {
					unlink($src.'/'.$file);
					$count++;
				}
			}
		}
		closedir($dir);
	}
	else {
		log_error(get_token_string(L_ERR_OPEN_DIR, array($src))." (task_cleanup)");
	}
	return $count;
}

// Updates a config file
function task_config_update($file_in,$file_out,$configs) {
	if (file_exists($file_in)) {
		$template = file($file_in);
		for ($i = 0; $i < sizeof($template); $i++) {
			if (preg_match("/^define\('(.*)', (.*)\);$/",trim($template[$i]),$args)) {
				list(,$key,) = $args;
				$template[$i] = str_replace('%'.$key.'%',$configs[$key],$template[$i]);
			}
		}
		$fp = fopen($file_out,'w+');
		flock($fp,2);
		for($i = 0; $i < sizeof($template); $i++) {
			fwrite($fp, $template[$i]);
		}
		fclose($fp);
	}
	else {
		log_error(get_token_string(L_ERR_FILE_INPUT,array($file_in))." (task_config_update)");
	}
}

// Create a PID file
function task_create_pid($file,$pid) {
	$fp = fopen(PROGRAM_PATH."/run/$file.pid",'w+');
	fwrite($fp, $pid);
	fclose($fp);
}
?>
