<?php
/*****************************************************************
* UCStats - A stats generation program for Half-Life games.
* Copyright (c) 2003 Mike Cao <mike@mikecao.com>
*****************************************************************/

/*
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, 
MA 02111-1307, USA.
*/

/*****************************************************************
* Live Statistics
*****************************************************************/

require(PROGRAM_PATH.'/includes/class_socket.php');

// Set the template
$Doc->setTemplate(TEMPLATE_PATH.'/live.tpl');

// Get list of servers
$result = db_web_servers(null);

$sid = 0;

if (!is_null($result)) {

	$Doc->setDocumentData('server_url',PHP_SELF.'?game='.$GAME.'&q='.$_GET['q']);

	$rows = mysql_num_rows($result);
	$servers = array();

	if ($rows) {
		$Doc->setDocumentData('multiple_servers',true);
	}
	
	if (!isset($_GET['show'])) $_GET['show'] = '0';
	
	$first = true;
	while($row = mysql_fetch_array($result)) {
		if ($row['lf_enabled'] == '1' && $row['GAME'] == $GAME) {
			$selected = ($row['SID']==$_GET['show'])?true:false;
			array_push(
				$servers,
				array(
					'SID'=>$row['SID'],
					'server_name'=>$row['server_name'],
					'server_selected'=>($selected)?'selected':''
				)
			);
			if ($first) {
				$sid = $row['SID'];
				$server_name = $row['server_name'];
				$server_address = $row['server_address'];
				$first = false;
			}
			if ($selected && $rows) {
				$sid = $row['SID'];
				$server_name = $row['server_name'];
				$server_address = $row['server_address'];
				$Doc->setDocumentData('server_select',"for $server_name");
			}
		}
	}
	$Doc->setDocumentData('servers',$servers);
}

$_GET['show'] = $sid;
$buffer = array();

// No servers
if (!sizeof($servers)) {
	$Doc->setTemplate(TEMPLATE_PATH.'/error.tpl');
	$Doc->setDocumentData('error_title', L_LIVE_FEED);
	$Doc->setDocumentData('error_msg', L_NO_REG_SERVERS_LF);
}
else if (strlen(CFG_LF_PASSWORD) && strlen(is_valid_address(CFG_LF_ADDRESS))) {
	// If there was a recent request for live data use the logged results.
	$LIVE_CACHE = PROGRAM_PATH.'/run/'.str_replace('.','_',str_replace(':','_',$server_address)).'_live.log';
	
	if (file_exists($LIVE_CACHE) && (time()-filemtime($LIVE_CACHE) < CFG_LF_DELAY)) {
		$buffer = file($LIVE_CACHE);
	}
	// Otherwise request it from the game server
	else if (strlen($server_address) && !is_null($server_address)) {
		$server_ip = trim(substr($server_address,0,strpos($server_address,':')));
		$server_port = trim(substr($server_address,strpos($server_address,':')+1,strlen($server_address)));
		
		$recv_ip = trim(substr(CFG_LF_ADDRESS,0,strpos(CFG_LF_ADDRESS,':')));
		$recv_port = trim(substr(CFG_LF_ADDRESS,strpos(CFG_LF_ADDRESS,':')+1,strlen(CFG_LF_ADDRESS)));
		
		// Connect to game server
		$socket = new Socket();
		$socket->connect($server_ip,$server_port,CFG_LF_PASSWORD);
		$command = "amx_livefeed";
		$socket->sendCommand($command);
		
		// Get data
		if ($socket->isConnected()) {
			sleep(1);
			if (file_exists($LIVE_CACHE)) {
				$buffer = file($LIVE_CACHE);
			}
			// Error handling
			else {
				$Doc->setDocumentData('error_title', L_ERROR);
				$Doc->setDocumentData('error_msg', get_token_string(L_NO_RESPONSE, array($server_name,$server_address)));
			}
		}
		// Error handling
		else {
			$Doc->setDocumentData('error_title', L_ERROR);
			$Doc->setDocumentData('error_msg', get_token_string(L_NO_CONTACT, array($server_name,$server_address)));
		}
		
		// Disconnect from game server
		$socket->disconnect();
	}
}
else {
	$Doc->setDocumentData('error_title', L_ERROR);
	$Doc->setDocumentData('error_msg', L_BAD_LF_CONFIG);
}

if (is_null($Doc->getDocumentData('error_msg'))) {

	// Display stats
	if (sizeof($buffer)) {
		$num_team1 = 0;
		$num_team2 = 0;
		$players_team1 = array();
		$players_team2 = array();
		$players_team3 = array();
		$cvars = '';
		$mapcycle = '';
		
		$totals = array('kills'=>0,'deaths'=>0,'ping'=>0,'online'=>0);
		$totals_team1 = array('kills'=>0,'deaths'=>0,'ping'=>0,'online'=>0);
		$totals_team2 = array('kills'=>0,'deaths'=>0,'ping'=>0,'online'=>0);
	
		foreach ($buffer as $value) {
			$data = explode('"',$value);
			// Player data
			if ($data[0] == 'PLAYER') {
				$player_stats = array(
					'player_name'=>get_hyperlink(htmlspecialchars($data[2]),PHP_SELF.'?game='.$GAME.'&q=player&uid='.get_unique_id($data[2],$data[3])),
					'player_ID'=>get_hyperlink(htmlspecialchars($data[3]),PHP_SELF.'?game='.$GAME.'&q=search&query='.rawurlencode($data[3]).'&qtype=ID'),
					'player_kills'=>$data[4],
					'player_deaths'=>$data[5],
					'player_status'=>($data[7])?'':'DEAD',
					'player_ping'=>$data[8],
					'player_online'=>get_time_online($data[9])
				);
				if ($data[6] == 1) {
					array_push(
						$players_team1,
						$player_stats
					);
					// Totals
					$totals_team1['kills'] += $data[4];
					$totals_team1['deaths'] += $data[5];
					$totals_team1['ping'] += $data[8];
					$totals_team1['online'] += $data[9];
					$num_team1++;
				}
				else if ($data[6] == 2) {
					array_push(
						$players_team2,
						$player_stats
					);
					// Totals
					$totals_team2['kills'] += $data[4];
					$totals_team2['deaths'] += $data[5];
					$totals_team2['ping'] += $data[8];
					$totals_team2['online'] += $data[9];
					$num_team2++;
				}
				else {
					array_push(
						$players_team3,
						$player_stats
					);
				}
				// Totals
				$totals['kills'] += $data[4];
				$totals['deaths'] += $data[5];
				$totals['ping'] += $data[8];
				$totals['online'] += $data[9];
			}
			// Server data
			else if ($data[0] == 'SERVER') {
				$Doc->setDocumentData('server',$data[1]);
				$Doc->setDocumentData('ip',trim(substr($data[2],0,strpos($data[2],':'))));
				$Doc->setDocumentData('port',trim(substr($data[2],strpos($data[2],':')+1,strlen($data[2]))));
				$Doc->setDocumentData('numplayers',$data[3]);
				$Doc->setDocumentData('maxplayers',$data[4]);
				$Doc->setDocumentData('map',$data[5]);
				
				$Doc->setDocumentData('maptime',$data[6]);
				$Doc->setDocumentData('timeleft',($data[7]>0)?($data[7]-1):0);
				$Doc->setDocumentData('connects',$data[8]);
				$Doc->setDocumentData('teamscore1',$data[9]);
				$Doc->setDocumentData('teamscore2',$data[10]);
				
				$_datetime = explode(' - ',$data[11]);
				$_date = explode('/',$_datetime[0]);
				$_time = explode(':',$_datetime[1]);
				
				$Doc->setDocumentData('servertime',date("l m/d/Y - g:i a",mktime($_time[0],$_time[1],$_time[2],$_date[0],$_date[1],$_date[2])));
				
				$map = $data[5];
				$numplayers = $data[3];
			}
			// Cvar
			else if ($data[0] == 'CVAR') {
				$cvars .= $data[1]." ".$data[2]."<br>";
			}
			// Map
			else if ($data[0] == 'MAPCYCLE') {
				for ($i = 1; $i < sizeof($data)-1; $i++) {
					if ($map == $data[$i]) {
						$mapcycle .= '<b>'.$data[$i].'</b><br>';
					}
					else {
						$mapcycle .= $data[$i].'<br>';
					}
				}
			}
		}
		
		$CHAT_CACHE = PROGRAM_PATH.'/run/'.str_replace('.','_',str_replace(':','_',$server_address)).'_chat.log';
		if (file_exists($CHAT_CACHE)) {
			$log = file($CHAT_CACHE);
			$teams = array();
			$events = array();
			
			$start = 0;
			$max = CFG_LF_EVENT_MAX;
			if (sizeof($log) > $max) {
				if (!isset($_GET['showall'])) {
					$start = sizeof($log)-$max;
				}
			}
			for ($i = $start; $i < sizeof($log); $i++) {
				$data = explode('"',$log[$i]);
				$team = '';
				if (sizeof($data)) {
					$str = date("h:i:s a: ",$data[0]);
					if ($data[1] == 'CHAT') {
						if ($data[2]=='say_team') {
							$team = '(TEAM) ';
						}
						else if ($data[2]=='WEB') {
							$team = '(WEB) ';
						}
						$str .= '<b>'.get_team_color($team.$data[3],$data[4]).': '.htmlspecialchars($data[5]).'</b>';
					}
					else if ($data[1] == 'EVENT') {
						if ($data[2] == 'KILL') {
							$str .= '<span class="kill">* '.htmlspecialchars($data[3]).' killed '.htmlspecialchars($data[5]).' with '.str_replace('weapon_','',$data[7]).(($data[8]==1)?' (headshot)':'').'</span>';
						}
						else if ($data[2] == 'ACTION') {
							$str .= '<span class="action">* '.htmlspecialchars($data[3]).' '.str_replace('_',' ',strtolower($data[5])).'</span>';
						}
						else if ($data[2] == 'WORLD') {
							$str .= '<span class="world">* '.str_replace('_',' ',$data[3]).'</span>';
						}
						else if ($data[2] == 'JOINED') {
							$str .= '<span class="event">*** '.htmlspecialchars($data[3]).' ('.$data[4].') has joined the game ***</span>';
						}
						else if ($data[2] == 'LEFT') {
							$str .= '<span class="event">*** '.htmlspecialchars($data[3]).' has left the game ***</span>';
						}
						else if ($data[2] == 'NAMECHANGE') {
							$str .= '<span class="action">* '.htmlspecialchars($data[3]).' changed name to '.htmlspecialchars($data[4]).'</span>';
						}
						else {
							$str = '';
						}
					}
					if (strlen($str)) $events[] = $str;
				}
			}
			if ($start != 0 && sizeof($log) > $max) {
				$events[] = '<br>Showing last '.CFG_LF_EVENT_MAX.' events';
			}
			$Doc->setDocumentData('event_log',implode('<br>',$events));
		}

		$Doc->setDocumentData('no_error_msg',true);
		
		$Doc->setDocumentData('teamname1',$GAME_teams['team1']['name']);
		$Doc->setDocumentData('teamname2',$GAME_teams['team2']['name']);
		$Doc->setDocumentData('teamname3',$GAME_teams['team3']['name']);

		$Doc->setDocumentData('map_image',get_image("images/$GAME/maps/$map.jpg",$map,0,0));
		$Doc->setDocumentData('players_team1',get_sorted_array($players_team1,'player_kills'));
		$Doc->setDocumentData('players_team2',get_sorted_array($players_team2,'player_kills'));
		$Doc->setDocumentData('players_team3',get_sorted_array($players_team3,'player_kills'));
	
		$Doc->setDocumentData('total_players_team1',sizeof($players_team1));
		$Doc->setDocumentData('total_players_team2',sizeof($players_team2));
		$Doc->setDocumentData('total_players_team3',sizeof($players_team3));
		
		$Doc->setDocumentData('total_player_kills_team1',$totals_team1['kills']);
		$Doc->setDocumentData('total_player_deaths_team1',$totals_team1['deaths']);
		$Doc->setDocumentData('avg_player_ping_team1',(int)($totals_team1['ping']/((!$num_team1)?1:$num_team1)));
		$Doc->setDocumentData('avg_player_online_team1',get_time_online($totals_team1['online']/((!$num_team1)?1:$num_team1)));
	
		$Doc->setDocumentData('total_player_kills_team2',$totals_team2['kills']);
		$Doc->setDocumentData('total_player_deaths_team2',$totals_team2['deaths']);
		$Doc->setDocumentData('avg_player_ping_team2',(int)($totals_team2['ping']/((!$num_team2)?1:$num_team2)));
		$Doc->setDocumentData('avg_player_online_team2',get_time_online($totals_team2['online']/((!$num_team2)?1:$num_team2)));
		
		$Doc->setDocumentData('total_player_kills',$totals['kills']);
		$Doc->setDocumentData('total_player_deaths',$totals['deaths']);
		$Doc->setDocumentData('avg_player_ping',(int)($totals['ping']/((!$numplayers)?1:$numplayers)));
		$Doc->setDocumentData('avg_player_online',get_time_online($totals['online']/((!$numplayers)?1:$numplayers)));
		
		$Doc->setDocumentData('cvars',$cvars);
		$Doc->setDocumentData('mapcycle',$mapcycle);
	}
}
?>
