<?php
/*****************************************************************
* UCStats - A stats generation program for Half-Life games.
* Copyright (c) 2003 Mike Cao <mike@mikecao.com>
*****************************************************************/

/*
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, 
MA 02111-1307, USA.
*/

/*****************************************************************
* Installation
*****************************************************************/

set_time_limit(0);

function read($prompt) {
	if (strlen($prompt)) {
		echo "$prompt";
	}
	$fp = fopen("php://stdin", "r");
	$input = fgets($fp, 255);
	fclose($fp);
	return trim($input);
}

define('PROGRAM_VERSION', '1.1');

$STAGE = 1;
$INPUT = array();

$INPUT['PROGRAM_VERSION'] = PROGRAM_VERSION;
$INPUT['GAME_TYPE'] = 'cstrike';
$INPUT['DB_TYPE'] = 'mysql';

echo "=============================================================\n";
echo "UCStats version ".PROGRAM_VERSION."\n";
echo "Copyright (c) ".date("Y")." Mike Cao\n";
echo "=============================================================\n\n";

echo "
INSTALLATION:

This script will configure and install UCStats on your system by 
asking you a series of questions. No changes will be made to your
system until you have answered all the questions.

";

// Get input
while ($STAGE > 0) {
	switch ($STAGE) {
		case 1:
		echo "1. Where is the UCStats program located?\n";
		echo "   Enter the full path and use forward slashes.\n";
		echo "   For example 'C:/UCStats'\n\n";
		$INPUT['PROGRAM_PATH'] = read('>> ');
		
		if (is_dir($INPUT['PROGRAM_PATH'])) {
			echo "Saved.\n\n";
			$STAGE++;
		}
		else {
			echo "The path you enterd is invalid or does not exist. Try again.\n\n";
		}
		break;
		
		case 2:
		echo "2. Where do you want the stats web pages to be installed?\n";
		echo "   Enter the full path and use forward slashes.\n";
		echo "   For example 'C:/Apache/htdocs'.\n\n";
		$INPUT['WEB_PATH'] = read('>> ');
		
		if (is_dir($INPUT['WEB_PATH'])) {
			echo "Saved.\n\n";
			$STAGE++;
		}
		else {
			echo "The path you enterd is invalid or does not exist. Try again.\n\n";
		}
		break;
		
		case 3:
		echo "3. What is your database hostname?\n\n";
		$INPUT['DB_HOST'] = read('>> ');
		echo "Saved.\n\n";
		$STAGE++;
		break;
		
		case 4:
		echo "4. What is your database login?\n\n";
		$INPUT['DB_LOGIN'] = read('>> ');
		echo "Saved.\n\n";
		$STAGE++;
		break;
		
		case 5:
		echo "5. What is your database password?\n\n";
		$INPUT['DB_PASSWORD'] = read('>> ');
		echo "Saved.\n\n";
		$STAGE++;
		break;
		
		case 6:
		echo "6. What is the name of the stats database you created?\n";
		echo "   This database should have been created prior to running\n";
		echo "   this install script. See README.txt for more details.\n\n";
		$INPUT['DB_NAME'] = read('>> ');
		echo "Saved.\n\n";
		$STAGE++;
		break;
		
		case 7:
		echo "7. Enable persistent database connections? (default = N)\n\n";
		$INPUT['DB_PCONNECT'] = read('>> (Y/N) ');
		
		if (strtolower($INPUT['DB_PCONNECT'])=='y') {
			$INPUT['DB_PCONNECT'] = 'true';
			echo "Saved.\n\n";
			$STAGE++;
		}
		else if (strtolower($INPUT['DB_PCONNECT'])=='n' || $INPUT['DB_PCONNECT']=='') {
			$INPUT['DB_PCONNECT'] = 'false';
			echo "Saved.\n\n";
			$STAGE++;
		}
		else {
			echo "Type 'Y' or 'N'.\n\n";
		}
		break;
		
		case 8:
		echo "8. Select a word for generating stats. This can be anything.\n";
		echo "   It is used as a seed for generating unique keys.\n\n";
		$INPUT['STATS_SEED'] = read('>> ');
		echo "Saved.\n\n";
		$STAGE++;
		break;
		
		case 9:
		echo "UCStats is configured and now ready to install. The following\n";
		echo "are the values you have entered:\n\n";
		
		foreach($INPUT as $key => $value) {
			echo "[$key] => $value\n";
		}
		
		echo "\nIf the values are correct type 'Y' to begin the installation.\n";
		echo "If not say 'N' to start over from the beggining.\n\n";
		$install = read('>> (Y/N) ');
		if (strtolower($install) == 'y') {
			$STAGE = 0;
			echo "\n\n";
		}
		else if (strtolower($install) == 'n') {
			$STAGE = 1;
			echo "\n\n";
		}
		else {
			echo "Type 'Y' or 'N'.\n\n";
		}
		break;
	}
}

// Test database settings
echo "Testing database connection... ";
if ($INPUT['DB_TYPE'] == 'mysql') {
	// Connect
	if($INPUT['DB_PCONNECT']) {
		if(!strlen($INPUT['DB_PASSWORD'])) {
			$dbc = mysql_pconnect($INPUT['DB_HOST'],$INPUT['DB_LOGIN'])
				or die ("\nCould not connect to database (".$INPUT['DB_HOST']."). Installation aborted.");
		} else {
			$dbc = mysql_pconnect($INPUT['DB_HOST'],$INPUT['DB_LOGIN'],$INPUT['DB_PASSWORD'])
				or die ("\nCould not connect to database (".$INPUT['DB_HOST']."). Installation aborted.");
		}
	} else {
		if(!strlen(DB_PASSWORD)) {
			$dbc = mysql_connect($INPUT['DB_HOST'],$INPUT['DB_LOGIN'])
				or die ("\nCould not connect to database (".$INPUT['DB_HOST']."). Installation aborted.");
		} else {
			$dbc = mysql_connect($INPUT['DB_HOST'],$INPUT['DB_LOGIN'],$INPUT['DB_PASSWORD'])
				or die ("\nCould not connect to database (".$INPUT['DB_HOST']."). Installation aborted.");
		}
	}
	// Select database
	mysql_select_db($INPUT['DB_NAME'],$dbc)
		or die ("\nCould not select database (".$INPUT['DB_NAME'].'@'.$INPUT['DB_HOST']."). Installation aborted.");
}
echo "Done.\n\n";

// Begin install
include('includes/func_task.php');

echo "Installing UCStats...\n";

$STAGE = 1;
while ($STAGE > 0) {
	switch ($STAGE) {
		case 1:
		echo "- Creating config file... ";
		$confirm = 'y';
		if (file_exists('./config/config.php')) {
			$confirm = read('\nA config file already exists. Do you want to overwrite it? (Y/N) ');
		}
		if (strtolower($confirm) == 'y') {
			task_config_update('./config/config.tpl','./config/config.php',$INPUT);
			echo "Done.\n";
			$STAGE++;
		}
		else if (strtolower($confirm) == 'n') {
			echo "Done.\n";
			$STAGE++;
		}
		else {
			echo "Type 'Y' or 'N'.\n\n";
		}
		break;

		case 2:
		echo '- Copying web files to '.$INPUT['WEB_PATH']."... ";
		task_dir_copy($INPUT['PROGRAM_PATH'].'/web/',$INPUT['WEB_PATH'].'/');

		if (!copy($INPUT['PROGRAM_PATH'].'/config/config.php',$INPUT['WEB_PATH'].'/config.php')) {
			echo "\nFailed to copy the config file to the web directory ".$INPUT['WEB_PATH'].". Check your folder permissions.\n";
			exit;
		}
		echo "Done.\n";
		$STAGE = 0;
		break;
	}
}
echo "Installation complete.\n\n";
echo "Read README.txt for instructions on collecting data.\n\n";
?>
