<?php
/*****************************************************************
* UCStats v1.0 - Stats program for Half-Life games.
* Copyright (c) 2003 Mike Cao <mike@mikecao.com>
*****************************************************************/

/*
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, 
MA 02111-1307, USA.
*/

/*****************************************************************
* Live Feed Daemon
*****************************************************************/

set_time_limit(0);

define("__SCRIPT__", true);

ini_set('output_buffering','0');

// Includes
include('config/config.php');
include(PROGRAM_PATH.'/includes/proc_load_base.php');
include(PROGRAM_PATH.'/includes/proc_db_open.php');
include(PROGRAM_PATH.'/includes/proc_load_configs.php');

/*****************************************************************
* Main
*****************************************************************/

echo get_token_string(L_SCRIPT_HEADER,array(PROGRAM_VERSION,date("Y")));

// Get list of registered servers
echo L_LOADING_SERVERS."\n";
include(PROGRAM_PATH.'/includes/proc_load_servers.php');

if (is_null($GAME_servers)) {
	echo L_ERR_LOAD_SERVERS."\n";
	exit;
}
else if (sizeof($GAME_servers) < 1) {
	echo L_ERR_NO_SERVERS."\n";
	exit;
}
else {
	foreach ($GAME_servers as $key => $value) {
		echo "- ".$GAME_servers[$key]['server_name']." (".$key.")\n";
	}
	echo get_token_string(L_FOUND_SERVERS,array(sizeof($GAME_servers)))."\n";
}

$RECV_IP = gethostbyname(trim(substr(CFG_LF_ADDRESS,0,strpos(CFG_LF_ADDRESS,':'))));
$RECV_PORT = trim(substr(CFG_LF_ADDRESS,strpos(CFG_LF_ADDRESS,':')+1,strlen(CFG_LF_ADDRESS)));

$buffer = array();
$count = 0;

if (!strlen($RECV_IP) || $RECV_PORT <= 0) {
	echo L_BAD_LF_CONFIG."\n";
	exit;
}

// Create receiving socket
echo L_SOCKET_CREATE."\n";
$socket_recv = socket_create(AF_INET,SOCK_DGRAM,0)
or die(L_ERR_SOCKET_CREATE."\n");

echo get_token_string(L_SOCKET_BIND,array($RECV_IP,$RECV_PORT))."\n";
$result = socket_bind($socket_recv,$RECV_IP,$RECV_PORT) 
or die(get_token_string(L_ERR_SOCKET_BIND,array($RECV_IP,$RECV_PORT))."\n");

// Socket ready
echo L_SOCKET_READY."\n";

while ($result = socket_recvfrom($socket_recv,$input,4096,0,$remote_ip,$remote_port)
or die(L_ERR_SOCKET_RECV."\n")) {

	$sid = $remote_ip.':'.$remote_port;
	$data = explode('"',$input);

	// Authorize data
	if (strlen($input) && isset($GAME_servers[$data[0]])) {

		echo "RECV <$sid>: $input\n";

		// Process data
		if ($data[1] == 'LIVEFEED') {
			if (!isset($buffer[$sid.'_live'])) {
				$buffer[$sid.'_live'] = '';
			}
			if ($data[2] == 'BEGIN') {
				$buffer[$sid.'_live'] = '';
			}
			else if ($data[2] == 'END') {
				$file = PROGRAM_PATH.'/run/'.str_replace('.','_',str_replace(':','_',$data[0])).'_live.log';
				$fp = fopen($file,'w+');
				flock($fp,2);
				fwrite($fp, $buffer[$sid.'_live']);
				fclose($fp);
			}
			else {
				$buffer[$sid.'_live'] .= str_replace($data[0].'"'.$data[1].'"','',$input)."\n";
			}
		}
		else if ($data[1] == 'EVENT' || $data[1] == 'CHAT') {
			$str = '';
			$fptype = 'a+';
			// Reset file on mapchange
			// Hack: prevent the file from getting too big
			if ($data[2] == 'MAPCHANGE' || $count > 1000) {
				$fptype = 'w+';
				$count = 0;
			}
			else {
				$str = time().'"'.str_replace($data[0].'"','',$input)."\n";
				$count++;
			}

			$file = PROGRAM_PATH.'/run/'.str_replace('.','_',str_replace(':','_',$data[0])).'_chat.log';
			$fp = fopen($file,$fptype);
			flock($fp,2);
			fwrite($fp, $str);
			fclose($fp);
		}
	}
	else {
		echo get_token_string(L_SERVER_NOT_REG,array($data[0]))."\n";
	}
}

// Close socket
echo L_SOCKET_CLOSE;
socket_close($socket_recv);
echo L_DONE.".\n";
?>
