<?php
/*****************************************************************
* UCStats - A stats generation program for Half-Life games.
* Copyright (c) 2003 Mike Cao <mike@mikecao.com>
*****************************************************************/

/*
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, 
MA 02111-1307, USA.
*/

/*****************************************************************
* Log Daemon
*****************************************************************/

set_time_limit(0);

define("__SCRIPT__", true);

ini_set('output_buffering','0');

// Includes
include('config/config.php');
include(PROGRAM_PATH.'/includes/proc_load_base.php');
include(PROGRAM_PATH.'/includes/proc_db_open.php');
include(PROGRAM_PATH.'/includes/proc_load_configs.php');
include(PROGRAM_PATH.'/includes/class_parser.php');
include(PROGRAM_PATH.'/includes/class_query.php');

// Initialize
global $_SERVER;

if (!isset($_SERVER['argc']) || !isset($_SERVER['argv'])) {
	echo L_ARGC_ARGV_MSG."\n";
	exit;
}

$argc = $_SERVER['argc'];
$argv = $_SERVER['argv'];

// Errors
function quit($msg) {
	echo "$msg\n";
	echo L_LOGD_HELP;
	exit;
}

// Check arguments
if ($argc < 2) {
	quit(L_LOGD_MSG);
}

$PID = $argv[1];
$INTERVAL = $argv[2];
$LOG_NUM = 0;

/*****************************************************************
* Main
*****************************************************************/

echo get_token_string(L_SCRIPT_HEADER,array(PROGRAM_VERSION,date("Y")));

// Get list of registered servers
echo L_LOADING_SERVERS."\n";
include(PROGRAM_PATH.'/includes/proc_load_servers.php');

if (is_null($GAME_servers)) {
	echo L_ERR_LOAD_SERVERS."\n";
	exit;
}
else if (sizeof($GAME_servers) < 1) {
	echo L_ERR_NO_SERVERS."\n";
	exit;
}
else {
	foreach ($GAME_servers as $key => $value) {
		echo "- ".$GAME_servers[$key]['server_name']." (".$key.")\n";
	}
	echo get_token_string(L_FOUND_SERVERS, array(sizeof($GAME_servers)))."\n";
}

// Load game support
$GAME = 'LOAD_ALL';
echo L_LOADING_GAME."\n";
include(PROGRAM_PATH.'/includes/proc_load_game.php');

// Create a PID file
task_create_pid('logd',getmypid());

$Query = new Query();
$Parser = new Parser($Query);

$lastreset = time();
$lastrun = time();

echo get_token_string(L_DONE_LOGD, array(getmypid()))."\n";

// Check if parent process is still alive
while ($PID == get_pid(PROGRAM_PATH.'/run/statsd.pid')) {
	// Process log queue for each server
	foreach ($GAME_servers as $key => $value) {
		// Get logs
		$result = db_get_log($key);

		// Parse logs
		if (!is_null($result)) {
			if (mysql_num_rows($result) > 0) {
				$Query->setOwner($key);

				$GAME = $GAME_servers[$key]['server_game'];
				
				$Parser->setConfig('use_log_date', false);
				$Parser->setConfig('ps_enabled', $GAME_servers[$key]['ps_enabled']);
				$Parser->setConfig('ss_enabled', $GAME_servers[$key]['ss_enabled']);
				$Parser->setConfig('lf_enabled', $GAME_servers[$key]['lf_enabled']);

				while($row = mysql_fetch_array($result)) {
					$Parser->parse(trim($row['log_value']));
					$LOG_NUM = $row['log_num'];
				}
				$lastrun = time();
			}
			
			// Delete logs
			db_delete_log($key,$LOG_NUM);
		}
	}

	// Update PID file
	$f = fopen (PROGRAM_PATH.'/run/logd.pid', 'a+');
	fclose($f);
	touch(PROGRAM_PATH.'/run/logd.pid');
	
	// Reset query cache every hour
	if ((time()-$lastreset) > 3600) {
		$Query->resetCache();
		$lastreset = time();
	}

	// Clean up old item from web cache
	task_dir_cleanup(WEB_PATH.'/cache/',time()-3600);

	// If statsd proccess died, then quit
	if ((time()-$lastrun) > 600) {
		log_error(L_ERR_LOGD_QUIT);
		break;
	}
	
	sleep($INTERVAL);
}

// Exit if parent dies
echo get_token_string(L_LOGD_EXIT, array($PID))."\n";

// Close database connection
include(PROGRAM_PATH.'/includes/proc_db_close.php');
