<?php
/*****************************************************************
* UCStats - A stats generation program for Half-Life games.
* Copyright (c) 2003 Mike Cao <mike@mikecao.com>
*****************************************************************/

/*
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, 
MA 02111-1307, USA.
*/

/*****************************************************************
* Stats Parser
*****************************************************************/

set_time_limit(0);

define("__SCRIPT__", true);

ini_set('output_buffering','0');

// Includes
include('config/config.php');
include(PROGRAM_PATH.'/includes/proc_load_base.php');
include(PROGRAM_PATH.'/includes/proc_db_open.php');
include(PROGRAM_PATH.'/includes/proc_load_configs.php');
include(PROGRAM_PATH.'/includes/class_parser.php');
include(PROGRAM_PATH.'/includes/class_query.php');

// Initialize
global $_SERVER;

if (!isset($_SERVER['argc']) || !isset($_SERVER['argv'])) {
	echo L_ARGC_ARGV_MSG."\n";
	exit;
}

$argc = $_SERVER['argc'];
$argv = $_SERVER['argv'];

$OPTIONS = array(
	'd' => false,
	'm' => false,
	'v' => false,
	'i' => false,
	'u' => false,
	'c' => false,
	't' => false,
	'w' => false
);

$LOG_SRC = null;
$SERVER_SRC = null;
$MOVE_DIR = null;

// Errors
function quit($msg) {
	echo "$msg\n";
	echo L_PARSER_HELP;
	exit;
}

// Check arguments
if ($argc < 2) {
	quit(L_PARSER_MSG);
}
for ($a = 0; $a < $argc; $a++) {
	$arg = $argv[$a];
	if ($arg{0} == '-') {
		for ($i = 1; $i < strlen($arg); $i++) {
			if (isset($OPTIONS[$arg{$i}])) {
				$OPTIONS[$arg{$i}] = true;
			}
			else {
				quit(get_token_string(L_ERR_INVALID_OPTION,array($arg{$i}))."\n");
			}
			// If 'm' flag found, next param should be a directory
			if ($arg{$i} == 'm' && $a+1 <= $argc-3) {
				if (is_dir($argv[$a+1])) {
					$MOVE_DIR = $argv[$a+1];
				}
				else {
					quit(L_ERR_INVALID_DIR."\n");
				}
			}
		}
	}
	else {
		if ($a == $argc-2) {
			if (file_exists($argv[$a]) && (is_file($argv[$a]) || is_dir($argv[$a]))) {
				$LOG_SRC = $argv[$a];
			}
			else {
				quit(L_ERR_INVALID_FILE_DIR."\n");
			}
		}
		else if ($a == $argc-1) {
			if (is_valid_address($argv[$a])) {
				$SERVER_SRC = $argv[$a];
			}
			else {
				quit(L_ERR_SERVER_ADDR."\n");
			}
		}
	}
}

// Last checks
if (is_null($LOG_SRC)) {
	quit(L_ERR_NO_FILE_DIR."\n");
}
if (is_null($SERVER_SRC)) {
	quit(L_ERR_NO_SERVER."\n");
}
if ($OPTIONS['m'] && is_null($MOVE_DIR)) {
	quit(L_ERR_NO_MOVE_DIR."\n");
}
if ($OPTIONS['m'] && $OPTIONS['d']) {
	quit(L_ERR_BOTH_OPTIONS."\n");
}


/*****************************************************************
* Main
*****************************************************************/

echo get_token_string(L_SCRIPT_HEADER,array(PROGRAM_VERSION,date("Y")));

// Get list of registered servers
echo L_LOADING_SERVERS."\n";
include(PROGRAM_PATH.'/includes/proc_load_servers.php');

if (is_null($GAME_servers)) {
	echo L_ERR_LOAD_SERVERS."\n";
	exit;
}
else if (sizeof($GAME_servers) < 1) {
	echo L_ERR_NO_SERVERS."\n";
	exit;
}
else {
	foreach ($GAME_servers as $key => $value) {
		echo "- ".$GAME_servers[$key]['server_name']." (".$key.")\n";
	}
	echo get_token_string(L_FOUND_SERVERS,array(sizeof($GAME_servers)))."\n";
}

// See if registered
if (!isset($GAME_servers[$SERVER_SRC])) {
	echo get_token_string(L_ERR_SERVER_REG,array($SERVER_SRC))."\n";
	exit;
}

// Load game support
$GAME = $GAME_servers[$SERVER_SRC]['server_game'];

echo get_token_string(L_LOAD_GAME, array(get_mod_name($GAME)))."\n";
include(PROGRAM_PATH.'/includes/proc_load_game.php');

$lastlogfile = '';
$lastlogline = 0;
$lastmodtime = 0;
$starttime = get_time_micro();

$Query = new Query();
$Parser = new Parser($Query);

$Query->setOwner($SERVER_SRC);

$Parser->setConfig('game', $GAME);
$Parser->setConfig('use_log_date', $OPTIONS['u']);
$Parser->setConfig('ps_enabled', $GAME_servers[$SERVER_SRC]['ps_enabled']);
$Parser->setConfig('ss_enabled', $GAME_servers[$SERVER_SRC]['ss_enabled']);
$Parser->setConfig('lf_enabled', $GAME_servers[$SERVER_SRC]['lf_enabled']);

$LOG_SRC = str_replace('\\','/',$LOG_SRC);

// Parse a file
if (is_file($LOG_SRC)) {
	echo "Processing file $LOG_SRC... ";
	$contents = file($LOG_SRC);

	$check = true;
	
	// Check if log is complete
	if (!$OPTIONS['i']) {
		if (!preg_match("/Log file closed/",$contents[sizeof($contents)-1])) {
			$check = false;
		
			echo L_ERROR.".\n";
			echo "- ".L_LOG_INCOMPLETE."\n";
			if (file_exists($LOG_SRC.'.incomplete')) unlink($LOG_SRC.'.incomplete');
			if (rename($LOG_SRC, $LOG_SRC.'.incomplete')) {
				$LOG_SRC .= '.incomplete';
			}
			else {
				echo get_token_string(L_ERR_RENAME, array($LOG_SRC))."\n";
				exit;
			}
		}
	}
	
   // Parse log
	if ($check) {
			$str = '';
			for ($i = 0; $i < sizeof($contents); $i++) {
					$Parser->parse(trim($contents[$i]));
					if ($OPTIONS['v']) {
							echo $contents[$i];
					}
			}
			echo L_DONE.".\n";
			echo "- ".get_token_string(L_PARSED_MSG, array(sizeof($contents),number_format(get_time_micro()-$starttime,4,'.','')))."\n";
	}
	
	// Timestamp file
	if ($OPTIONS['t']) {
		$timestamp = time();
		$dir = substr($LOG_SRC,0,strrpos($LOG_SRC,'/')+1);
		$file = substr($LOG_SRC,strrpos($LOG_SRC,'/')+1);
		if (rename($LOG_SRC, $dir.$timestamp.'.'.$file)) {
			$LOG_SRC = $dir.$timestamp.'.'.$file;
		}
		else {
			echo get_token_string(L_ERR_RENAME, array($LOG_SRC))."\n";
			exit;
		}
	}
	
	// Move or delete file
	if ($OPTIONS['m']) {
		if (copy($LOG_SRC,$MOVE_DIR)) {
			unlink($LOG_SRC);
			echo "- ".get_token_string(L_FILE_MOVED, array($MOVE_DIR))."\n";
		}
		else {
			echo get_token_string(L_ERR_MOVE, array($MOVE_DIR));
			exit;
		}
	}
	else if ($OPTIONS['d']) {
		unlink($LOG_SRC);
		echo "- ".L_FILE_DELETED."\n";
	}
}

// Parse a directory
else if (is_dir($LOG_SRC)) {
	$dir = opendir($LOG_SRC);
	
	echo get_token_string(L_LOG_SCAN, array($LOG_SRC))."\n";
	
	$num = 0;
	$total = 0;
	
	while(($file = readdir($dir)) != false) {
		if (is_file($LOG_SRC.'/'.$file) && $file != "." && $file != ".." && substr($file,-4)==".log") {
			$total++;
		}
	}
	echo get_token_string(L_FOUND_TOTAL, array($total))."\n";
	rewinddir($dir);
	
	// Get last modified file
	$lastlog = get_last_mod_file($LOG_SRC);
	echo get_token_string(L_MOD_FILE, array($lastlog))."\n";
	
	// Get list of log files from directory
	while(($file = readdir($dir)) != false) {
		if (is_file($LOG_SRC.'/'.$file) && $file != "." && $file != ".." && substr($file,-4)==".log") {
			$num++;
			echo get_token_string(L_PROCESS_FILE, array($file,$num,$total));
			$time = get_time_micro();
			$contents = file($LOG_SRC.'/'.$file);

			$check = true;
			$is_complete = true;
			
			// Check if log is complete
			if (!preg_match("/Log file closed/",$contents[sizeof($contents)-1])) {
				$is_complete = false;
			}

			if (!$OPTIONS['i'] && !$is_complete) {
				$check = false;
				if ($file != $lastlog) {
					echo L_ERROR.".\n";
					echo "- ".L_LOG_INCOMPLETE."\n";
					if (file_exists($LOG_SRC.'.incomplete')) unlink($LOG_SRC.'.incomplete');
					if (rename($LOG_SRC.'/'.$file, $LOG_SRC.'/'.$file.'.incomplete')) {
						$file .= '.incomplete';
					}
					else {
						echo get_token_string(L_ERR_RENAME, array($LOG_SRC))."\n";
						exit;
					}
				}
				else {
					echo L_ABORT.".\n";
					echo "- ".L_LOG_ACTIVE."\n";
					continue;
				}
			}
			
			// Parse log
			if ($check) {
				$str = '';
				for ($i = 0; $i < sizeof($contents); $i++) {
					$Parser->parse(rtrim($contents[$i]));
					if ($OPTIONS['v']) {
						echo $contents[$i];
					}
				}
				echo L_DONE.".\n";
				echo "- ".get_token_string(L_PARSED_MSG, array(sizeof($contents),number_format(get_time_micro()-$time,4,'.','')))."\n";
			}

			// Timestamp file
			if ($OPTIONS['t']) {
				$timestamp = time();
				if (rename($LOG_SRC.'/'.$file,$LOG_SRC.'/'.$timestamp.'.'.$file)) {
					$file = $timestamp.'.'.$file;
				}
				else {
					echo get_token_string(L_ERR_RENAME, array($file))."\n";
					exit;
				}
			}

			// Move or delete log
			if ($OPTIONS['m']) {
				if (copy($LOG_SRC.'/'.$file,$MOVE_DIR.'/'.$file)) {
					unlink($LOG_SRC.'/'.$file);
					echo "- ".get_token_string(L_FILE_MOVED, array($MOVE_DIR))."\n";
				}
				else {
					echo get_token_string(L_ERR_MOVE, array($MOVE_DIR));
					exit;
				}
			}
			else if ($OPTIONS['d']) {
				unlink($LOG_SRC.'/'.$file);
				echo "- ".L_FILE_DELETED."\n";
			}
		}
	}
	
	echo "\n".get_token_string(L_BATCH_COMPLETE, array($num,number_format((get_time_micro()-$starttime),4,'.','')))."\n";
	
	// Close directory
	closedir($dir);
}

// Clear web cache
if ($OPTIONS['w']) {
	echo "\n".L_CLEAR_CACHE;
	$count = task_dir_cleanup(WEB_PATH.'/cache/',time());
	echo L_DONE.".\n";
	echo "- ".get_token_string(L_FILES_REMOVED, array($count))."\n";
}

// Close database connection
include(PROGRAM_PATH.'/includes/proc_db_close.php');
