/*
*	AMXMOD script.
*	(ucstats_cstrike.sma)
*	by mike_cao <mike@mikecao.com>
*	This file is provided as is (no warranties).
*
*/

#include <amxmod>
#include <ucstats>

// Weapon names for DOD
new gDodWeapons[45][] = {
	"","amerknife","gerknife","colt","luger","garand","scopedkar","thompson","mp44",
	"spring","kar","bar","mp40","grenade","grenade2","grenade","grenade2",
	"mg42","30cal","spade","m1carbine","mg34","greasegun","fg42","k43","enfield",
	"sten","bren","webly","bazooka","pschreck","piat","scoped_fg42","fcarbine",
	"bayonet","scoped_enfield","mills_bomb","brit_knife","garandbutt","enf_bayonet",
	"k43butt","","","",""
}

// Globals
new gPlayerStats[MAX_PLAYERS+1][PLAYER_INDEX]
new gWeaponStats[MAX_PLAYERS+1][WEAPON_INDEX]
new gSurvivorIndex[MAX_PLAYERS+1]
new gPlayerClass[MAX_PLAYERS+1][MAX_NAME_LENGTH]
new gTeam[MAX_NAME_LENGTH]

// Admin access level
#define ACCESS_LEVEL ADMIN_LEVEL_A


/************************************************************
* LOG FUNCTIONS
************************************************************/

log_trigger(id,trigger[])
{
	new iUserid = get_user_userid(id)
	if (iUserid < 0) return PLUGIN_HANDLED

	new sName[MAX_NAME_LENGTH]
	new sAuthid[MAX_NAME_LENGTH]

	get_user_name(id,sName,MAX_NAME_LENGTH)
	get_user_authid(id,sAuthid,MAX_NAME_LENGTH)
	get_team_name(id)

	log_message("^"%s<%d><%s><%s>^" triggered ^"%s^"",sName,iUserid,sAuthid,gTeam,trigger)
	return PLUGIN_HANDLED
}

log_weapon_stats(id)
{
        new iUserid = get_user_userid(id)
        if (iUserid < 0) return PLUGIN_HANDLED

	new iIndex
	new sName[MAX_NAME_LENGTH]
	new sAuthid[MAX_NAME_LENGTH]

	get_user_name(id,sName,MAX_NAME_LENGTH)
	get_user_authid(id,sAuthid,MAX_NAME_LENGTH)
	get_team_name(id)

	for (new i = 0; i < MAX_DOD_WEAPONS; ++i) {
		// Get index
		iIndex = (i*INDEX_WEAPONS)
		
		// If weapon used
		if (gWeaponStats[id][iIndex+INDEX_WEAPON]) {
			// "Weapon_Stats" (weapon "kills,deaths,headshots,teamkills,shots,hits,damage")
			if (get_weapons_total(id,iIndex) > 0) {
				// Fix for melee weapons
				if (is_melee(i)) {
					gWeaponStats[id][iIndex+INDEX_SHOTS] = 0
				}
				log_message(
					"^"%s<%d><%s><%s>^" triggered ^"Weapon_Stats^" (%s ^"%i,%i,%i,%i,%i,%i,%i^")",
					sName,
					iUserid,
					sAuthid,
					gTeam,
					gDodWeapons[i],
					gWeaponStats[id][iIndex+INDEX_KILLS],
					gWeaponStats[id][iIndex+INDEX_DEATHS],
					gWeaponStats[id][iIndex+INDEX_HS],
					gWeaponStats[id][iIndex+INDEX_TK],
					gWeaponStats[id][iIndex+INDEX_SHOTS],
					gWeaponStats[id][iIndex+INDEX_HITS],
					gWeaponStats[id][iIndex+INDEX_DMG]
				)
			}
			
			// "Weapon_Hits" (weapon "generic,head,chest,stomach,leftarm,rightarm,leftleg,rightleg")
			iIndex += INDEX_BODY
			if (get_hits_total(id,iIndex) > 0) {
				log_message(
					"^"%s<%d><%s><%s>^" triggered ^"Weapon_Hits^" (%s ^"%i,%i,%i,%i,%i,%i,%i,%i^")",
					sName,
					iUserid,
					sAuthid,
					gTeam,
					gDodWeapons[i],
					gWeaponStats[id][iIndex+INDEX_GENERIC],
					gWeaponStats[id][iIndex+INDEX_HEAD],
					gWeaponStats[id][iIndex+INDEX_CHEST],
					gWeaponStats[id][iIndex+INDEX_STOMACH],
					gWeaponStats[id][iIndex+INDEX_LEFTARM],
					gWeaponStats[id][iIndex+INDEX_RIGHTARM],
					gWeaponStats[id][iIndex+INDEX_LEFTLEG],
					gWeaponStats[id][iIndex+INDEX_RIGHTLEG]
				)
			}
		}
	}
        return PLUGIN_HANDLED
}

log_player_stats(id)
{
        new iUserid = get_user_userid(id)
        if (iUserid < 0) return PLUGIN_HANDLED

	new sName[MAX_NAME_LENGTH]
	new sAuthid[MAX_NAME_LENGTH]
	new sMap[MAX_NAME_LENGTH]

	get_user_name(id,sName,MAX_NAME_LENGTH)
	get_user_authid(id,sAuthid,MAX_NAME_LENGTH)
	get_mapname(sMap,MAX_NAME_LENGTH)
	get_team_name(id)

	if ((gPlayerStats[id][PLAYER_KILLS]+gPlayerStats[id][PLAYER_DEATHS]) > 0) {
		// "Player_Stats" (map "kills,deaths,killstreak,rounds_survived,rounds_played,playing_time,misc")
		log_message("^"%s<%d><%s><%s>^" triggered ^"Player_Stats^" (%s ^"%i,%i,%i,%i,%i,%i^")",
			sName,
			iUserid,
			sAuthid,
			gTeam,
			sMap,
			gPlayerStats[id][PLAYER_KILLS],
			gPlayerStats[id][PLAYER_DEATHS],
			gPlayerStats[id][PLAYER_KILLSTREAK],
			gPlayerStats[id][PLAYER_SURVIVE],
			gPlayerStats[id][PLAYER_ROUNDS],
			floatround(get_gametime())-gPlayerStats[id][PLAYER_TIME]
		)
	}
        return PLUGIN_HANDLED
}

/************************************************************
* EVENT FUNCTIONS
************************************************************/

public event_health()
{
	new iPlayer = read_data(0)
	new iHealth = read_data(1)

	if (is_user_alive(iPlayer)) {
		new iWeapon, iBody
		new iPrevHealth = gPlayerStats[iPlayer][PLAYER_HEALTH]
		new iAttacker = get_user_attacker(iPlayer,iWeapon,iBody)
		new iDamage = iPrevHealth - iHealth

		if (iDamage > 0 && iAttacker > 0 && iAttacker != iPlayer) {
			iWeapon = get_dod_weapon(iAttacker,iWeapon)

                        // Set weapon use
                        gWeaponStats[iAttacker][(iWeapon*INDEX_WEAPONS)+INDEX_WEAPON] = 1

                        // Set damage
                        gWeaponStats[iAttacker][(iWeapon*INDEX_WEAPONS)+INDEX_DMG] += iDamage

                        // Set weapon hit
                        gWeaponStats[iAttacker][(iWeapon*INDEX_WEAPONS)+INDEX_HITS] += 1

                        // Set body hit
                        gWeaponStats[iAttacker][(iWeapon*INDEX_WEAPONS)+INDEX_BODY+iBody] += 1
                }
		gPlayerStats[iPlayer][PLAYER_HEALTH] = iHealth
	}
	return PLUGIN_CONTINUE
}

public event_respawn(id)
{
	gPlayerStats[id][PLAYER_HEALTH] = get_user_health(id)
	return PLUGIN_CONTINUE
}

public event_death()
{
	new iKiller = read_data(1)
	new iVictim = read_data(2)

	// Stats
	if (iKiller != iVictim && iKiller != 0) {
		new iWeapon, iKillerWeapon, iVictimWeapon, iClip, iAmmo 
		new iTeamKill = 0

		iWeapon = read_data(3)
		iKillerWeapon = get_user_weapon(iKiller,iClip,iAmmo)
		iVictimWeapon = get_user_weapon(iVictim,iClip,iAmmo)

		// Hack to fix wrong indexes given by DOD
		if (iWeapon == DOD_WEAPON42) {
			if (iKillerWeapon == DOD_GARAND) {
				iKillerWeapon = DOD_GARANDBUTT
			}
			else if (iKillerWeapon == DOD_K43) {
				iKillerWeapon = DOD_K43BUTT
			}
		}
		else if (iWeapon == DOD_BRIT_KNIFE && iKillerWeapon == DOD_KAR) {
			iKillerWeapon = DOD_BAYONET
		}
		else if (iWeapon == DOD_WEAPON43 && iKillerWeapon == DOD_ENFIELD) {
			iKillerWeapon = DOD_ENF_BAYONET
		}
		else if (iWeapon == DOD_K43BUTT && iKillerWeapon == DOD_AMERKNIFE) {
			iKillerWeapon = DOD_BRIT_KNIFE
		}
		else if (iWeapon == DOD_SCOPED_ENFIELD && iKillerWeapon == DOD_FG42) {
			iKillerWeapon = DOD_SCOPED_FG42
		}
		else if (iWeapon == DOD_GARANDBUTT && iKillerWeapon == DOD_ENFIELD) {
			iKillerWeapon = DOD_SCOPED_ENFIELD
		}
		else if (iWeapon == DOD_ENF_BAYONET) {
			iKillerWeapon = DOD_MILLS_BOMB
		}
		else if (iWeapon == DOD_MILLS_BOMB && iKillerWeapon == DOD_M1CARBINE) {
			iKillerWeapon = DOD_FCARBINE
		}
		else if (is_grenade(iWeapon) || is_rocket(iWeapon)) {
			iKillerWeapon = iWeapon
		}
	
		iKillerWeapon = get_dod_weapon(iKiller,iKillerWeapon)
		iVictimWeapon = get_dod_weapon(iVictim,iVictimWeapon)

		// Team kill
		new sTeamKiller[MAX_NAME_LENGTH], sTeamVictim[MAX_NAME_LENGTH]
		get_user_info(iKiller,"team",sTeamKiller,MAX_NAME_LENGTH)
		get_user_info(iVictim,"team",sTeamVictim,MAX_NAME_LENGTH)

		if (equal(sTeamKiller,sTeamVictim)) {
			iTeamKill = 1
		}

		// Player stats
	        gPlayerStats[iKiller][PLAYER_KILLS] += 1
	        gPlayerStats[iVictim][PLAYER_DEATHS] += 1
		gPlayerStats[iKiller][PLAYER_KILLCOUNT] += 1
		gPlayerStats[iVictim][PLAYER_KILLCOUNT] = 0

		// New killstreak
		if (gPlayerStats[iKiller][PLAYER_KILLCOUNT] > gPlayerStats[iKiller][PLAYER_KILLSTREAK]) {
			gPlayerStats[iKiller][PLAYER_KILLSTREAK] = gPlayerStats[iKiller][PLAYER_KILLCOUNT]
		}

		// Weapon stats
	        new iPrevHealth = gPlayerStats[iVictim][PLAYER_HEALTH]
        	new iHealth = get_user_health(iVictim)
	        new iDamage = (-1*iHealth) + iPrevHealth

        	gWeaponStats[iKiller][(iKillerWeapon*INDEX_WEAPONS)+INDEX_WEAPON] = 1
	        gWeaponStats[iKiller][(iKillerWeapon*INDEX_WEAPONS)+INDEX_KILLS] += 1
        	gWeaponStats[iKiller][(iKillerWeapon*INDEX_WEAPONS)+INDEX_DMG] += iDamage
	        gWeaponStats[iKiller][(iKillerWeapon*INDEX_WEAPONS)+INDEX_HITS] += 1
	        gWeaponStats[iVictim][(iVictimWeapon*INDEX_WEAPONS)+INDEX_DEATHS] += 1

		// Teamkill
		if (iTeamKill) {
			gWeaponStats[iKiller][(iKillerWeapon*INDEX_WEAPONS)+INDEX_TK] += 1
		}

		// Survivor
		gSurvivorIndex[iVictim] = 0

	        //if (is_grenade(iKillerWeapon) || is_rocket(iKillerWeapon) || is_melee(iKillerWeapon)) {
        	//     	gWeaponStats[iKiller][(iKillerWeapon*INDEX_WEAPONS)+INDEX_BODY+random_num(INDEX_HEAD,INDEX_RIGHTLEG)] += 1
	        //}
	        //else {
		new iAttackerWeapon, iBody
                new iAttacker = get_user_attacker(iVictim,iAttackerWeapon,iBody)
 	       	iAttackerWeapon = get_dod_weapon(iAttacker,iAttackerWeapon)

		if (iAttacker == iKiller && iAttackerWeapon == iKillerWeapon) {
                        gWeaponStats[iKiller][(iKillerWeapon*INDEX_WEAPONS)+INDEX_BODY+iBody] += 1

               	        // Headshot
			if (iBody == INDEX_HEAD) {
				log_trigger(iKiller,"Head_Shot")
				gWeaponStats[iKiller][(iKillerWeapon*INDEX_WEAPONS)+INDEX_HS] += 1
			}
		}
		//}
	}
	return PLUGIN_CONTINUE
}

public event_weapon()
{
	new iPlayer = read_data(0)
	new iWeapon = read_data(2)
	new iAmmo = read_data(3)
	new iPrevWeapon = gPlayerStats[iPlayer][PLAYER_LASTWEAPON]
	new iPrevAmmo = gPlayerStats[iPlayer][PLAYER_LASTAMMO]

	// Fix for dod weapons
	iWeapon = get_dod_weapon(iPlayer,iWeapon)

	// Threw a grenade
	if (is_grenade(iPrevWeapon)) {
		new iGrenClip, iGrenAmmo
		get_user_ammo(iPlayer,iPrevWeapon,iGrenClip,iGrenAmmo)
		iGrenClip -= 1
		if (iPrevAmmo > iGrenClip) {
			gWeaponStats[iPlayer][(iPrevWeapon*INDEX_WEAPONS)+INDEX_WEAPON] = 1
			gWeaponStats[iPlayer][(iPrevWeapon*INDEX_WEAPONS)+INDEX_SHOTS] += (iPrevAmmo - iGrenClip)
		}
	}

	// Same weapon and shot fired
	else if (iPrevWeapon == iWeapon && iPrevAmmo > iAmmo) {
	        gWeaponStats[iPlayer][(iWeapon*INDEX_WEAPONS)+INDEX_WEAPON] = 1
		gWeaponStats[iPlayer][(iWeapon*INDEX_WEAPONS)+INDEX_SHOTS] += 1
	}

	gPlayerStats[iPlayer][PLAYER_LASTWEAPON] = iWeapon
	gPlayerStats[iPlayer][PLAYER_LASTAMMO] = iAmmo

	return PLUGIN_CONTINUE
}

public event_reset_stats(id)
{
	for (new i = 0; i < PLAYER_INDEX; i++) {
		gPlayerStats[id][i] = 0
	}
	for (new i = 0; i < WEAPON_INDEX; i++) {
		gWeaponStats[id][i] = 0
	}
	gPlayerStats[id][PLAYER_TIME] = floatround(get_gametime())
	return PLUGIN_CONTINUE
}

public event_player_score()
{
	new iPlayer = read_data(1)
	new iScore = read_data(2)

	if (gPlayerStats[iPlayer][PLAYER_MISC] < iScore) {
		gPlayerStats[iPlayer][PLAYER_MISC] = iScore
	}

	return PLUGIN_CONTINUE
}

public event_survivor()
{
	new iPlayers[MAX_PLAYERS], iNumPlayers
	get_players(iPlayers,iNumPlayers)
	for (new i = 0; i < iNumPlayers; i++) {
		// Add round played
		gPlayerStats[iPlayers[i]][PLAYER_ROUNDS] += 1

		// Add round survived
		if (gSurvivorIndex[iPlayers[i]]) {
			gPlayerStats[iPlayers[i]][PLAYER_SURVIVE] += 1
		}

		// Reset index
		gSurvivorIndex[iPlayers[i]] = 1
	}
	return PLUGIN_CONTINUE
}

public event_team_win()
{
	new iTeam = read_data(1)
	if (iTeam == 3) {
		// allies win
	}
	else {
		// axis win
	}
	return PLUGIN_CONTINUE
}

public event_class_change()
{
	new sLog[MAX_NAME_LENGTH]
	new sClass[MAX_NAME_LENGTH]
	new sName[MAX_NAME_LENGTH]
	new sAuthid[MAX_NAME_LENGTH]
	new sTeam[MAX_NAME_LENGTH]
	new iUserId
	
	read_logargv(0,sLog,MAX_NAME_LENGTH)
	parse_loguser(sLog,sName,MAX_NAME_LENGTH,iUserId,sAuthid,MAX_NAME_LENGTH,sTeam,MAX_NAME_LENGTH)
	read_logargv(2,sClass,MAX_NAME_LENGTH)

	new iPlayer = find_player("c",sAuthid)
	format(gPlayerClass[iPlayer],MAX_NAME_LENGTH,"%s",sClass)

	return PLUGIN_CONTINUE
}


/************************************************************
* HELPER FUNCTIONS
************************************************************/

is_melee(weapon)
{
	if (weapon == DOD_AMERKNIFE || weapon == DOD_GERKNIFE || weapon == DOD_SPADE ||
		weapon == DOD_BAYONET || weapon == DOD_BRIT_KNIFE || weapon == DOD_GARANDBUTT ||
		weapon == DOD_ENF_BAYONET) {
		return 1
	}
	return 0
}

is_grenade(weapon)
{
	if (weapon == DOD_GRENADE || weapon == DOD_GRENADE_EX || weapon == DOD_GRENADE2 || 
		weapon == DOD_GRENADE2_EX || weapon == DOD_MILLS_BOMB) {
		return 1
	}
	return 0
}

is_rocket(weapon)
{
	if (weapon == DOD_PSCHRECK || weapon == DOD_PIAT || weapon == DOD_BAZOOKA) {
		return 1
	}
	return 0
}

get_dod_weapon(player,weapon)
{
	if (weapon == DOD_FG42 && equal(gPlayerClass[player],"#class_axispara_fg42scope")) {
		return DOD_SCOPED_FG42
	}
	else if (weapon == DOD_ENFIELD && equal(gPlayerClass[player],"#class_brit_sniper")) {
		return DOD_SCOPED_ENFIELD
	}
	else if (weapon == DOD_M1CARBINE && equal(gPlayerClass[player],"#class_alliedpara_carbine")) {
		return DOD_FCARBINE
	}
	else if (is_melee(weapon) && containi(gPlayerClass[player],"#class_brit") > 0) {
		return DOD_BRIT_KNIFE
	}
	return weapon
}

get_team_name(player) {
	get_user_info(player,"team",gTeam,MAX_NAME_LENGTH)
	if (gTeam[0] == 'a') {
		gTeam[0] = 'A'
	}
	else if (gTeam[0] == 's') {
		gTeam[0] = 'S'
	}
}

get_weapons_total(id,index)
{
        new iTotal = gWeaponStats[id][index+INDEX_KILLS]
        iTotal += gWeaponStats[id][index+INDEX_DEATHS]
        iTotal += gWeaponStats[id][index+INDEX_HS]
        iTotal += gWeaponStats[id][index+INDEX_TK]
        iTotal += gWeaponStats[id][index+INDEX_SHOTS]
        iTotal += gWeaponStats[id][index+INDEX_HITS]
        iTotal += gWeaponStats[id][index+INDEX_DMG]
        return iTotal
}

get_hits_total(id,index)
{
        new iTotal = gWeaponStats[id][index+INDEX_GENERIC]
        iTotal += gWeaponStats[id][index+INDEX_HEAD]
        iTotal += gWeaponStats[id][index+INDEX_CHEST]
        iTotal += gWeaponStats[id][index+INDEX_STOMACH]
        iTotal += gWeaponStats[id][index+INDEX_LEFTARM]
        iTotal += gWeaponStats[id][index+INDEX_RIGHTARM]
        iTotal += gWeaponStats[id][index+INDEX_LEFTLEG]
        iTotal += gWeaponStats[id][index+INDEX_RIGHTLEG]
        return iTotal
}


/************************************************************
* PLUGIN FUNCTIONS
************************************************************/

public client_connect(id)
{
	event_reset_stats(id)

	return PLUGIN_CONTINUE
}

public client_disconnect(id)
{
	if (get_user_time(id) > MIN_PLAYTIME) {
		log_player_stats(id)
		log_weapon_stats(id)
	}
	event_reset_stats(id)

	return PLUGIN_CONTINUE
}

public client_infochanged(id)
{
	new sName[MAX_TEXT_LENGTH], sNewName[MAX_TEXT_LENGTH]

	get_user_name(id,sName,MAX_TEXT_LENGTH)
	get_user_info(id,"name",sNewName,MAX_TEXT_LENGTH)

	if (!equal(sName,sNewName) && !equal(sName,"")) {
		// Log old name's stats
		if (get_user_time(id) > MIN_PLAYTIME) {
			log_player_stats(id)
			log_weapon_stats(id)
		}

		// Reset stats for new name
		event_reset_stats(id)
	}
	return PLUGIN_CONTINUE
}

public plugin_init() {
	register_plugin("UCStats DOD","1.1","mike_cao")

	// Mod specific events
	register_logevent("event_class_change",3,"1=changed role to")
	register_event("Health","event_health","b")
	register_event("ObjScore","event_player_score","a")
	register_event("RoundState","event_team_win","a","1=3","1=4") 
	set_task(60.0,"event_survivor",0,"",0,"b")

	// General events
	register_event("DeathMsg","event_death","a")
	register_event("CurWeapon","event_weapon","be","1=1")
	register_event("ResetHUD","event_respawn","be","1=1")
	
	server_print("[UCStats] DOD: Logging initialized successfully.")
	
	return PLUGIN_CONTINUE
}
