/*
*	AMXMOD script.
*	(ucstats_live.sma)
*	by mike_cao <mike@mikecao.com>
*	This file is provided as is (no warranties).
*
*	Provides a live server feed to UCStats.
*	You need to add the following cvars to your AMX config:
*
*	ucstats_live_ip <ip>
*	ucstats_live_port <port>
*
*	These cvars specify the ip and port of your live feed
*	daemon. This is the address the data will be sent to.
*
*	If you are behind a firewall and your local server ip 
*	does not match your external ip you need to use the
*	following cvar to set your real server ip and port:
*
*	ucstats_live_fw <ip:port>
*
*/

#include <amxmod>
#include <amxmisc>
#include <udp>
#include <ucstats>

// Admin access level
#define ACCESS_LEVEL ADMIN_LEVEL_A

// Globals
new gMod = 0
new gIP[MAX_NAME_LENGTH]
new gPort = 0
new gConnect = 0
new gSender = 0
new gTeamScore[2]
new gTeam[MAX_NAME_LENGTH]
new gMapCycle[MAX_TEXT_LENGTH]
new gNetAddress[MAX_NAME_LENGTH]
new gPlayerClass[MAX_PLAYERS+1][MAX_NAME_LENGTH]
new gLastMsg[MAX_UDP_LENGTH]

// Cvars
new gCvars[MAX_CVARS][] = {"mp_timelimit","mp_autokick","mp_autoteambalance","mp_c4timer",
			"mp_flashlight","mp_footsteps","mp_forcechasecam","mp_freezetime",
			"mp_friendlyfire","mp_hostagepenalty","mp_roundtime","mp_startmoney",
			"sv_aim","sv_cheats","sv_maxrate","sv_minrate"}

// Weapon names for DOD
new gDodWeapons[40][] = {
        "","amerknife","gerknife","colt","luger","garand","scopedkar","thompson","mp44",
        "spring","kar","bar","mp40","grenade","grenade2","grenade","grenade2",
        "mg42","30cal","spade","m1carbine","mg34","greasegun","fg42","k43","enfield",
        "sten","bren","webly","bazooka","pschreck","piat","scoped_fg42","fcarbine",
        "bayonet","scoped_enfield","mills_bomb","brit_knife","garandbutt","enf_bayonet"
}

/************************************************************
* UDP FUNCTIONS
************************************************************/

// UDP Sender
public udp_send_msg(msg[])
{
	new sUDPMsg[MAX_UDP_LENGTH]

	if (gSender == 0) {
		get_cvar_string("ucstats_live_ip",gIP,MAX_NAME_LENGTH)
		gPort = get_cvar_num("ucstats_live_port")
		gSender = set_iaddress(gIP,gPort)

		if (cvar_exists("ucstats_live_fw")) {
			new fwNetAddress[MAX_NAME_LENGTH]
			get_cvar_string("ucstats_live_fw",fwNetAddress,MAX_NAME_LENGTH)
			if (!equal(fwNetAddress,"")) {
				format(gNetAddress,MAX_NAME_LENGTH,"%s",fwNetAddress)
			}
		}
	}

	format(sUDPMsg,MAX_UDP_LENGTH,"%s^"%s",gNetAddress,msg)

	if (!equal(sUDPMsg,gLastMsg)) {
		if (send_udp_msg(gSender,sUDPMsg)) {
			server_print("[UCStats] UDP SEND: <%s:%i>: %s",gIP,gPort,sUDPMsg)
		}
		else {
			server_print("[UCStats] UDP FAILED: <%s:%i>: %s",gIP,gPort,sUDPMsg)
		}
	}

	format(gLastMsg,MAX_UDP_LENGTH,"%s",sUDPMsg)
}

public admin_livefeed(id,level,cid)
{
	if (!cmd_access(id,level,cid,0)) return PLUGIN_HANDLED

	new iPlayers[MAX_PLAYERS], iNumPlayers, iMaxPlayers
	new sMsg[MAX_UDP_LENGTH], sName[MAX_NAME_LENGTH], sIP[MAX_NAME_LENGTH]
	new sMap[MAX_NAME_LENGTH], sNextMap[MAX_NAME_LENGTH], sServer[MAX_NAME_LENGTH], sAddress[MAX_VAR_LENGTH], sServerTime[MAX_NAME_LENGTH]
	new iIndex, iPing, iLoss, iKills, iDeaths, iTeam, iUserTime, iAlive
	new sAuthid[MAX_NAME_LENGTH]
	
	// Get list of players
	get_players(iPlayers,iNumPlayers)
	
	// Send server info
	get_mapname(sMap,MAX_NAME_LENGTH)
	get_cvar_string("hostname",sServer,MAX_VAR_LENGTH)
	get_cvar_string("net_address",sAddress,MAX_NAME_LENGTH)
	get_cvar_string("amx_nextmap",sNextMap,MAX_NAME_LENGTH)

	if (!is_map_valid(sNextMap)) {
		copy(sNextMap,MAX_VAR_LENGTH,"[UNKNOWN]")
	}

	// Fix
	replace(sServer,MAX_VAR_LENGTH,sNextMap,"")
	
	get_time("%m/%d/%Y - %H:%M:%S",sServerTime,MAX_NAME_LENGTH)
	
	iMaxPlayers = get_maxplayers()
	if (iMaxPlayers > get_cvar_num("sv_visiblemaxplayers") && get_cvar_num("sv_visiblemaxplayers") > 0) {
		iMaxPlayers = get_cvar_num("sv_visiblemaxplayers")
	}

	// Send package header
	udp_send_msg("LIVEFEED^"BEGIN")
	
	// Format message
	// LIVEFEED"SERVER"name"address"numplayers"maxplayers"map"maptime"timeleft"connects"tscore"ctscore"server_time
	format(sMsg,MAX_UDP_LENGTH,"LIVEFEED^"SERVER^"%s^"%s^"%i^"%i^"%s^"%i^"%i^"%i^"%i^"%i^"%s^"%s",
	sServer,sAddress,iNumPlayers,iMaxPlayers,sMap,get_cvar_num("mp_timelimit"),(get_timeleft()/60),gConnect,gTeamScore[0],gTeamScore[1],sServerTime,sNextMap)
	udp_send_msg(sMsg)

	// Send status of every player
	for (new i = 0; i < iNumPlayers; i++) {
		// Get data
		iIndex = iPlayers[i]
		get_user_name(iIndex,sName,MAX_NAME_LENGTH)
		if (is_user_bot(iIndex)) {
			format(sAuthid,MAX_NAME_LENGTH,"%s","BOT")
		}
		else {
			get_user_authid(iIndex,sAuthid,MAX_NAME_LENGTH)
		}
		iKills = get_user_frags(iIndex)
		iDeaths = get_user_deaths(iIndex)
		iTeam = get_team_num(iIndex)
		iAlive = is_user_alive(iIndex)
		iUserTime = get_user_time(iIndex,1)
		get_user_ip(iIndex,sIP,MAX_NAME_LENGTH,1)
		get_user_ping(iIndex,iPing,iLoss)
	
		// Format message and send
		// LIVEFEED"PLAYER"index"name"authid"kills"deaths"team"alive"ping"playingtime"ip
		format(sMsg,MAX_UDP_LENGTH,"LIVEFEED^"PLAYER^"%i^"%s^"%s^"%i^"%i^"%i^"%i^"%i^"%i^"%s",
		iIndex,sName,sAuthid,iKills,iDeaths,iTeam,iAlive,iPing,iUserTime,sIP)
		udp_send_msg(sMsg)
	}
	
	// Send list of cvars
	// Format message and send
	// CVAR"name"value
	for (new i = 0; i < MAX_CVARS; i++) {
		if (cvar_exists(gCvars[i])) {
			format(sMsg,MAX_UDP_LENGTH,"LIVEFEED^"CVAR^"%s^"%i",gCvars[i],get_cvar_num(gCvars[i]))
			udp_send_msg(sMsg)
		}
	}

	// Send mapcycle
	// MAPCYCLE"map"map"map
	format(sMsg,MAX_UDP_LENGTH,"LIVEFEED^"MAPCYCLE^"%s",gMapCycle)
	udp_send_msg(sMsg)
	
	// Send package footer
	udp_send_msg("LIVEFEED^"END")
	
	return PLUGIN_CONTINUE
}

public admin_livechat(id,level,cid)
{
	if (!cmd_access(id,level,cid,3)) return PLUGIN_HANDLED

	// Get parameters
	new sArg1[MAX_NAME_LENGTH]
	new sArg2[MAX_TEXT_LENGTH]
	read_argv(1,sArg1,MAX_NAME_LENGTH)
	read_argv(2,sArg2,MAX_TEXT_LENGTH)

	new iPlayer, iPlayers[MAX_PLAYERS], iNumPlayers
	get_players(iPlayers,iNumPlayers,"c")

	for (new i = 0; i < iNumPlayers; i++) {
		iPlayer = iPlayers[i]
		client_print(iPlayer,print_chat, "(WEB) %s :    %s",sArg1,sArg2)
	}
	console_print(id,"[UCStats] CHAT: %s: %s",sArg1,sArg2)

	// Make sure it comes back
	new sMsg[MAX_UDP_LENGTH]

	// Format and send
	// CHAT,say,player,team,message
	format(sMsg,MAX_UDP_LENGTH,"CHAT^"%s^"%s^"%i^"%s","WEB",sArg1,0,sArg2)
	udp_send_msg(sMsg)

	return PLUGIN_CONTINUE
}

public send_mapchange()
{
	new sMsg[MAX_UDP_LENGTH]
	
	// Format and send
	// EVENT,MAPCHANGE
	format(sMsg,MAX_UDP_LENGTH,"EVENT^"MAPCHANGE")
	udp_send_msg(sMsg)
}


/************************************************************
* EVENT FUNCTIONS
************************************************************/

public event_death() {
	new iKiller = read_data(1)
	new iVictim = read_data(2)

	new sMsg[MAX_UDP_LENGTH]
	new sKiller[MAX_NAME_LENGTH], sVictim[MAX_NAME_LENGTH], sWeapon[MAX_NAME_LENGTH]
	new sKillerTeam[MAX_NAME_LENGTH], sVictimTeam[MAX_NAME_LENGTH]
	new iKillerWeapon, iWeapon, iBody, iHeadshot

	if (gMod == MOD_CSTRIKE) {
		iHeadshot = read_data(3)

		get_user_attacker(iVictim,iWeapon,iBody)

		get_weaponname(iWeapon,sWeapon,MAX_NAME_LENGTH)
		get_user_team(iKiller,sKillerTeam,MAX_NAME_LENGTH)
		get_user_team(iVictim,sVictimTeam,MAX_NAME_LENGTH)

	} else if (gMod == MOD_DOD) {
		iKillerWeapon = get_dod_weapon(iKiller,read_data(3))

		format(sWeapon,MAX_NAME_LENGTH,"%s",gDodWeapons[iKillerWeapon])
		get_user_info(iKiller,"team",sKillerTeam,MAX_NAME_LENGTH)
		get_user_info(iVictim,"team",sVictimTeam,MAX_NAME_LENGTH)
		get_user_attacker(iVictim,iWeapon,iBody)
		if (iBody == INDEX_HEAD) iHeadshot = 1
	}

	get_user_name(iKiller,sKiller,MAX_NAME_LENGTH)
	get_user_name(iVictim,sVictim,MAX_NAME_LENGTH)
	
	// Format and send
	// EVENT,KILL,killer,team,victim,team,weapon,headshot
	format(sMsg,MAX_UDP_LENGTH,"EVENT^"KILL^"%s^"%s^"%s^"%s^"%s^"%i",
	sKiller,sKillerTeam,sVictim,sVictimTeam,sWeapon,iHeadshot)
	udp_send_msg(sMsg)
	
	return PLUGIN_CONTINUE
}

public event_world_action()
{
	new sMsg[MAX_UDP_LENGTH]
	new sAction[MAX_NAME_LENGTH]
	read_logargv(1,sAction,MAX_VAR_LENGTH)	
	
	// Format and send
	// EVENT,WORLD,event
	format(sMsg,MAX_UDP_LENGTH,"EVENT^"WORLD^"%s",sAction)
	udp_send_msg(sMsg)
	
	return PLUGIN_CONTINUE
}

public event_player_action()
{
	new iUserId
	new sArg[MAX_VAR_LENGTH]
	new sName[MAX_NAME_LENGTH]
	new sAction[MAX_NAME_LENGTH]
	
	read_logargv(0,sArg,MAX_VAR_LENGTH)
	read_logargv(2,sAction,MAX_VAR_LENGTH)
	parse_loguser(sArg,sName,MAX_NAME_LENGTH,iUserId)

	remove_quotes(sAction)

	new sMsg[MAX_UDP_LENGTH]
	new sAuthid[MAX_NAME_LENGTH]
	
	if (is_user_bot(iUserId)) {
		format(sAuthid,MAX_NAME_LENGTH,"%s","BOT")
	}
	else {
		get_user_authid(iUserId,sAuthid,MAX_NAME_LENGTH)
	}
	get_team_name(iUserId)
	
	// Format and send
	// EVENT,ACTION,name,team,action
	format(sMsg,MAX_UDP_LENGTH,"EVENT^"ACTION^"%s^"%s^"%s",sName,gTeam,sAction)
	udp_send_msg(sMsg)
	
	return PLUGIN_CONTINUE
}

public event_say(id)
{
	new sMsg[MAX_UDP_LENGTH]
	new sName[MAX_NAME_LENGTH]
	new sCommand[MAX_NAME_LENGTH]
	new sMessage[MAX_TEXT_LENGTH]
	new iTeam

	read_argv(0,sCommand,MAX_TEXT_LENGTH)
	read_argv(1,sMessage,MAX_TEXT_LENGTH)
	get_user_name(id,sName,MAX_NAME_LENGTH)
	iTeam = get_team_num(id)

	// Format and send
	// CHAT,say,name,team,message
	format(sMsg,MAX_UDP_LENGTH,"CHAT^"%s^"%s^"%i^"%s",sCommand,sName,iTeam,sMessage)
	udp_send_msg(sMsg)
	
	return PLUGIN_CONTINUE
}

public event_team_score()
{
        if (gMod == MOD_CSTRIKE) {
                new sTeam[MAX_NAME_LENGTH]
                read_data(1,sTeam,MAX_NAME_LENGTH)
                if (equal(sTeam,"CT")) {
                        gTeamScore[0] = read_data(2)
                }
                else {
                        gTeamScore[1] = read_data(2)
                }
        }
        else if (gMod == MOD_DOD) {
		gTeamScore[read_data(1)-1] = read_data(2)
        }

        return PLUGIN_CONTINUE
}

public event_class_change()
{
	new sMsg[MAX_UDP_LENGTH]
        new sLog[MAX_NAME_LENGTH]
        new sClass[MAX_NAME_LENGTH]
        new sName[MAX_NAME_LENGTH]
        new sAuthid[MAX_NAME_LENGTH]
        new sTeam[MAX_NAME_LENGTH]
        new iUserId

        read_logargv(0,sLog,MAX_NAME_LENGTH)
        parse_loguser(sLog,sName,MAX_NAME_LENGTH,iUserId,sAuthid,MAX_NAME_LENGTH,sTeam,MAX_NAME_LENGTH)
        read_logargv(2,sClass,MAX_NAME_LENGTH)

        new iPlayer = find_player("c",sAuthid)
        format(gPlayerClass[iPlayer],MAX_NAME_LENGTH,"%s",sClass)

	// Format and send
	// EVENT,CLASS,name,class
	format(sMsg,MAX_UDP_LENGTH,"EVENT^"CLASS^"%s^"%s",sName,sClass)

        return PLUGIN_CONTINUE
}


/************************************************************
* HELPER FUNCTIONS
************************************************************/

public get_mapcycle()
{
	new iLen, iIndex = 0
	new sFilename[MAX_VAR_LENGTH], sBuffer[MAX_VAR_LENGTH]
	get_cvar_string("mapcyclefile",sFilename,MAX_VAR_LENGTH)
	if (file_exists(sFilename)) {
		while(read_file(sFilename,iIndex++,sBuffer,MAX_VAR_LENGTH,iLen)) {
			if (is_alpha(sBuffer[0]) && is_map_valid(sBuffer)) {
				add(gMapCycle,MAX_TEXT_LENGTH,sBuffer,MAX_VAR_LENGTH)
				add(gMapCycle,MAX_TEXT_LENGTH,"^"",1)
			}
		}
	}
}

bool:is_alpha(c)
{
	return ((c>='a'&&c<='z')||(c>='A'&&c<='Z')) ? true : false
}

get_dod_weapon(player,weapon)
{
        //client_print(player,print_chat,"Check: %s (%s)",gDodWeapons[weapon],gPlayerClass[player]);
        if (weapon == DOD_FG42 || weapon == DOD_SCOPED_FG42) {
                if (equal(gPlayerClass[player],"#class_axispara_fg42scope")) {
                        return DOD_SCOPED_FG42
                }
                else {
                        return DOD_FG42
                }
        }
        else if (weapon == DOD_ENFIELD || weapon == DOD_SCOPED_ENFIELD) {
                if (equal(gPlayerClass[player],"#class_brit_sniper")) {
                        return DOD_SCOPED_ENFIELD
                }
                else {
                        return DOD_ENFIELD
                }
        }
        return weapon
}

get_team_name(player) {
	if (gMod == MOD_CSTRIKE) {
		get_user_team(player,gTeam,MAX_NAME_LENGTH)	
	}
	else if (gMod == MOD_DOD) {
	        get_user_info(player,"team",gTeam,MAX_NAME_LENGTH)
	        if (gTeam[0] == 'a') {
			gTeam[0] = 'A'
		}
	        else if (gTeam[0] == 's') {
			gTeam[0] = 'S'
		}
	}
}

get_team_num(player) {
        if (gMod == MOD_CSTRIKE) {
                return get_user_team(player)
        }
        else if (gMod == MOD_DOD) {
                get_user_info(player,"team",gTeam,MAX_NAME_LENGTH)
                if (gTeam[1] == 'x') {
                        return 1;
                }
                else if (gTeam[1] == 'l') {
                        return 2;
                }
		else {
			return 3;
		}
        }
	return 0;
}



/************************************************************
* PLUGIN FUNCTIONS
************************************************************/

public plugin_end()
{
	send_mapchange()
	return PLUGIN_CONTINUE
}

public client_putinserver(id)
{
	new sMsg[MAX_UDP_LENGTH]
	new sName[MAX_NAME_LENGTH]
	new sIP[MAX_NAME_LENGTH]
	new sAuthid[MAX_TEXT_LENGTH]
	new sMap[MAX_NAME_LENGTH], iNumPlayers, iMaxPlayers

	get_user_name(id,sName,MAX_NAME_LENGTH)
	if (is_user_bot(id)) {
		format(sAuthid,MAX_NAME_LENGTH,"%s","BOT")
	}
	else {
		get_user_authid(id,sAuthid,MAX_NAME_LENGTH)
	}
	get_user_ip(id,sIP,MAX_NAME_LENGTH,1)
	
	// Format and send
	// EVENT,JOINED,player,id,ip
	format(sMsg,MAX_UDP_LENGTH,"EVENT^"JOINED^"%s^"%s^"%s",sName,sAuthid,sIP)
	udp_send_msg(sMsg)
	
	get_mapname(sMap,MAX_NAME_LENGTH)
	iNumPlayers = get_playersnum()
	iMaxPlayers = get_maxplayers()
	if (iMaxPlayers > get_cvar_num("sv_visiblemaxplayers") && get_cvar_num("sv_visiblemaxplayers") > 0) {
		iMaxPlayers = get_cvar_num("sv_visiblemaxplayers")
	}
	
	// EVENT,MAPSTATUS,map,numplayers,maxplayers
	format(sMsg,MAX_UDP_LENGTH,"EVENT^"MAPSTATUS^"%s^"%i^"%i",sMap,iNumPlayers,iMaxPlayers)
	udp_send_msg(sMsg)
	
	return PLUGIN_CONTINUE
}

public client_connect(id)
{
	if (!is_user_bot(id)) {
		gConnect += 1
	}

	return PLUGIN_CONTINUE
}

public client_disconnect(id)
{
	new sMsg[MAX_UDP_LENGTH]
	new sName[MAX_NAME_LENGTH]
	new sIP[MAX_NAME_LENGTH]
	new sAuthid[MAX_TEXT_LENGTH]
	new sMap[MAX_NAME_LENGTH], iNumPlayers, iMaxPlayers

	get_user_name(id,sName,MAX_NAME_LENGTH)
	if (is_user_bot(id)) {
		format(sAuthid,MAX_NAME_LENGTH,"%s","BOT")
	}
	else {
		get_user_authid(id,sAuthid,MAX_NAME_LENGTH)
	}
	get_user_ip(id,sIP,MAX_NAME_LENGTH,1)
	
	// Format and send
	// EVENT,JOINED,player,id,ip
	format(sMsg,MAX_UDP_LENGTH,"EVENT^"LEFT^"%s^"%s^"%s",sName,sAuthid,sIP)
	udp_send_msg(sMsg)
	
	get_mapname(sMap,MAX_NAME_LENGTH)
	iNumPlayers = get_playersnum()-1
	iMaxPlayers = get_maxplayers()
	if (iMaxPlayers > get_cvar_num("sv_visiblemaxplayers") && get_cvar_num("sv_visiblemaxplayers") > 0) {
		iMaxPlayers = get_cvar_num("sv_visiblemaxplayers")
	}
	
	// EVENT,MAPSTATUS,map,numplayers,maxplayers
	format(sMsg,MAX_UDP_LENGTH,"EVENT^"MAPSTATUS^"%s^"%i^"%i",sMap,iNumPlayers,iMaxPlayers)
	udp_send_msg(sMsg)

	return PLUGIN_CONTINUE
}

public client_infochanged(id)
{
	new sMsg[MAX_UDP_LENGTH]
	new sName[MAX_TEXT_LENGTH], sNewName[MAX_TEXT_LENGTH]

	get_user_name(id,sName,MAX_TEXT_LENGTH)
	get_user_info(id,"name",sNewName,MAX_TEXT_LENGTH)

	if (!equal(sName,sNewName) && !equal(sName,"") && !equal(sName,"(null)")) {
		// Format and send
		// EVENT,NAMECHANGE,name,newname
		format(sMsg,MAX_UDP_LENGTH,"EVENT^"NAMECHANGE^"%s^"%s",sName,sNewName)
		udp_send_msg(sMsg)
	}
	return PLUGIN_CONTINUE
}

public plugin_init() {
	register_plugin("UCStats LIVE","1.1","mike_cao")
	register_srvcmd("amx_livefeed","admin_livefeed",ACCESS_LEVEL,"amx_livefeed")
	register_srvcmd("amx_livechat","admin_livechat",ACCESS_LEVEL,"amx_livechat <name> <message>")

        // Mod specific events
        new sMod[MAX_NAME_LENGTH]
        get_modname(sMod,MAX_NAME_LENGTH)

        if (equal(sMod,"cstrike")) {
                gMod = MOD_CSTRIKE
		register_logevent("event_world_action",2,"0=World triggered")
	}
        else if (equal(sMod,"dod")) {
                gMod = MOD_DOD
		register_logevent("event_class_change",3,"1=changed role to")
		register_logevent("event_player_action",5,"1=triggered a")
		register_logevent("event_player_action",6,"1=triggered a")
		register_logevent("event_player_action",4,"1=triggered a")
	}

	// Global events
	register_event("DeathMsg","event_death","a")
	register_event("TeamScore","event_team_score","a")
	register_logevent("event_player_action",3,"1=triggered")

	// Cvars
	register_cvar("ucstats_live_ip","")
	register_cvar("ucstats_live_port","")
	register_cvar("ucstats_live_fw","")

	// Init
	set_task(1.0,"send_mapchange")	
	set_task(2.0,"get_mapcycle")
	
	// Register say
	register_clcmd("say","event_say")
	register_clcmd("say_team","event_say")

	get_cvar_string("net_address",gNetAddress,MAX_NAME_LENGTH)
	if (strlen(gNetAddress) == 0) {
		format(gNetAddress,MAX_NAME_LENGTH,"%s","127.0.0.1");
	}
	
	// Initialize UDP session
	if (get_udp_status()) {
		server_print("[UCStats] LIVE: UDP initialized successfully.")
	}
	else {
		server_print("[UCStats] LIVE: Failed to initialize UDP.")
	}
	
	return PLUGIN_CONTINUE
}
