/*
*	AMXMODX script.
*	(ucstats_cstrike.sma)
*	by mike_cao <mike@mikecao.com>
*	This file is provided as is (no warranties).
*
*/

#include <amxmodx>
#include <ucstats>

// Globals
new gMod = 0
new gConnect = 0
new gTotalKills = 0
new gTotalActions = 0
new gTotalConnect = 0
new gTeamScore[2]
new gLastPlayers = 0 

// Admin access level
#define ACCESS_LEVEL ADMIN_LEVEL_A


/************************************************************
* LOG FUNCTIONS
************************************************************/
/*
log_trigger(id,trigger[])
{
	new iUserid = get_user_userid(id)
	new sTeam[MAX_NAME_LENGTH]
	new sName[MAX_NAME_LENGTH]
	new sAuthid[MAX_NAME_LENGTH]

	get_user_name(id,sName,MAX_NAME_LENGTH)
	get_user_authid(id,sAuthid,MAX_NAME_LENGTH)

	if (gMod == MOD_CSTRIKE) {
		get_user_team(id,sTeam,MAX_NAME_LENGTH)
	} else if (gMod == MOD_DOD) {
		get_user_info(id,"team",sTeam,MAX_NAME_LENGTH)
	}

	log_message("^"%s<%d><%s><%s>^" triggered ^"%s^"",sName,iUserid,sAuthid,sTeam,trigger)
}
*/
log_disconnect(id)
{
	new sTeam[MAX_NAME_LENGTH]
	new sName[MAX_NAME_LENGTH]
	new sAuthid[MAX_NAME_LENGTH]
	new iUserid
	
	iUserid = get_user_userid(id)
	get_user_name(id,sName,MAX_NAME_LENGTH)
	get_user_authid(id,sAuthid,MAX_NAME_LENGTH)

        if (gMod == MOD_CSTRIKE) {
                get_user_team(id,sTeam,MAX_NAME_LENGTH)
        } else if (gMod == MOD_DOD) {
                get_user_info(id,"team",sTeam,MAX_NAME_LENGTH)
        }
	
	log_message("^"%s<%d><%s><%s>^" disconnected",sName,iUserid,sAuthid,sTeam)
}

log_map_stats()
{
	new sMap[MAX_NAME_LENGTH]
	get_mapname(sMap,MAX_NAME_LENGTH)

	// "Map_Stats" (map "kills,actions,teamscore1,teamscore2,map_time,connects")
	log_message("Server triggered ^"Map_Stats^" (%s ^"%i,%i,%i,%i,%i,%i^")",
		sMap,gTotalKills,gTotalActions,gTeamScore[0],gTeamScore[1],floatround(get_gametime()),gTotalConnect)
}

public log_server_stats()
{
	new sMap[MAX_NAME_LENGTH]
	get_mapname(sMap,MAX_NAME_LENGTH)

	// "Server_Stats" ("map,num_players,connections")
	log_message("Server triggered ^"Server_Stats^" (^"%s,%i,%i^")",sMap,get_playersnum(),gConnect)

	gConnect = 0
}

log_num_players()
{
	new iNumPlayers = get_playersnum()
	if (iNumPlayers != gLastPlayers) {
		log_message("Server has ^"%i^" players",iNumPlayers)
	}
	gLastPlayers = iNumPlayers
}

/************************************************************
* EVENT FUNCTIONS
************************************************************/

public event_say(id)
{
	new sName[MAX_NAME_LENGTH]
	new sAuthid[MAX_NAME_LENGTH]
	new sHost[MAX_TEXT_LENGTH]
	new sUrl[MAX_TEXT_LENGTH]
	new sMod[MAX_NAME_LENGTH]
	new sArg[MAX_NAME_LENGTH]
	read_argv(1,sArg,MAX_NAME_LENGTH)

	// Check for @ char
	if (sArg[0] == '@') {
		get_modname(sMod,MAX_NAME_LENGTH)
		get_cvar_string("ucstats_url",sHost,MAX_NAME_LENGTH)
        	get_user_name(id,sName,MAX_NAME_LENGTH)
	        get_user_authid(id,sAuthid,MAX_NAME_LENGTH)

		if ( containi(sArg,"@stats") != -1 ) {
			format(sUrl,MAX_TEXT_LENGTH,"%s?game=%s&q=player&name=%s&id=%s",sHost,sMod,sName,sAuthid)
			show_motd(id,sUrl,"UCStats: Player Stats")
		}
		else if ( containi(sArg,"@top10") != -1 ) {
			format(sUrl,MAX_TEXT_LENGTH,"%s?game=%s&q=top&name=%s&id=%s",sHost,sMod,sName,sAuthid)
			show_motd(id,sUrl,"UCStats: Top 10 Players")
		}
		else if ( containi(sArg,"@resetme") != -1 ) {
			client_print(id,print_chat,"* Your stats have been reset.")
		}
		else if ( containi(sArg,"@password") != -1 ) {
			client_print(id,print_chat,"* Your profile password has been set.")
		}
		else if ( containi(sArg,"@help") != -1 ) {
			client_print(id,print_chat,"* See the console for a list of ucstats commands.")
			client_print(id,print_console,"UCStats Commands:");
			client_print(id,print_console,"============================================");
			client_print(id,print_console,"@stats - Displays your current stats.");
			client_print(id,print_console,"@top10 - Displays a list of the top 10 players.");
			client_print(id,print_console,"@resetme - Resets your stats.");
			client_print(id,print_console,"@password <pass> - Sets a password so you can edit your profile.");
			client_print(id,print_console,"@help - Displays this help text.");
			client_print(id,print_console,"============================================");
		}
		else {
			client_print(id,print_chat,"* Invalid command. Say @help for more details.")
		}
		return PLUGIN_HANDLED
	}
	return PLUGIN_CONTINUE
}

public event_death()
{
	gTotalKills += 1
	return PLUGIN_CONTINUE
}

public event_team_score()
{
	if (gMod == MOD_CSTRIKE) {
		new sTeam[MAX_NAME_LENGTH]
        	read_data(1,sTeam,MAX_NAME_LENGTH)
	        if (equal(sTeam,"CT")) {
        	        gTeamScore[0] = read_data(2)
	        }
        	else {
	                gTeamScore[1] = read_data(2)
        	}
	}
	else if (gMod == MOD_DOD) {
		if (read_data(1) == 1){
			gTeamScore[0] = read_data(2)
		}
		else if (read_data(1) == 2) {
			gTeamScore[1] = read_data(2)
		}
	}

	return PLUGIN_CONTINUE
}

public event_map_change()
{
	new iPlayers[MAX_PLAYERS], iNumPlayers
	get_players(iPlayers,iNumPlayers)
	for (new i = 0; i < iNumPlayers; i++) {
		log_disconnect(iPlayers[i])
	}

	return PLUGIN_CONTINUE
}

public event_player_action()
{
	gTotalActions += 1
	return PLUGIN_CONTINUE
}

/************************************************************
* PLUGIN FUNCTIONS
************************************************************/

public plugin_end()
{
	log_server_stats()
	log_map_stats()

	return PLUGIN_CONTINUE
}

public client_connect(id)
{
	if (!is_user_bot(id)) {
		gConnect += 1
		gTotalConnect += 1
	}
	log_num_players()

	return PLUGIN_CONTINUE
}

public client_disconnect(id)
{
	log_num_players()
	return PLUGIN_CONTINUE
}

public plugin_init() {
	register_plugin("UCStats BASE","1.1","mike_cao")

	// Mod specific events
	new sMod[MAX_NAME_LENGTH]
	get_modname(sMod,MAX_NAME_LENGTH)

	if (equal(sMod,"cstrike")) {
		gMod = MOD_CSTRIKE
		register_logevent("event_player_action",6,"2=triggered","3=Target_Bombed")
		register_logevent("event_player_action",3,"1=triggered")
	}
	else if (equal(sMod,"dod")) {
		gMod = MOD_DOD
		register_event("ObjScore","event_player_action","a")
	}

	// General events
	register_event("DeathMsg","event_death","a")
	register_event("30","event_map_change","a")
	register_event("TeamScore","event_team_score","a")

	register_clcmd("say","event_say",0,"")
	
	// Log server stats
	set_task(300.0,"log_server_stats",0,"",0,"b")
	
	server_print("[UCStats] BASE: Logging initialized successfully.")
	
	return PLUGIN_CONTINUE
}
