<?php
/*****************************************************************
* UCStats - A stats generation program for Half-Life games.
* Copyright (c) 2003 Mike Cao <mike@mikecao.com>
*****************************************************************/

/*
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, 
MA 02111-1307, USA.
*/

/*****************************************************************
* Stats Daemon
*****************************************************************/

set_time_limit(0);

define("__SCRIPT__", true);

ini_set('output_buffering','0');

// Includes
include('config/config.php');
include(PROGRAM_PATH.'/includes/proc_load_base.php');
include(PROGRAM_PATH.'/includes/proc_db_open.php');
include(PROGRAM_PATH.'/includes/proc_load_configs.php');
include(PROGRAM_PATH.'/includes/class_parser.php');
include(PROGRAM_PATH.'/includes/class_query.php');

// Initialize
global $_SERVER;

if (!isset($_SERVER['argc']) || !isset($_SERVER['argv'])) {
	echo L_ARGC_ARGV_MSG."\n";
	exit;
}

$argc = $_SERVER['argc'];
$argv = $_SERVER['argv'];

$OPTIONS = array(
	'v' => false,
	'i' => false,
	'w' => false
);

$INTERVAL = 10; // default
$LOCAL_IP = null;
$LOCAL_PORT = null;

// Errors
function quit($msg) {
	echo "$msg\n";
	echo L_STATSD_HELP;
	exit;
}

// Check arguments
if ($argc < 2) {
	quit(L_STATSD_MSG);
}
for ($a = 0; $a < $argc; $a++) {
	$arg = $argv[$a];
	if ($arg{0} == '-') {
		for ($i = 1; $i < strlen($arg); $i++) {
			if (isset($OPTIONS[$arg{$i}])) {
				$OPTIONS[$arg{$i}] = true;
			}
			else {
				quit(get_token_string(L_ERR_INVALID_OPTION,array($arg{$i}))."\n");
			}
			// If 'i' flag found, next param should be a number
			if ($arg{$i} == 'i' && $a+1 <= $argc-3) {
				if (is_numeric($argv[$a+1]) && $argv[$a+1] >= 0) {
					$INTERVAL = $argv[$a+1];
				}
				else {
					quit(L_ERR_INTERVAL_OPTION."\n");
				}
			}
		}
	}
	else {
		if ($a == $argc-2) {
			$LOCAL_IP = $argv[$a];
		}
		else if ($a == $argc-1) {
			$LOCAL_PORT = $argv[$a];
		}
	}
}

// Last checks
if (!is_valid_address("$LOCAL_IP:$LOCAL_PORT")) {
	quit(L_ERR_INVALID_ADDR."\n");
}


/*****************************************************************
* Main
*****************************************************************/

echo get_token_string(L_SCRIPT_HEADER,array(PROGRAM_VERSION,date("Y")));

// Get list of registered servers
echo L_LOADING_SERVERS."\n";
include(PROGRAM_PATH.'/includes/proc_load_servers.php');

if (is_null($GAME_servers)) {
	echo L_ERR_LOAD_SERVERS."\n";
	exit;
}
else if (sizeof($GAME_servers) < 1) {
	echo L_ERR_NO_SERVERS."\n";
	exit;
}
else {
	foreach ($GAME_servers as $key => $value) {
		echo "- ".$GAME_servers[$key]['server_name']." (".$key.")\n";
	}
	echo get_token_string(L_FOUND_SERVERS,array(sizeof($GAME_servers)))."\n";
}

// Create receive socket
echo L_SOCKET_CREATE."\n";
$socket_recv = socket_create(AF_INET,SOCK_DGRAM,0)
or die(L_ERR_SOCKET_CREATE."\n");

// Bind receive socket
echo get_token_string(L_SOCKET_BIND,array($LOCAL_IP,$LOCAL_PORT))."\n";
$result = socket_bind($socket_recv,$LOCAL_IP,$LOCAL_PORT) 
or die(get_token_string(L_ERR_SOCKET_BIND,array($LOCAL_IP,$LOCAL_PORT))."\n");

// Socket ready
echo L_SOCKET_READY."\n";

// Create a PID file
task_create_pid('statsd',getmypid());

// Start up the log daemon
echo L_INIT_LOGD."\n";
if (file_exists(PROGRAM_PATH.'/run/logd.pid')) {
	unlink(PROGRAM_PATH.'/run/logd.pid');
}
if (strpos(strtolower(PHP_OS),'win') !== false) {
	exec("bgrun.exe php -q ./logd.php ".getmypid()." $INTERVAL >nul");
}
else {
	exec("php -q ./logd.php ".getmypid()." $INTERVAL > /dev/null &");
}

// Make sure the log daemon loaded
sleep(1);
if (file_exists(PROGRAM_PATH.'/run/logd.pid')) {
        echo get_token_string(L_DONE_LOGD, array(get_pid(PROGRAM_PATH.'/run/logd.pid')))."\n";
}
else {
        echo L_ERR_LOGD."\n";
        exit;
}

// Wait for data
while ($result = socket_recvfrom($socket_recv,$input,4096,0,$remote_ip,$remote_port) 
or die(L_ERR_SOCKET_RECV."\n")) {

	$sid = $remote_ip.':'.$remote_port;

	// Authorize data
	if (strlen($input) && isset($GAME_servers[$sid])) {
			$log = trim(strstr($input,'L'));

			if ($OPTIONS['v']) echo "RECV <$sid>: $log\n";

			// Filter and insert into the database
			if (strpos($log,'say') !== false ||
					strpos($log,'killed') !== false ||
					strpos($log,'triggered') !== false ||
					strpos($log,'connected') !== false ||
					strpos($log,'entered') !== false ||
					strpos($log,'validated') !== false ||
					strpos($log,'suicide') !== false ||
					strpos($log,'changed') !== false ||
					strpos($log,'Loading map') !== false ||
					strpos($log,'Server has') !== false ||
					strpos($log,'Log file') !== false)
			{
					db_new_log($sid,$log);
			}
	}
	else {
			echo get_token_string(L_SERVER_NOT_REG,array($sid))."\n";
	}

	// If logd process died, quit
	clearstatcache();
	if (filemtime(PROGRAM_PATH.'/run/logd.pid') < (time() - 600)) {
		log_error(L_ERR_STATSD_QUIT);
		break;
	}
}

// Close socket
echo L_SOCKET_CLOSE."\n";
socket_close($socket_recv);
echo L_DONE.".\n";

// Close database connection
include(PROGRAM_PATH.'/includes/proc_db_close.php');
