#
#	UCStats - A stats generation program for Half-Life games.
#	Copyright (c) 2003 Mike Cao <mike@mikecao.com>
#

DROP database ucstats;
CREATE database ucstats;
USE ucstats;

#
#	Players
#
CREATE TABLE ucstats_players(
	GAME varchar(8) NOT NULL,
	UID varchar(32) NOT NULL,
	name varchar(32) NOT NULL,
	id varchar(32) NOT NULL,
	clan varchar(32) DEFAULT '',
	score INT DEFAULT 0,
	kills INT DEFAULT 0,
	deaths INT DEFAULT 0,
	online INT DEFAULT 0,
	lastonline INT DEFAULT 0,
	ip varchar(15) NOT NULL,
	PRIMARY KEY(GAME, UID)
);

CREATE INDEX score_sort ON ucstats_players (score);
CREATE INDEX kills_sort ON ucstats_players (kills);
CREATE INDEX deaths_sort ON ucstats_players (deaths);
CREATE INDEX online_sort ON ucstats_players (online);
CREATE INDEX kpm_sort ON ucstats_players (kills,online);
CREATE INDEX kpd_sort ON ucstats_players (kills,deaths);

#
#	Weapon Stats
#
CREATE TABLE ucstats_weapons(
	UID varchar(32) NOT NULL,
	weapon_name varchar(32) NOT NULL,
	weapon_kills INT DEFAULT 0,
	weapon_deaths INT DEFAULT 0,
	weapon_headshots INT DEFAULT 0,
	weapon_teamkills INT DEFAULT 0,
	weapon_shots INT DEFAULT 0,
	weapon_hits INT DEFAULT 0,
	weapon_damage INT DEFAULT 0,
	PRIMARY KEY(UID, weapon_name)
);
CREATE INDEX weapon_rank ON ucstats_weapons (weapon_name,weapon_kills);
CREATE INDEX weapon_accuracy ON ucstats_weapons (weapon_name,weapon_shots,weapon_hits);
CREATE INDEX weapon_headshots ON ucstats_weapons (weapon_name,weapon_headshots);
CREATE INDEX weapon_harm ON ucstats_weapons (weapon_name,weapon_kills,weapon_damage);

#
#	Hit Stats
#
CREATE TABLE ucstats_hits(
	UID varchar(32) NOT NULL,
	weapon_name varchar(32) NOT NULL,
	hits_generic INT DEFAULT 0,
	hits_head INT DEFAULT 0,
	hits_chest INT DEFAULT 0,
	hits_stomach INT DEFAULT 0,
	hits_leftarm INT DEFAULT 0,
	hits_rightarm INT DEFAULT 0,
	hits_leftleg INT DEFAULT 0,
	hits_rightleg INT DEFAULT 0,
	PRIMARY KEY(UID, weapon_name)
);

#
#  Player Actions
#
CREATE TABLE ucstats_actions(
	UID varchar(32) NOT NULL,
	action_name varchar(32) NOT NULL,
	action_num INT DEFAULT 0,
	PRIMARY KEY(UID, action_name)
);
CREATE INDEX action_rank ON ucstats_actions (action_name,action_num);

#
#	Player Aliases
#
CREATE TABLE ucstats_alias(
	UID varchar(32) NOT NULL,
	alias_name varchar(32) NOT NULL,
	alias_num INT DEFAULT 0,
	alias_date INT DEFAULT 0,
	PRIMARY KEY(UID, alias_name)
);

#
#  Player Roles
#
CREATE TABLE ucstats_roles(
	UID varchar(32) NOT NULL,
	role_name varchar(32) NOT NULL,
	role_num INT DEFAULT 0,
	PRIMARY KEY(UID, role_name)
);

#
#  Kills History
#
CREATE TABLE ucstats_kills(
	UID1 varchar(32) NOT NULL,
	UID2 varchar(32) NOT NULL,
	kills1 INT DEFAULT 0,
	kills2 INT DEFAULT 0,
	PRIMARY KEY(UID1, UID2)
);

#
#  Player Stats
#
CREATE TABLE ucstats_playerstats(
	UID varchar(32) NOT NULL,
	map varchar(32) NOT NULL,
	kills INT DEFAULT 0,
	deaths INT DEFAULT 0,
	killstreak INT DEFAULT 0,
	rounds_survived INT DEFAULT 0,
	rounds_played INT DEFAULT 0,
	playing_time INT DEFAULT 0,
	PRIMARY KEY(UID, map)
);
CREATE INDEX killstreak_sort ON ucstats_playerstats (killstreak);
CREATE INDEX survivor_sort ON ucstats_playerstats (rounds_survived,rounds_played);

#
#  Score History
#
CREATE TABLE ucstats_score(
	UID varchar(32) NOT NULL PRIMARY KEY,
	score_history TEXT
);

#
#  Server Registry
#
CREATE TABLE ucstats_servers(
	GAME varchar(8) NOT NULL,
	SID varchar(32) NOT NULL,
	server_num INT NOT NULL,
	server_address varchar(32) NOT NULL,
	server_name varchar(255) NOT NULL,
	ps_enabled varchar(1) DEFAULT '1',
	ss_enabled varchar(1) DEFAULT '1',
	lf_enabled varchar(1) DEFAULT '1',
	PRIMARY KEY(GAME, SID, server_num)
);


#
#  Server History
#
CREATE TABLE ucstats_serverstats(
	SID varchar(32) NOT NULL,
	server_date INT NOT NULL,
	server_map varchar(32),
	server_slots INT(2) DEFAULT 0,
	server_connects INT(2) DEFAULT 0,
	PRIMARY KEY(SID, server_date)
);

#
#	Maps Stats
#
CREATE TABLE ucstats_maps(
	SID varchar(32) NOT NULL,
	map_name varchar(32) NOT NULL,
	map_kills INT DEFAULT 0,
	map_actions INT DEFAULT 0,
	map_wins_team1 INT DEFAULT 0,
	map_wins_team2 INT DEFAULT 0,
	map_time INT DEFAULT 0,
	map_connects INT DEFAULT 0,
	PRIMARY KEY(SID, map_name)
);

#
#  Player Awards
#
CREATE TABLE ucstats_awards(
	GAME varchar(8) NOT NULL,
	UID varchar(32) NOT NULL,
	name varchar(32) NOT NULL,
	id varchar(32) NOT NULL,
	clan varchar(32) DEFAULT NULL,
	award_name varchar(255) NOT NULL,
	award_month TINYINT NOT NULL,
	award_year SMALLINT NOT NULL,
	award_num INT DEFAULT 0,
	PRIMARY KEY(GAME, award_month, award_year, award_name)
);

#
#	Game Keys
#
CREATE TABLE ucstats_keys(
	key_owner varchar(32) NOT NULL,
	key_name varchar(32) NOT NULL,
	key_value varchar(255),
	PRIMARY KEY(key_owner, key_name)
);

#
#	Program Configs
#
CREATE TABLE ucstats_configs(
	config_owner varchar(32) NOT NULL,
	config_class varchar(32) NOT NULL,
	config_name varchar(32) NOT NULL,
	config_value varchar(32),
	config_default varchar(32),
	config_type varchar(1),
	config_desc varchar(255),
	config_sort int DEFAULT 0,
	PRIMARY KEY(config_class, config_owner, config_name)
);

#
#	User Accounts
#
CREATE TABLE ucstats_users(
	user_login varchar(32) NOT NULL PRIMARY KEY,
	user_password varchar(255) NOT NULL,
	user_access_level tinyint UNSIGNED DEFAULT 1,
	user_desc varchar(255)
);

#
#	Logs
#
CREATE TABLE ucstats_logs(
	log_num INT UNSIGNED AUTO_INCREMENT NOT NULL PRIMARY KEY,
	log_owner varchar(32),
	log_value varchar(255)
);

#
#	Setup
#
INSERT INTO ucstats_keys VALUES ('STATS', 'TIMESTAMP', 0);

INSERT INTO ucstats_configs VALUES ('STATS','CONFIG','CFG_CACHE','false','false','B','Enable web caching',0);
INSERT INTO ucstats_configs VALUES ('STATS','CONFIG','CFG_AWARDS','true','true','B','Enable monthly awards',0);
INSERT INTO ucstats_configs VALUES ('STATS','CONFIG','CFG_DB_CLEAN','true','true','B','Enable database cleanup',0);
INSERT INTO ucstats_configs VALUES ('STATS','CONFIG','CFG_DB_RESET','true','true','B','Enable database reset',0);
INSERT INTO ucstats_configs VALUES ('STATS','CONFIG','CFG_STATS_MODE','0','0','N','Stats parsing mode',0);
INSERT INTO ucstats_configs VALUES ('STATS','CONFIG','CFG_BOT_STATS','true','true','B','Enable bot stats',0);
INSERT INTO ucstats_configs VALUES ('STATS','CONFIG','CFG_SCORE_START','1000','1000','N','New player starting score',0);
INSERT INTO ucstats_configs VALUES ('STATS','CONFIG','CFG_SCORE_BASE','20','20','N','Base points',0);
INSERT INTO ucstats_configs VALUES ('STATS','CONFIG','CFG_SCORE_MAX','100','100','N','Maximum point gain',0);
INSERT INTO ucstats_configs VALUES ('STATS','CONFIG','CFG_SCORE_MIN','100','100','N','Maximum point loss',0);
INSERT INTO ucstats_configs VALUES ('STATS','CONFIG','CFG_DIFFICULTY','true','true','B','Enable difficulty',0);
INSERT INTO ucstats_configs VALUES ('STATS','CONFIG','CFG_DIFFICULTY_MAX','2','2','N','Maximum difficulty ratio',0);
INSERT INTO ucstats_configs VALUES ('STATS','CONFIG','CFG_SCORES_MAX','100','100','N','Maximum records to keep in score history',0);
INSERT INTO ucstats_configs VALUES ('STATS','CONFIG','CFG_PLAYERS_MIN','4','4','N','Minimum number of players required for stats parsing',0);
INSERT INTO ucstats_configs VALUES ('STATS','CONFIG','CFG_KILLS_MIN','20','20','N','Minimum kills for certain award categories',0);
INSERT INTO ucstats_configs VALUES ('STATS','CONFIG','CFG_WEB_TEMPLATE','default','default','T','Web template',0);
INSERT INTO ucstats_configs VALUES ('STATS','CONFIG','CFG_WEB_PLAYERS_MAX','100','100','N','Maximum players to display under player rankings',0);
INSERT INTO ucstats_configs VALUES ('STATS','CONFIG','CFG_WEB_RANK_MAX','10','10','N','Maximum players to show under weapon and misc rankings',0);
INSERT INTO ucstats_configs VALUES ('STATS','CONFIG','CFG_WEB_KILLS_MAX','20','20','N','Maximum players to show under kill history',0);
INSERT INTO ucstats_configs VALUES ('STATS','CONFIG','CFG_WEB_REVENGE_MAX','10','10','N','Maximum players to show under revenge list',0);
INSERT INTO ucstats_configs VALUES ('STATS','CONFIG','CFG_WEB_SCORES_MAX','20','20','N','Maximum records to show under score history',0);
INSERT INTO ucstats_configs VALUES ('STATS','CONFIG','CFG_WEB_AWARDS_MAX','5','5','N','Maximum records to show under awards',0);
INSERT INTO ucstats_configs VALUES ('STATS','CONFIG','CFG_LIVE_FEED','false','false','B','Live feed enabled',0);
INSERT INTO ucstats_configs VALUES ('STATS','CONFIG','CFG_LF_ADDRESS','','','T','Live feed receive address',0);
INSERT INTO ucstats_configs VALUES ('STATS','CONFIG','CFG_LF_PASSWORD','','','T','RCON password for live feed',0);
INSERT INTO ucstats_configs VALUES ('STATS','CONFIG','CFG_LF_DELAY','60','60','N','Live feed delay',0);
INSERT INTO ucstats_configs VALUES ('STATS','CONFIG','CFG_LF_EVENT_MAX','50','50','N','Maximum events to show in live feed',0);
INSERT INTO ucstats_configs VALUES ('STATS','CONFIG','CFG_USAGE_LOG','false','false','B','Enable usage logging',0);
INSERT INTO ucstats_configs VALUES ('STATS','CONFIG','CFG_ERROR_LOG','true','true','B','Enable error logging',0);
INSERT INTO ucstats_configs VALUES ('STATS','CONFIG','CFG_DEBUG','false','false','B','Enable debugging',0);
INSERT INTO ucstats_configs VALUES ('STATS','CONFIG','CFG_DEBUG_LOG','0','0','N','Debug logging mode',0);
INSERT INTO ucstats_configs VALUES ('STATS','CONFIG','CFG_LANGUAGE','english','english','T','Language',0);

INSERT INTO ucstats_users VALUES ('admin', '5f4dcc3b5aa765d61d8327deb882cf99', 0, 'Administrator Account');
